#ifndef ABSTRACT_SCENE_OBJ_H
#define ABSTRACT_SCENE_OBJ_H

#include "ColorSpec.h"
#include "BoundingBox.h"
#include "FogSpec.h"
//#include "AbstractMaterial.h"
#include "SimpleMaterialColoring.h"
#include "ObjectDisplayFlags.h"

#include "GLES/gl.h"

class AbstractMaterial;  // forward declaration
class MaterialColoring;
class SimpleMaterialColoring;

class AbstractSceneObject {
    public:
      AbstractSceneObject(ObjectDisplayFlags &odf);
      virtual ~AbstractSceneObject();
      virtual void renderObject() = 0;
      virtual void renderSimpleColoring(SimpleMaterialColoring* m) = 0;
      virtual void renderDoubleSidedColoring(MaterialColoring* m) =  0;
      void addMaterial(AbstractMaterial* m);
      void addEnclosingBox(BoundingBox* bb);
      void setName(char const* n);
      void setVertexColors(GLubyte* arrayOfColors);
      void setVisibility(bool v);
      bool isVisible();
      bool* getLightSettings();
      AbstractMaterial* getMaterial();
      FogSpec* getFogSpec();
      virtual bool showTexture();
      virtual bool hasFog();
      virtual bool showFog();
      virtual bool showWireframe();
      virtual bool useLightModelTwoSide();
      virtual void setShowTexture(bool st);
      virtual void setShowFog(bool sf);
      virtual void setShowWireframe(bool flag);
      virtual void setFogSpec(FogSpec* fspec);
      virtual void setLightModelTwoSide(bool v);
      virtual void toggleShowObject();
      virtual void toggleShowWireframe();
      virtual void toggleShowTexture();
      virtual void toggleShowFog();
      virtual void toggleLightModelTwoSide();
      virtual void toggleLightSource(int pos);
      const char* getName();
      
    protected:
      static SimpleMaterialColoring defaultMaterial;
      const char* objectName;
      BoundingBox boundingBox;
      AbstractMaterial* material;     
      GLfloat* vertices;
      GLubyte* vertexColors; 
      GLfloat* textureCoordinates;
      char* textureBuffer;
      ObjectDisplayFlags  &objectDisplayFlags;
      FogSpec* fogSpec;
};
#endif
