#ifndef LIGHT_SOURCE_PROPERTIES_H
#define LIGHT_SOURCE_PROPERTIES_H

#include "ColorSpec.h"
#include "SpacialPoint.h"
#include <glm/glm.hpp>

enum TypeOfLightSource { DIRECTIONAL, POSITIONAL, SPOT };
class LightSourceProperties {
	
	public: 
	   static GLenum LIGHT_ID[8];
	
	  LightSourceProperties();
	
	  TypeOfLightSource lightType;
	  bool isOn;
	  bool isFixed;
      ColorSpec ambientColor;
      ColorSpec diffuseColor;
      ColorSpec specularColor;
	  SpacialPoint direction;
	  // additional properties of positional light:
      float constantAttenuation;
      float linearAttenuation;
      float quadraticAttenuation;
      // additional properties of a spotlight
      SpacialPoint spotDirection;
      float exponent;
      float cutoff;
      
      
      void isFixedLight(bool v);
      bool isFixedLight();
      void installLight();
      void installTransformedLight(glm::mat4 &modelViewMatrix);
      ColorSpec* getPtrToAmbientColor();
      ColorSpec* getPtrToDiffuseColor();
      ColorSpec* getPtrToSpecularColor();
      
      
      void setLightID(GLenum id);
      void setConstantAttenuation(float v);
      void setLinearAttenuation(float v);
      void setQuadraticAttenuation(float v);
      
   // private:
      
      GLenum lightID;
      //GLfloat vec4[4];  //  müssen u.U. Instanzvariablen sein, siehe Erfahrung
                      //  mit Projektionsmatrizen
      //GLfloat vec3[3];
	  
};


#endif
