#ifndef LIGHTS_EDITOR_H
#define LIGHTS_EDITOR_H

#include <gtkmm/adjustment.h>
#include <gtkmm/frame.h>
#include <gtkmm/scale.h>
#include <gtkmm/box.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/frame.h>
#include "LightSourceProperties.h"
#include "ColorDisplayField.h"
#include "SceneHolder.h"
#include "SceneObserver.h"

class LightsEditor : public Gtk::Frame, public SceneObserver {
    public:
        LightsEditor(SceneHolder& sm);
        ~LightsEditor();
        virtual void onSceneChanged();
    private:
        SceneHolder& sceneModel;
        LightSourceProperties* currentlySelectedLight;
        ColorSpec* currentlySelectedColor;
        ColorDisplayField* currentColorDisplayField;
        bool suspendUpdates;
        float minDistanceFromLightSource;
        
        bool hasSelectedLight();
        bool hasSelectedColor();
        
        Gtk::RadioButton noLightSourceSelected;
        Gtk::RadioButton btn1;
        Gtk::RadioButton btn2;
        Gtk::RadioButton btn3;
        Gtk::RadioButton btn4;
        Gtk::RadioButton btn5;
        Gtk::RadioButton btn6;
        Gtk::RadioButton btn7;
        Gtk::RadioButton btn8;
        Gtk::CheckButton fixedPositionBttn;
        
        Gtk::RadioButton noSelectionBttn;
        Gtk::RadioButton ambientLightBttn;
        Gtk::RadioButton diffuseLightBttn;
        Gtk::RadioButton specularLightBttn;
        ColorDisplayField ambientLight;
        ColorDisplayField diffuseLight;
        ColorDisplayField specularLight;
        
        Glib::RefPtr<Gtk::Adjustment> red;
        Glib::RefPtr<Gtk::Adjustment> green;
        Glib::RefPtr<Gtk::Adjustment> blue;
        Glib::RefPtr<Gtk::Adjustment> constAttenuation;
        Glib::RefPtr<Gtk::Adjustment> linearAttenuation;
        Glib::RefPtr<Gtk::Adjustment> quadraticAttenuation;
        Gtk::Scale reglerR;
        Gtk::Scale reglerG;
        Gtk::Scale reglerB;
        Gtk::Scale reglerConstAttn;
        Gtk::Scale reglerLinAttn;
        Gtk::Scale reglerQuadAttn;
        
        Gtk::Frame labelledFrameAttenuation;
        Gtk::Box container;
        Gtk::Box lightSelectionBox;
        Gtk::Box lightTypeSelectionBox;
        Gtk::Box sliderBox;
        Gtk::Box positionBox;
        Gtk::Box spotlightPropertiesBox;

        void noLightSourceSelectedClicked();
        void lightSourceClicked(Gtk::RadioButton* bttn, int idx);
        void fixedPositionBttnClicked();
        
        void noSelectionBttnClicked();
        void ambientLightBttnClicked();
        void diffuseLightBttnClicked();
        void specularLightBttnClicked();
        
        void onRValueChanged();
        void onGValueChanged();
        void onBValueChanged();
        
        void onConstAttnChanged();
        void onLinAttnChanged();
        void onQuadAttnChanged();
        
        void enableButtons();
        void disableButtons();
        void enableSliders();
        void disableSliders();
        
        static const char* LiteralIsFixed;
        static const char* LiteralAmbient;
        static const char* LiteralDiffuse;
        static const char* LiteralSpecular;
        static const char* LiteralAttenuation;
};

#endif
