#ifndef SCENE_H
#define SCENE_H

#include "AbstractProjectionSpec.h"
#include "AbstractSceneObject.h"
#include "BoundingBox.h"
#include "CameraSettings.h"
#include "ColorSpec.h"
#include "LightSourceProperties.h"
#include "Sphere.h"
#include <list>
#include <glm/glm.hpp>

#include "GLES/gl.h"

class Scene {
    public:
      Scene(ColorSpec &cc, ColorSpec &al);
      ~Scene();
      static const int NUMBER_OF_LIGHTS = 8;
      ColorSpec getAmbientLightColor();
      ColorSpec& getAmbientLightColorRef();
      void addLightSources(LightSourceProperties* ls);
      ColorSpec getClearColor();
      ColorSpec& getClearColorRef();
      Sphere enclosingSphere();
      SpacialPoint getCenterPoint();
      void addSceneObject(AbstractSceneObject* obj);
      void addProjectionSpec(AbstractProjectionSpec* p);
      CameraSettings* getCameraSettings();
      std::list<AbstractSceneObject*>& getObjectList();
      LightSourceProperties* getRefToLight(int idx);
      
      void setTransformationMatrix(glm::mat4 *tm);
      glm::mat4 * getCurrentTransformationMatrix();
      
      void installProjection();
      void installStage(glm::mat4 &modelViewMatrix);
      void renderScene();
      
    protected:
      ColorSpec &clearColor;
      ColorSpec &ambientLightColor;
      std::list<AbstractSceneObject*> sceneObjects;
      LightSourceProperties *lightSources;
      AbstractProjectionSpec* projection;
      CameraSettings cameraSettings;
      glm::mat4 *transformationMatrix;

    private:
      BoundingBox enclosingBox();
};
#endif
