#ifndef SCENE_OBJECTS_EDITOR_H
#define SCENE_OBJECTS_EDITOR_H

#include "Configuration.h"
#include "SceneHolder.h"
#include "SceneObserver.h"
#include "NothingSelectedHint.h"
#include "MaterialPropertiesEditor.h"
#include "TexturePropertiesEditor.h"
#include "FogSpecEditor.h"
#include "ObjectSelectionObserver.h"


#include <gtkmm/frame.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/notebook.h>
#include <gtkmm/liststore.h>
#include <gtkmm/treemodelcolumn.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/box.h>
#include <gtkmm/treeview.h>
#include <list>

class SceneObjectsEditor : public Gtk::Frame, public SceneObserver {
    public:
        SceneObjectsEditor(SceneHolder& sm);
        ~SceneObjectsEditor();
        virtual void onSceneChanged();
        bool hasCurrentSelection();
        AbstractSceneObject* getCurrentlySelectedObject();
        void addObserver (ObjectSelectionObserver* observer);
        void removeObserver (ObjectSelectionObserver* observer);

    protected:
    
        void onLightToggled(const Glib::ustring& path, int pos);
        
       class ObjectList : public Gtk::TreeModel::ColumnRecord {
           public:
               ObjectList () {
                   add(stringColumn);
                   add(indexColumn);
                   add(showObject);
                   add(useLightModelTwoSide);
                   add(showWireframe);
                   add(showTexture);
                   add(showFog);
                   for (int i = 0; i < Scene::NUMBER_OF_LIGHTS; i++) {
					   add(useLight[i]);
				   }
               }
           
               Gtk::TreeModelColumn<Glib::ustring> stringColumn;
               Gtk::TreeModelColumn<int> indexColumn;
               Gtk::TreeModelColumn<bool> showObject;
               Gtk::TreeModelColumn<bool> useLightModelTwoSide;
               Gtk::TreeModelColumn<bool> showWireframe;
               Gtk::TreeModelColumn<bool> showTexture;
               Gtk::TreeModelColumn<bool> showFog;
               Gtk::TreeModelColumn<bool> useLight[Scene::NUMBER_OF_LIGHTS];
        };
        //  Instanzen der nachfolgenden Klasse stellen für die Callback-
        //  funktionen der Checkboxen die die Auswahlder Lichtquellen
        //  eine Aufrufumgebung zur Verfügung, in der die Nummer der
        //  Lichtquelle bekannt ist. 
        class Adapter {
			public:
			    Adapter() {
			    }
			private:
			    int positionIndex;
			    SceneObjectsEditor *receiver;
			public:
			    void onToggleClicked(const Glib::ustring& path) {
					receiver -> onLightToggled(path, positionIndex);
				}
				void setPositionIndex(int idx, SceneObjectsEditor*rc) {
					positionIndex = idx;
					receiver = rc;
				}
		};

    private:
        Adapter adapters[Scene::NUMBER_OF_LIGHTS]; 
        SceneHolder& sceneModel;
        std::list<ObjectSelectionObserver*> observer;  //  das ist schon aufwendig
        Gtk::TreeView sceneObjectsList;
        Gtk::ScrolledWindow scSceneObjectList;
        Gtk::RadioButton noSelectionBttn;


        Gtk::Box box3;
        Gtk::Box listContainer;
        Gtk::Box container;
        // In gtkmm 3.10 könnten Stack und StackSwitcher
        // verwendet werden, hier wird ein Notebook mit
        // einer Buttonleiste verwendet.
        Gtk::RadioButton selectMaterialEditor;
        Gtk::RadioButton selectTextureEditor;
        Gtk::RadioButton selectFogEditor;
        Gtk::Notebook editors;
        Gtk::Notebook materialEditors;
        NothingSelectedHint* emptyPage;
        TexturePropertiesEditor* textureEditor;
        MaterialPropertiesEditor* materialEditor;
        SimpleMaterialPropertiesEditor* simpleMaterialEditor;
        FogSpecEditor* fogEditor;
        Glib::RefPtr<Gtk::ListStore> refSceneObjects;
        ObjectList objectList;
        AbstractSceneObject* currentlySelectedSceneObject;
        void onSelectionChanged(); 

        void onMaterialButtonClicked();
        void onFogButtonClicked();
        void onShowObjectToggleClicked(const Glib::ustring& path);
        void onLightModelToggleClicked(const Glib::ustring& path);
        void onShowWireframeToggleClicked(const Glib::ustring& path);
        void onShowTextureToggleClicked(const Glib::ustring& path);
        void onShowFogToggleClicked(const Glib::ustring& path);
        void onTextureButtonClicked();
        
        void fillListstore();
        void enablePropertyEditors();
        void disablePropertyEditors();
        
        void notifyChange();

        
        static const char* LiteralShowObject;
        static const char* LiteralMaterial;
        static const char* LiteralTexture;
        static const char* LiteralFog;
        static const char* LiteralObjects;
        static const char* LiteralShow;
};

#endif
