#ifndef SIMPLE_MATERIAL_PROPERTIES_EDITOR_H
#define SIMPLE_MATERIAL_PROPERTIES_EDITOR_H

#include <gtkmm/adjustment.h>
#include <gtkmm/scale.h>
#include <gtkmm/box.h>
#include <gtkmm/frame.h>
#include <gtkmm/radiobutton.h>
#include "SceneHolder.h"
#include "ObjectSelectionObserver.h"
#include "ColorDisplayField.h"
#include "LogarithmicScaleTransformation.h"

class SceneObjectsEditor;

class SimpleMaterialPropertiesEditor : public Gtk::Frame,
                                 public ObjectSelectionObserver {

    public:
        static SimpleMaterialPropertiesEditor* create(SceneHolder& sm, SceneObjectsEditor& se);
        ~SimpleMaterialPropertiesEditor();
        void onObjectSelectionChanged();
        virtual void loadSceneObjectData();

    protected:        
        SimpleMaterialPropertiesEditor(SceneHolder& sm, SceneObjectsEditor& se);

        SceneHolder& sceneModel;
        SceneObjectsEditor& parentEditor;
        
        MaterialProperties* currentMaterialSpec;
        ColorSpec* currentlySelectedColor;
        ColorDisplayField* currentColorDisplayField;
        
        Glib::RefPtr<Gtk::Adjustment> red;
        Glib::RefPtr<Gtk::Adjustment> green;
        Glib::RefPtr<Gtk::Adjustment> blue;
        Glib::RefPtr<Gtk::Adjustment> shininess;
        Gtk::Scale reglerR;
        Gtk::Scale reglerG;
        Gtk::Scale reglerB;
        Gtk::Scale reglerShininess;
        Gtk::RadioButton noSelectionBttn;
        Gtk::RadioButton ambientColorBttn;
        Gtk::RadioButton diffuseColorBttn;
        Gtk::RadioButton specularColorBttn;
        Gtk::RadioButton emissiveColorBttn;
        ColorDisplayField ambientColor;
        ColorDisplayField diffuseColor;
        ColorDisplayField specularColor;
        ColorDisplayField emissiveColor;
        Gtk::Box container;
        Gtk::Box container1;
        Gtk::Box container2;
        bool suspendUpdates;
        
        LogarithmicScaleTransformation scaleTransformation;
        void onRValueChanged();
        void onGValueChanged();
        void onBValueChanged();
        void onShininessValueChanged();

        void noSelectionBttnClicked();
        void ambientColorBttnClicked();
        void diffuseColorBttnClicked();
        void specularColorBttnClicked();
        void emissiveColorBttnClicked();
        
        void configureComponents();
        void buildForm(); 
        
        void setSelectedColor(ColorSpec* c, ColorDisplayField* cdf);
        bool hasSelectedColor();
        bool hasSelectedMaterial();
        void updateSliderSettings();
        void updateVisualSettings();
        
        virtual void enableFirstSelection();
        virtual void disableFirstSelection();
        void enableSliders();
        void disableSliders();
        
        static const char* LiteralEmissive;
        static const char* LiteralAmbient;
        static const char* LiteralDiffuse;
        static const char* LiteralSpecular;
};

#endif
