#include "AbstractSceneObject.h"
//#include <iostream>

//
//  Die Farbwerte dieses Materials entsprechen den default Werten aus OpenGL 1.4
//  
SimpleMaterialColoring AbstractSceneObject::defaultMaterial(
                        MaterialProperties(ColorSpec(0.2f, 0.2f, 0.2f, 1.0f),
                                           ColorSpec(0.8f, 0.8f, 0.8f, 1.0f),
                                           ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                           ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                           0.0f));
                                    
AbstractSceneObject::AbstractSceneObject(ObjectDisplayFlags& odf)  :
  objectDisplayFlags(odf)
{
	textureCoordinates = (GLfloat*) 0;
	textureBuffer = (char*) 0;
    vertexColors = (GLubyte*) 0;
	fogSpec = (FogSpec*) 0;
}

AbstractSceneObject::~AbstractSceneObject() {
    //delete material;
    
    if (textureCoordinates != (GLfloat*) 0) {
		delete [] textureCoordinates;
		delete textureBuffer;
	}

    if (vertexColors != 0) {
		delete [] vertexColors;
	}
    /*
    if (fogSpec != (FogSpec*) 0) {
		delete fogSpec;
	    fogSpec = (FogSpec*) 0;
	}  */
}

/*
void AbstractSceneObject::renderObject() {
    //ObjectIsVisible = true;
}  */

void AbstractSceneObject::addEnclosingBox(BoundingBox* bb) {
    boundingBox.addBox(bb);
}

void AbstractSceneObject::setName(char const* n) {
    objectName = n;
}

char const* AbstractSceneObject::getName() {
    return objectName;
}

void AbstractSceneObject::setVisibility(bool v) {
	objectDisplayFlags.setVisibility(v);
}

bool AbstractSceneObject::isVisible() {
	return objectDisplayFlags.isVisible();
}

bool* AbstractSceneObject::getLightSettings() {
	return objectDisplayFlags.lights;
}

void AbstractSceneObject::setShowTexture(bool st) {
	objectDisplayFlags.setShowTexture(st);
}

void AbstractSceneObject::toggleShowObject() {
	objectDisplayFlags.toggleShowObject();
}

void AbstractSceneObject::toggleShowTexture() {
	objectDisplayFlags.toggleShowTexture();
}

void AbstractSceneObject::toggleShowFog() {
	objectDisplayFlags.toggleShowFog();
}

bool AbstractSceneObject::showTexture() {
	return objectDisplayFlags.showTexture();
}

bool AbstractSceneObject:: hasFog() {
	return fogSpec != (FogSpec*) 0;
}

void AbstractSceneObject::setShowFog(bool sf) {
	objectDisplayFlags.setShowFog(sf);
}

bool AbstractSceneObject::showFog() {
	return objectDisplayFlags.showFog();
}

bool AbstractSceneObject::showWireframe() {
	return objectDisplayFlags.showWireframe();
}

void AbstractSceneObject::toggleShowWireframe() {
	objectDisplayFlags.toggleShowWireframe();
}

bool AbstractSceneObject::useLightModelTwoSide() {
	return objectDisplayFlags.useLightModelTwoSide();
}

void AbstractSceneObject::setLightModelTwoSide(bool v) {
	objectDisplayFlags.setLightModelTwoSide(v);
}

void AbstractSceneObject::toggleLightModelTwoSide() {
	objectDisplayFlags.toggleLightModelTwoSide();
}

void AbstractSceneObject::setShowWireframe(bool flag) {
	objectDisplayFlags.setShowWireframe(flag);
}

void AbstractSceneObject::toggleLightSource(int i) {
	objectDisplayFlags.toggleLightSource(i);
}

/*
bool AbstractSceneObject::showObjectFlag() {
	return objectDisplayFlags.isVisible();
} */

FogSpec* AbstractSceneObject::getFogSpec() {
	return fogSpec;
} 

void AbstractSceneObject:: setFogSpec(FogSpec* fspec) {
	fogSpec = fspec;
}

void AbstractSceneObject::addMaterial(AbstractMaterial* m) {
    material = m;
}

void AbstractSceneObject::setVertexColors(GLubyte* array) {
	vertexColors = array;
}

AbstractMaterial* AbstractSceneObject::getMaterial() {
    if (material == (AbstractMaterial*) 0) {
        return &defaultMaterial;
    } else {
        return material;
    }
}
