#include "Example001.h"
#include "Example001Strings.h"
#include "Sphere.h"
#include "PerspectiveProjectionSpec.h"
#include <math.h>

//
//  zwei Helices
//
Example001::Example001() :
  AbstractExample(ColorSpec(0.3f, 0.3f, 0.3f),
                  ColorSpec(0.3f, 0.3f, 0.3f)),
  mObj1(MaterialColoring(
           MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                              ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                              ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                              ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                              90.0f),
           MaterialProperties(ColorSpec(0.1f, 0.2f, 0.1f, 1.0f),
                              ColorSpec(0.3f, 0.7f, 0.2f, 1.0f),
                              ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                              ColorSpec(0.1f, 0.3f, 0.0, 1.0f),
                              90.0f))),
  mObj2(MaterialColoring(
           MaterialProperties(ColorSpec(0.0f, 0.0f, 0.6f, 1.0f),
                              ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                              ColorSpec(0.4f, 0.5f, 0.0f, 1.0f),
                              ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                              90.0f),
           MaterialProperties(ColorSpec(0.3f, 0.3f, 0.1f, 1.0f),
                              ColorSpec(0.7f, 0.7f, 0.2f, 1.0f),
                              ColorSpec(0.5f, 0.6f, 0.4f, 1.0f),
                              ColorSpec(0.3f, 0.3f, 0.0, 1.0f),
                              90.0f))),
  fogSpec1(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0)),
  fogSpec2(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0))
{
	
    polygonSpec1.nx = 15;
    polygonSpec1.x0 = -7.0f/2.0f*M_PI;
    polygonSpec1.xStop = 7.0f/2.0f*M_PI;
    polygonSpec1.adaptiveGrid = false;
    polygonSpec2.nx = 15;
    polygonSpec2.x0 = -7.0f/2.0f*M_PI;
    polygonSpec2.xStop = 7.0f/2.0f*M_PI;
    polygonSpec2.adaptiveGrid = false;

    displayFlags1.setShowFog(true);
    displayFlags2.setShowFog(true);
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
    
    c1.setInst(this);
    c1.setF(refCFn(&Example001::fn1));
    c1.setd1F(refCFn(&Example001::fn1d1));
    c1.setd2F(refCFn(&Example001::fn1d2));
    
    
    c2.setInst(this);
    c2.setF(refCFn(&Example001::fn2));
    c2.setd1F(refCFn(&Example001::fn2d1));
    c2.setd2F(refCFn(&Example001::fn2d2));
    projectionSpec = new PerspectiveProjectionSpec();
    projectionSpec -> setZoomFactor(0.85f);
}

Scene* Example001::createScene() {
    
    Scene* scene = this ->setupScene(); // new Scene(clearColor, ambientLight);
    
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags1,
          &polygonSpec1, 1.0f, 
          &c1);

    SpacialSurfaceObject* obj2 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags2,
          &polygonSpec2, 1.0f, 
          &c2);

    obj1 -> addMaterial(&mObj1);
    obj2 -> addMaterial(&mObj2);
    
    obj1 -> setName (Example001::LiteralHelix1);
    obj2 -> setName (Example001::LiteralHelix2);
    scene -> addSceneObject(obj1);
    scene -> addSceneObject(obj2);
    
    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    SpacialPoint displacement(0.8*r, 0.8*r, 0.8*r);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    fogSpec2.setRegion(d1, d2);
    obj2 -> setFogSpec (&fogSpec2);
    return scene;
}


void Example001::fn1 (float t, SpacialPoint* ft) {
    ft -> x = 3.0f*cos(t);
    ft -> y = 3.0f*sin(t);
    ft -> z = 2.0f/3.0f*t;
}
void Example001::fn1d1 (float t, SpacialPoint* ft) {
    ft -> x = -3.0f*sin(t);
    ft -> y = 3.0f*cos(t);
    ft -> z =2.0f/3.0f;
}
void Example001::fn1d2 (float t, SpacialPoint* ft) {
    ft -> x = -3.0f*cos(t);
    ft -> y = -3.0f*sin(t);
    ft -> z = 0.0f;
}

void Example001::fn2 (float t, SpacialPoint* ft) {
    ft -> x = 6.0f*cos(-t);
    ft -> y = 6.0f*sin(-t);
    ft -> z = 2.0f/3.0f*t;
}
void Example001::fn2d1 (float t, SpacialPoint* ft) {
    ft -> x = 6.0f*sin(-t);
    ft -> y = -6.0f*cos(-t);
    ft -> z =2.0f/3.0f;
}
void Example001::fn2d2 (float t, SpacialPoint* ft) {
    ft -> x = -6.0f*cos(-t);
    ft -> y = -6.0f*sin(-t);
    ft -> z = 0.0f;
}
