#include "Example001a.h"
#include "Example001.h"
#include "Sphere.h"
#include "PerspectiveProjectionSpec.h"
#include <math.h>
#include <iostream> 
//
//  zwei Helices
//
Example001a::Example001a() :
  AbstractExample(ColorSpec(0.3f, 0.3f, 0.3f), ColorSpec(0.3f, 0.3f, 0.3f)),
  mObj1(MaterialColoring(
            MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                               ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                               ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                               ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                               90.0f),
            MaterialProperties(ColorSpec(0.1f, 0.2f, 0.1f, 1.0f),
                               ColorSpec(0.3f, 0.7f, 0.2f, 1.0f),
                               ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                               ColorSpec(0.1f, 0.3f, 0.0, 1.0f),
                               90.0f))),
  mObj2(MaterialColoring(
            MaterialProperties(ColorSpec(0.0f, 0.0f, 0.6f, 1.0f),
                               ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                               ColorSpec(0.4f, 0.5f, 0.0f, 1.0f),
                               ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                               90.0f),
            MaterialProperties(ColorSpec(0.3f, 0.3f, 0.1f, 1.0f),
                               ColorSpec(0.7f, 0.7f, 0.2f, 1.0f),
                               ColorSpec(0.5f, 0.6f, 0.4f, 1.0f),
                               ColorSpec(0.3f, 0.3f, 0.0, 1.0f),
                               90.0f))),
  fogSpec1(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0)),
  fogSpec2(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0))
{
    polygonSpec1.nx = 15;
    polygonSpec1.x0 = -7.0f/2.0f*M_PI;
    polygonSpec1.xStop = 7.0f/2.0f*M_PI;
    polygonSpec1.adaptiveGrid = false;
    
    polygonSpec2.nx = 15;
    polygonSpec2.x0 = -7.0f/2.0f*M_PI;
    polygonSpec2.xStop = 7.0f/2.0f*M_PI;
    polygonSpec2.adaptiveGrid = false;

    displayFlags1.setShowFog(true);
    displayFlags2.setShowFog(true);
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
    
    c1.setInst(this);
    c1.setF(refCFn(&Example001a::fn1));
    c1.setd1F(refCFn(&Example001a::fn1d1));
    c1.setd2F(refCFn(&Example001a::fn1d2));
    
    
    c2.setInst(this);
    c2.setF(refCFn(&Example001a::fn2));
    c2.setd1F(refCFn(&Example001a::fn2d1));
    c2.setd2F(refCFn(&Example001a::fn2d2));
    
    projectionSpec = new PerspectiveProjectionSpec();
}

Scene* Example001a::createScene() {
    
    Scene* scene = this -> setupScene();
    
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags1,
          &polygonSpec1, 1.0f, 
          &c1);


    SpacialSurfaceObject* obj2 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags2,
          &polygonSpec2, 1.0f, 
          &c2);

    obj1 -> addMaterial(&mObj1);
    obj2 -> addMaterial(&mObj2);
    
    obj1 -> setName (Example001::LiteralHelix1);
    obj2 -> setName (Example001::LiteralHelix2);
    
    createVertexColoring (&polygonSpec1, obj1);
    createVertexColoring2 (&polygonSpec2, obj2);
    scene -> addSceneObject(obj1);
    scene -> addSceneObject(obj2);
    
    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(2*r2, 2*r2, 2*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
        
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    fogSpec2.setRegion(d1, d2);
    obj2 -> setFogSpec (&fogSpec2);
    return scene;
}

void Example001a::createVertexColoring(TubularGridSpec* gridSpec,
                                       SpacialSurfaceObject* obj) {
    int nx = gridSpec -> nx;
    int ny = gridSpec -> ny;
    int rgb[3];
    GLubyte* vertexColors = new GLubyte[4*nx*ny];
    GLubyte* ptr = vertexColors;
    for (int i = 0; i < ny; i++) {
		// berechne hue und konvertiere zu RGB
		float hue = (ny - 1 - i)*360.0/(ny);
		convertToRGB(hue, 0.8, 0.7, rgb);
		for (int j = 0; j < nx; j++) {
			*ptr++ = rgb[0];
			*ptr++ = rgb[1];
			*ptr++ = rgb[2];
			*ptr++ = 255;
		}
	}
    obj -> setVertexColors(vertexColors);
}

void Example001a::createVertexColoring2(TubularGridSpec* gridSpec,
                                        SpacialSurfaceObject* obj) {
    int nx = gridSpec -> nx;
    int ny = gridSpec -> ny;
    int rgb[3];
    GLubyte* vertexColors = new GLubyte[4*nx*ny];
    GLubyte* ptr = vertexColors;
    for (int i = 0; i < ny; i++) {
		// berechne hue und konvertiere zu RGB
		float hue = i*360.0/(ny);
		convertToRGB(hue, 0.8, 0.7, rgb); 
		for (int j = 0; j < nx; j++) {
			*ptr++ = rgb[0];
			*ptr++ = rgb[1];
			*ptr++ = rgb[2];
			*ptr++ = 255;
		}
	}
    obj -> setVertexColors(vertexColors);
}

void Example001a::convertToRGB(float hue, float ps, float pv, int* rgb) {
	float b = (pv < 0.0 ? 0.0 : (pv > 1.0 ? 1.0 : pv));
	float s = (ps < 0.0 ? 0.0 : (ps > 1.0 ? 1.0 : ps));
	float h = hue / 60.0;
	if (s == 0.0) {
		int bInt = (int)(b * 255);
		rgb[0] = bInt;
		rgb[1] = bInt;
		rgb[2] = bInt;
		return;
	}
	float intpart, fractpart;
	fractpart = modf(h, &intpart);
	float p = (1.0 - s)*b;
	float q = (1.0 - (s * fractpart))*b;
	float t = (1.0 - (s * (1.0 - fractpart)))*b;
	int ip = (int) intpart;
	switch (ip)
	{
		case 0:
		    scaleRgbToInt(b, t, p, rgb);
		    break;
		case 1:
		    scaleRgbToInt(q, b, p, rgb);
		    break;
		case 2:
		    scaleRgbToInt(p, b, t, rgb);
		    break;
		case 3:
		    scaleRgbToInt(p, q, b, rgb);
		    break;
		case 4:
		    scaleRgbToInt(t, p, b, rgb);
		    break;
		default:
		   scaleRgbToInt(b, p, q, rgb);
	}
	
}

void Example001a::scaleRgbToInt(float r, float g, float b, int* rgb) {
    rgb[0] = round(255*r);
    rgb[1] = round(255*g);
    rgb[2] = round(255*b);
}

void Example001a::fn1 (float t, SpacialPoint* ft) {
    ft -> x = 3.0f*cos(t);
    ft -> y = 3.0f*sin(t);
    ft -> z = 2.0f/3.0f*t;
}
void Example001a::fn1d1 (float t, SpacialPoint* ft) {
    ft -> x = -3.0f*sin(t);
    ft -> y = 3.0f*cos(t);
    ft -> z =2.0f/3.0f;
}
void Example001a::fn1d2 (float t, SpacialPoint* ft) {
    ft -> x = -3.0f*cos(t);
    ft -> y = -3.0f*sin(t);
    ft -> z = 0.0f;
}

void Example001a::fn2 (float t, SpacialPoint* ft) {
    ft -> x = 6.0f*cos(-t);
    ft -> y = 6.0f*sin(-t);
    ft -> z = 2.0f/3.0f*t;
}
void Example001a::fn2d1 (float t, SpacialPoint* ft) {
    ft -> x = 6.0f*sin(-t);
    ft -> y = -6.0f*cos(-t);
    ft -> z =2.0f/3.0f;
}
void Example001a::fn2d2 (float t, SpacialPoint* ft) {
    ft -> x = -6.0f*cos(-t);
    ft -> y = -6.0f*sin(-t);
    ft -> z = 0.0f;
}
