#include "Example002a.h"
#include "Example002aStrings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>

//
//  Plucker-Flaeche
//
Example002a::Example002a() :
  AbstractExample(ColorSpec(0.0f, 0.0f, 0.0f), ColorSpec(0.3f, 0.3f, 0.3f)),
  mObj1(MaterialColoring(
            MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                               ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                               ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                               ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                               70.0f),
            MaterialProperties(ColorSpec(0.098f, 0.2f, 0.376f, 1.0f),
                               ColorSpec(0.298f, 0.7f, 0.2f, 1.0f),
                               ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                               ColorSpec(0.0f, 0.0f, 0.0, 1.0f),
                               50.0f))),
  fogSpec(FogSpec(LINEAR,       //  LINEAR, EXP, EXP2
                  ColorSpec(0.7f, 0.7f, 0.7f, 0.3f),
                  0.02,         //  fog density, forEXP, EXP" only
                  0.0,          //  distances, for LINEAR only
                  100.0))
{
    blaetter = 5;
    hoehe = 2.3f;
    gridSpec.nx = 12;
    gridSpec.ny = 121;
    gridSpec.x0 = 2.0f;
    gridSpec.xStop= 5.0f;
    gridSpec.y0 = -M_PI;
    gridSpec.yStop = M_PI;

    displayFlags.setShowFog(true);
    projectionSpec = new OrthographicProjectionSpec();
    projectionSpec -> setZoomFactor(0.8f);
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
}

Scene* Example002a::createScene() {
    Scene* scene = this -> setupScene();

    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createParametricFunctionMesh(
          displayFlags,
          &gridSpec,
          refPFn(&Example002a::fnp),
          (AbstractParametricFunctionExample*) this);

       
    obj1 -> addMaterial(&mObj1);
    
    obj1 -> setName (Example002a::LiteralSurface1);
    scene -> addSceneObject(obj1);

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    
    SpacialPoint displacement(2.0*r2, 2.0*r2, 2.0*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec);
    return scene;
}


void Example002a::fnp (float r, float t, VTriple* ft) {
    ft -> fn.x = r*cos(t);
    ft -> fn.y = r*sin(t);
    ft -> fn.z = hoehe * sin(blaetter*t);
    ft -> fdx.x = cos(t);
    ft -> fdx.y = sin(t);
    ft -> fdx.z = 0.0f;
    ft -> fdy.x = -r*sin(t);
    ft -> fdy.y = r*cos(t);
    ft -> fdy.z = hoehe*blaetter*cos(blaetter*t);
}


