#include "Example003.h"
#include "Example003Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>



//
//  Torus and Torusknot mit 5 Umläufen
//
Example003::Example003() :
   AbstractExample(ColorSpec(0.5f, 0.5f, 0.5f), ColorSpec(0.5f, 0.5f, 0.5f)),
   mObj1(SimpleMaterialColoring(
             MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                                ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                ColorSpec(0.0f, 0.3f, 0.0f, 1.0f),
                                90.0f))),
   mObj2(SimpleMaterialColoring(
             MaterialProperties(ColorSpec(0.1f, 0.4f, 0.2f, 1.0f),
                                ColorSpec(0.3f, 0.4f, 0.2f, 1.0f),
                                ColorSpec(0.1f, 0.4f, 0.0f, 1.0f),
                                ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                90.0f))),
  fogSpec1(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  20.0,
                  100.0)),
  fogSpec2(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  20.0,
                  100.0))
{    
    polygonSpec1.nx = 15;
    polygonSpec1.x0 = 0.0f;
    polygonSpec1.xStop = 2.0f*M_PI;
    polygonSpec1.adaptiveGrid = false;   
    polygonSpec2.nx = 15;
    polygonSpec2.x0 = 0.0f;
    polygonSpec2.xStop = 2.0f*M_PI;
    polygonSpec2.adaptiveGrid = false;

    displayFlags1.setShowFog(false);
    displayFlags1.setLightModelTwoSide(false);
    displayFlags2.setShowFog(false);
    displayFlags2.setLightModelTwoSide(false);
    
    projectionSpec = new OrthographicProjectionSpec();
    projectionSpec -> setZoomFactor(0.8f);
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
    
    curveSpec1.setInst(this);
    curveSpec1.setF(refCFn(&Example003::fn1));
    curveSpec1.setd1F(refCFn(&Example003::fn1d1));
    curveSpec1.setd2F(refCFn(&Example003::fn1d2));
    
    curveSpec2.setInst(this);
    curveSpec2.setF(refCFn(&Example003::fn2));
    curveSpec2.setd1F(refCFn(&Example003::fn2d1));
    curveSpec2.setd2F(refCFn(&Example003::fn2d2));
}

Scene* Example003::createScene() {
    
    Scene* scene = this -> setupScene();
    
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createTubularMesh2(
          displayFlags1,
          &polygonSpec1, 1.2f, 
          &curveSpec1);

    SpacialSurfaceObject* obj2 = 
       SpacialSurfaceBuilder::createTubularMesh2(
          displayFlags2,
          &polygonSpec2, 1.5f, 
          &curveSpec2);
          
    obj1 -> addMaterial(&mObj1);
    obj2 -> addMaterial(&mObj2);
    
    obj1 -> setName (Example003::LiteralTorusKnot);
    obj2 -> setName (Example003::LiteralTorus);
    scene -> addSceneObject(obj2);
    scene -> addSceneObject(obj1);

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(2.0*r2, 2.0*r2, 2.0*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    fogSpec2.setRegion(d1, d2);
    obj2 -> setFogSpec (&fogSpec2);
    return scene;
}

//  Formeln für den Torusknoten
void Example003::fn1 (float t, SpacialPoint* ft) {
    ft -> x = (3.0f*cos(5.0f*t) + 8.0f)*cos(2.0f*t);
    ft -> y = (3.0f*cos(5.0f*t) + 8.0f)*sin(2.0f*t);
    ft -> z = sin(5.0f*t)*3.0f;
}
void Example003::fn1d1 (float t, SpacialPoint* ft) {
    ft -> x = - 6.0f*cos(5.0f*t)*sin(2.0f*t) - 15.0f*cos(2.0f*t)*sin(5.0f*t) - 16.0f*sin(2.0f*t);
    ft -> y =   6.0f*cos(5.0f*t)*cos(2.0f*t) - 15.0f*sin(2.0f*t)*sin(5.0f*t) + 16.0f*cos(2.0f*t);
    ft -> z =   cos(5.0f*t) * 15.0f;
}
void Example003::fn1d2 (float t, SpacialPoint* ft) {
    ft -> x = -87.0f*cos(5.0f*t)*cos(2.0f*t) - 32.0f*cos(2.0f*t) + 60.0f*sin(2.0f*t)*sin(5.0f*t);
    ft -> y = -87.0f*cos(5.0f*t)*sin(2.0f*t) - 32.0f*sin(2.0f*t) - 60.0f*cos(2.0f*t)*sin(5.0f*t);
    ft -> z =  - sin(5.0f*t)*75.0f;
}

//  Formeln für den Torus
void Example003::fn2 (float t, SpacialPoint* ft) {
    ft -> x = 8.0f*cos(t);
    ft -> y = 8.0f*sin(t);
    ft -> z = 0.0f;
}
void Example003::fn2d1 (float t, SpacialPoint* ft) {
    ft -> x = - 8.0f*sin(t);
    ft -> y =   8.0f*cos(t);
    ft -> z =   0.0f;
}
void Example003::fn2d2 (float t, SpacialPoint* ft) {
    ft -> x = - 8.0f*cos(t);
    ft -> y = - 8.0f*sin(t);
    ft -> z =   0.0f;
}
