#include "Example004.h"
#include "Example004Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>


//
//  Torus und Torusknoten mit 10 Umläufen
//
Example004::Example004() :
  AbstractExample(ColorSpec(0.5f, 0.5f, 0.5f), ColorSpec(0.5f, 0.5f, 0.5f)),
  mObj1(SimpleMaterialColoring(
                 MaterialProperties(ColorSpec(0.6f, 0.345f, 0.0f, 1.0f),
                                    ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                    ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                    ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                    90.0f))),
  mObj2(SimpleMaterialColoring(
                 MaterialProperties(ColorSpec(0.1f, 0.4f, 0.2f, 1.0f),
                                    ColorSpec(0.3f, 0.4f, 0.2f, 1.0f),
                                    ColorSpec(0.1f, 0.4f, 0.0f, 1.0f),
                                    ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                    90.0f))),
  fogSpec1(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  20.0,
                  100.0)),
  fogSpec2(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  20.0,
                  100.0))
{   
    polygonSpec1.nx = 15;
    polygonSpec1.x0 = 0.0f;
    polygonSpec1.xStop = 2.0f*M_PI;
    polygonSpec1.adaptiveGrid = false;
    
    polygonSpec2.nx = 15;
    polygonSpec2.x0 = 0.0f;
    polygonSpec2.xStop = 2.0f*M_PI;
    polygonSpec2.adaptiveGrid = false;

    displayFlags1.setShowFog(false);
    displayFlags1.setLightModelTwoSide(false);
    displayFlags2.setShowFog(false);
    displayFlags2.setLightModelTwoSide(false);
    
    projectionSpec = new OrthographicProjectionSpec();
    projectionSpec -> setZoomFactor(0.8f);
    
    c1.setInst(this);
    c1.setF(refCFn(&Example004::fn1));
    c1.setd1F(refCFn(&Example004::fn1d1));
    c1.setd2F(refCFn(&Example004::fn1d2));
    
    c2.setInst(this);
    c2.setF(refCFn(&Example004::fn2));
    c2.setd1F(refCFn(&Example004::fn2d1));
    c2.setd2F(refCFn(&Example004::fn2d2));
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
}

Scene* Example004::createScene() {
    
    Scene* scene = this ->setupScene();
    
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createTubularMesh2(
          displayFlags1,
          &polygonSpec1, 0.8f, 
          &c1);

    SpacialSurfaceObject* obj2 = 
       SpacialSurfaceBuilder::createTubularMesh2(
          displayFlags2,
          &polygonSpec2, 1.2f, 
          &c2);

    obj1 -> addMaterial(&mObj1);
    obj2 -> addMaterial(&mObj2);
    
    obj1 -> setName (Example004::LiteralTorusKnot);
    obj2 -> setName (Example004::LiteralTorus);
    scene -> addSceneObject(obj2);
    scene -> addSceneObject(obj1);
    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(2.0*r2, 2.0*r2, 2.0*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    fogSpec2.setRegion(d1, d2);
    obj2 -> setFogSpec (&fogSpec2);
    return scene;
}

// Gleichungen des Torusknotens

void Example004::fn1 (float t, SpacialPoint* ft) {
    ft -> x = (cos(10.0f*t) * 3.0f + 8.0f)*cos(t);
    ft -> y = (cos(10.0f*t) * 3.0f + 8.0f)*sin(t);
    ft -> z = sin(10.0f*t) * 3.0f;
}
void Example004::fn1d1 (float t, SpacialPoint* ft) {
    ft -> x = - 3*cos(10*t)*sin(t) - 30*cos(t)*sin(10*t) - 8*sin(t);
    ft -> y =   3*cos(10*t)*cos(t) - 30*sin(t)*sin(10*t) + 8*cos(t);
    ft -> z =   cos(10.0f*t) * 30.0f;
}
void Example004::fn1d2 (float t, SpacialPoint* ft) {
    ft -> x = - 303*cos(10*t)*cos(t) + 60*sin(t)*sin(10*t) - 8*cos(t);
    ft -> y = - 303*cos(10*t)*sin(t) - 60*cos(t)*sin(10*t) - 8*sin(t);
    ft -> z = - 300*sin(10.0f*t);
}

//  Gleichungen des Torus

void Example004::fn2 (float t, SpacialPoint* ft) {
    ft -> x = 8.0f*cos(t);
    ft -> y = 8.0f*sin(t);
    ft -> z = 0.0f;
}

void Example004::fn2d1 (float t, SpacialPoint* ft) {
    ft -> x = - 8.0f*sin(t);
    ft -> y =   8.0f*cos(t);
    ft -> z =   0.0f;
}
void Example004::fn2d2 (float t, SpacialPoint* ft) {
    ft -> x = - 8.0f*cos(t);
    ft -> y = - 8.0f*sin(t);
    ft -> z =   0.0f;
}
