#include "Example005.h"
#include "Example005Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>



//
//  Whitney-Umbrella
//
Example005::Example005() :
   AbstractExample(ColorSpec(0.5f, 0.5f, 0.5f), ColorSpec(0.5f, 0.5f, 0.5f)),
   mObj1(MaterialColoring(
                MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                                   ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                   ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                   ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                                   70.0f),
                MaterialProperties(ColorSpec(0.1f, 0.2f, 0.1f, 1.0f),
                                   ColorSpec(0.3f, 0.7f, 0.2f, 1.0f),
                                   ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                                   ColorSpec(0.1f, 0.3f, 0.0, 1.0f),
                                   50.0f))),
  fogSpec(FogSpec(LINEAR,       //  LINEAR, EXP, EXP2
                  ColorSpec(0.7f, 0.7f, 0.7f, 0.3f),
                  0.02,         // fog density, forEXP, EXP" only
                  0.0,       //  distances, for LINEAR only
                  100.0))
{
    gridSpec.nx = 48;
    gridSpec.ny = 48;
    gridSpec.x0 = -3.0f;
    gridSpec.xStop= 3.0f;
    gridSpec.y0 = -3.0f;
    gridSpec.yStop = 3.0f;

    displayFlags.setShowFog(true);
    projectionSpec = new OrthographicProjectionSpec();
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
}

Scene* Example005::createScene() {
    Scene* scene = this ->setupScene();
    
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createParametricFunctionMesh(
          displayFlags,
          &gridSpec, refPFn(&Example005::fn5),
          (AbstractParametricFunctionExample*) this);

    obj1 -> addMaterial(&mObj1);
    
    obj1 -> setName (Example005::LiteralSurface1);
    scene -> addSceneObject(obj1);
    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(4.0*r2, 0.0, 0.0);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
        
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec);
    
    return scene;
}


void Example005::fn5 (float u, float v, VTriple* ft) {
    ft -> fn.x = u*v;
    ft -> fn.y = 3*u;
    ft -> fn.z = v*v;
    
    ft -> fdx.x = v;
    ft -> fdx.y = 3.0f;
    ft -> fdx.z = 0.0f;
    
    ft -> fdy.x = u;
    ft -> fdy.y = 0.0f;
    ft -> fdy.z = 2.0f*v;
}
