#include "Example006.h"
#include "Example006Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>

//
//  Moebiusband
//
Example006::Example006() :
    AbstractExample(ColorSpec(1.0f, 1.0f, 1.0f), ColorSpec(0.3f, 0.3f, 0.3f)),
    a(9.0f),
    mObj1(MaterialColoring(
               MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                                  ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                  ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                  ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                                  70.0f),
               MaterialProperties(ColorSpec(0.1f, 0.2f, 0.1f, 1.0f),
                                  ColorSpec(0.3f, 0.7f, 0.2f, 1.0f),
                                  ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                                  ColorSpec(0.1f, 0.3f, 0.0, 1.0f),
                                  50.0f))),
  fogSpec(FogSpec(LINEAR,       //  LINEAR, EXP, EXP2
                  ColorSpec(1.0f, 1.0f, 1.0f, 1.0f),
                  0.02,         // fog density, forEXP, EXP" only
                  0.0,       //  distances, for LINEAR only
                  100.0))
{
    gridSpec.nx = 41;
    gridSpec.ny = 6;
    gridSpec.x0 = -M_PI;
    gridSpec.xStop= M_PI;
    gridSpec.y0 = -0.2f;
    gridSpec.yStop = 0.2f;

    displayFlags.setShowFog(true);
    projectionSpec = new OrthographicProjectionSpec();
    projectionSpec -> setZoomFactor(0.8f);
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
}

Scene* Example006::createScene() {
    Scene* scene = this -> setupScene();
    
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createParametricFunctionMesh(
          displayFlags,
          &gridSpec, refPFn(&Example006::fn5),
          (AbstractParametricFunctionExample*) this);


    obj1 -> addMaterial(&mObj1);
    
    obj1 -> setName (Example006::LiteralSurface1);
    scene -> addSceneObject(obj1);
    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(2*r2, 2*r2, 2*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 1, 0);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
        
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec);
    return scene;
}




void Example006::fn5 (float u, float v, VTriple* ft) {
    ft -> fn.x = a*(cos(u) + v*cos(u/2.0f)*cos(u));
    ft -> fn.y = a*(sin(u) + v*cos(u/2.0f)*sin(u));
    ft -> fn.z = a*v*sin(u/2.0f);
    
    ft -> fdx.x = - (1/2.0f)*a*cos(u)*v*sin((1/2.0f)*u) - a*v*cos((1/2.0f)*u)*sin(u) - a*sin(u);
    ft -> fdx.y = - (1/2.0f)*a*v*sin(u)*sin((1/2.0f)*u) + a*v*cos((1/2.0f)*u)*cos(u) + a*cos(u);
    ft -> fdx.z = (1/2.0f)*a*v*cos((1/2.0f)*u);
    
    ft -> fdy.x = a*cos(u)*cos((1/2.0f)*u);
    ft -> fdy.y = a*sin(u)*cos((1/2.0f)*u);
    ft -> fdy.z = a*sin(u/2.0f);
}
