#include "Example007.h"
#include "Example007Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>


//
//  Sphärische Helix und Kugel
//
Example007::Example007() :
  AbstractExample(ColorSpec(0.5f, 0.5f, 0.5f),
                  ColorSpec(0.5f, 0.5f, 0.5f)),
  mObj1(SimpleMaterialColoring(
                 MaterialProperties(ColorSpec(0.6f, 0.15f, 0.0f, 1.0f),
                                    ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                    ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                    ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                    90.0f))),
  mObj2(MaterialColoring(
                 MaterialProperties(ColorSpec(0.0f, 0.45f, 0.6f, 1.0f),
                                    ColorSpec(0.7f, 0.25f, 0.3f, 1.0f),
                                    ColorSpec(0.0f, 0.9f, 0.6f, 1.0f),
                                    ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                    90.0f),
                 MaterialProperties(ColorSpec(0.7f, 0.7f, 0.05f, 1.0f),
                                    ColorSpec(0.7f, 0.25f, 0.3f, 1.0f),
                                    ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                    ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                    90.0f))
                                    ),
  mObj3(SimpleMaterialColoring(
                 MaterialProperties(ColorSpec(0.0f, 0.05f, 1.0f, 1.0f),
                                    ColorSpec(0.07f, 0.2f, 0.3f, 1.0f),
                                    ColorSpec(0.3f, 0.1f, 0.6f, 1.0f),
                                    ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                    90.0f))),
  fogSpec1(FogSpec(LINEAR,       //  LINEAR, EXP, EXP2
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.02,          // fog density, forEXP, EXP" only
                  0.0,           //  distances, for LINEAR only
                  100.0)),
  fogSpec2(FogSpec(LINEAR,       //  LINEAR, EXP, EXP2
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.02,          // fog density, forEXP, EXP" only
                  0.0,           //  distances, for LINEAR only
                  100.0)),
  fogSpec3(FogSpec(LINEAR,       //  LINEAR, EXP, EXP2
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.02,          // fog density, forEXP, EXP" only
                  0.0,           //  distances, for LINEAR only
                  100.0))
{
    polygonSpec.nx = 9;
    polygonSpec.x0 = M_PI / 2.0f + 0.3f;
    polygonSpec.xStop = 3.0f/2.0f*M_PI - 0.3f;
    polygonSpec.adaptiveGrid = false;
    
    
    gridSpec.nx = 27;
    gridSpec.ny = 37;
    gridSpec.x0 = -(M_PI/2.0 - 0.3);   // phi (Breitengrade)
    gridSpec.xStop= M_PI/2.0 - 0.3;
    gridSpec.y0 = -M_PI;               // t   (Längengrade)
    gridSpec.yStop = M_PI;
    
    displayFlags1.setShowFog(true);
    displayFlags2.setShowFog(true);
    displayFlags3.setShowFog(true);
    projectionSpec = new OrthographicProjectionSpec();
    projectionSpec -> setZoomFactor(0.7f);
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
    
    ls = &(lightSources[2]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(1.0f, 1.0f, 1.0f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(0.0f, 0.0f, 0.0f);

    curveSpec1.setInst(this);
    curveSpec1.setF(refCFn(&Example007::fn1));
    curveSpec1.setd1F(refCFn(&Example007::fn1d1));
    curveSpec1.setd2F(refCFn(&Example007::fn1d2));
    
    curveSpec3.setInst(this);
    curveSpec3.setF(refCFn(&Example007::fn3));
    curveSpec3.setd1F(refCFn(&Example007::fn3d1));
    curveSpec3.setd2F(refCFn(&Example007::fn3d2));
}

Scene* Example007::createScene() {
    Scene* scene = this -> setupScene();
    
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags1,
          &polygonSpec, 0.04f, 
          &curveSpec1);
    
    SpacialSurfaceObject* obj2 = 
       SpacialSurfaceBuilder::createParametricFunctionMesh(
          displayFlags2,
          &gridSpec, refPFn(&Example007::fn2),
          (AbstractParametricFunctionExample*) this);
          
    SpacialSurfaceObject* obj3 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags3,
          &polygonSpec, 0.03f, 
          &curveSpec3);

    obj1 -> addMaterial(&mObj1);
    obj1 -> setName (Example007::LiteralSurface1);
    scene -> addSceneObject(obj1);
   
    obj2 -> addMaterial(&mObj2);
    obj2 -> setName(Example007::LiteralSurface2);
    scene -> addSceneObject(obj2);
    
    obj3 -> addMaterial(&mObj3);
    obj3 -> setName(Example007::LiteralSurface3);
    scene -> addSceneObject(obj3);

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);    
    SpacialPoint displacement(2.0*r2, 2.0*r2, 2.0*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 1, 0);
    scene -> getCameraSettings() -> position = centerPt + displacement;
        
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    fogSpec2.setRegion(d1, d2);
    fogSpec3.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    obj2 -> setFogSpec (&fogSpec2);
    obj3 -> setFogSpec (&fogSpec3);
    return scene;
}


void Example007::fn1 (float t, SpacialPoint* ft) {
    ft -> x = cos(t)*cos(24.0f*t);
    ft -> y = cos(t)*sin(24.0f*t);
    ft -> z = sin(t);
}
void Example007::fn1d1 (float t, SpacialPoint* ft) {
    ft -> x = -24.0f*cos(t)*sin(24.0f*t) - sin(t)*cos(24.0f*t);
    ft -> y =  24.0f*cos(t)*cos(24.0f*t) - sin(t)*sin(24.0f*t);
    ft -> z =  cos(t);
}
void Example007::fn1d2 (float t, SpacialPoint* ft) {
    ft -> x = -577.0f*cos(t)*cos(24.0f*t) + 48.0f*sin(t)*sin(24.0f*t);
    ft -> y = -577.0f*cos(t)*sin(24.0f*t) - 48.0f*sin(t)*cos(24.0f*t);
    ft -> z =  - sin(t);
}

//  Parameterdarstellung der Kugel
void Example007::fn2 (float phi, float t, VTriple* ft) {
	const float r = 0.90f;
	
    ft -> fn.x = r*cos(phi)*cos(t);
    ft -> fn.y = r*cos(phi)*sin(t);
    ft -> fn.z = -r*sin(phi);  //  Entfernung des Minuszeichens hier
                               //  und in der ersten Ableitung vertauscht
                               //  Vorderseite und Rückseite.  
    // Ableitungen nach phi
    ft -> fdx.x = -r*sin(phi)*cos(t);
    ft -> fdx.y = -r*sin(phi)*sin(t);
    ft -> fdx.z = -r*cos(phi);
    // Ableitungen nach t 
    ft -> fdy.x = -r*cos(phi)*sin(t);
    ft -> fdy.y = r*cos(phi)*cos(t);
    ft -> fdy.z = 0.0f;
}


void Example007::fn3 (float t, SpacialPoint* ft) {
	const float r3 = 0.79f;
    ft -> x = r3*cos(t)*cos(24.0f*t);
    ft -> y = r3*cos(t)*sin(24.0f*t);
    ft -> z = r3*sin(t);
}
void Example007::fn3d1 (float t, SpacialPoint* ft) {
	const float r3 = 0.79f;
    ft -> x = r3*( -24.0f*cos(t)*sin(24.0f*t) - sin(t)*cos(24.0f*t));
    ft -> y = r3*(  24.0f*cos(t)*cos(24.0f*t) - sin(t)*sin(24.0f*t));
    ft -> z = r3*cos(t);
}
void Example007::fn3d2 (float t, SpacialPoint* ft) {
	const float r3 = 0.79f;
    ft -> x = r3*(-577.0f*cos(t)*cos(24.0f*t) + 48.0f*sin(t)*sin(24.0f*t));
    ft -> y = r3*(-577.0f*cos(t)*sin(24.0f*t) - 48.0f*sin(t)*cos(24.0f*t));
    ft -> z =  - r3*sin(t);
}

/*

void Example007::fn3 (float t, SpacialPoint* ft) {
	const float r3 = 0.79f;
    ft -> x = r3*cos(t)*cos(24.0f*t);
    ft -> y = r3*cos(t)*sin(-24.0f*t);
    ft -> z = r3*sin(t);
}
//  1. Ableitung nach t
void Example007::fn3d1 (float t, SpacialPoint* ft) {
	const float r3 = 0.79f;
    ft -> x = r3*(-24.0f*cos(t)*sin(24.0f*t) - sin(t)*cos(24.0f*t));
    ft -> y = r3*(-24.0f*cos(-t)*cos(-24.0f*t) + sin(-t)*sin(-24.0f*t));
    ft -> z =  r3*cos(t);
}
//  2. Ableitung nach t
void Example007::fn3d2 (float t, SpacialPoint* ft) {
	const float r3 = 0.79f;
    ft -> x = r3*(-577.0f*cos(t)*cos(24.0f*t) + 48.0f*sin(t)*sin(24.0f*t));
    ft -> y = r3*(577.0f*cos(t)*sin(24.0f*t) + 48.0f*sin(t)*cos(24.0f*t));
    ft -> z =  - r3*sin(t);
}  */

