#include "Example009.h"
#include "Example009Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>


//
//  Interferenz zweier Wellen
//
Example009::Example009() :
   AbstractExample(ColorSpec(0.2f, 0.2f, 0.2f),   //  clear color
                   ColorSpec(0.4f, 0.4f, 0.4f)),  //  ambient light color)
   mObj1(MaterialColoring(
              MaterialProperties(ColorSpec(0.1255f, 0.2f, 0.098f, 1.0f),
                                 ColorSpec(0.937f, 0.698f, 0.2627f, 1.0f),
                                 ColorSpec(0.0f, 0.8f, 0.145f, 1.0f),
                                 ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                 90.0f),
              MaterialProperties(ColorSpec(0.2f, 0.2f, 0.3f, 1.0f),  // ambient
                                 ColorSpec(0.3f, 0.6f, 0.7f, 1.0f),  // directed
                                 ColorSpec(0.3f, 0.9f, 0.0f, 1.0f),  // specular
                                 ColorSpec(0.1f, 0.1f, 0.0f, 1.0f),  // emissive
                                 90.0f)
                            )),
  fogSpec1(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0))
{
	a1 = 3.0f;
	b1 = 6.0f;
	c1 = 1.8f;
	a2 = 4.0f;
	b2 = 6.0f;
	c2 = 1.8f;
	cx1 = 2.6f;   cy1 = 2.6;
	cx2 = -2.6f;  cy2 = -2.6f;
	
    gridSpec.nx = 103;
    gridSpec.ny = 103;
    gridSpec.x0 = -10.0f;
    gridSpec.xStop= 10.0f;
    gridSpec.y0 = -10.0f;
    gridSpec.yStop = 10.0f;

    displayFlags.setShowFog(false);
    projectionSpec = new OrthographicProjectionSpec();
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.5f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(0.8f, 0.0f, 0.8f, 1.0f);
    ls -> direction = SpacialPoint(90.0f, 125.0f, 55.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.35f, 0.5f, 0.3f, 1.0f);
    ls -> specularColor = ColorSpec(0.2f, 0.2f, 0.2f, 1.0f);
    ls -> direction = SpacialPoint(130.0f, 130.0f, 125.0f);
    
    ls = &(lightSources[2]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.0f, 0.6f, 0.0f, 1.0f);
    ls -> specularColor = ColorSpec(0.2f, 0.5f, 0.2f, 1.0f);
    ls -> direction = SpacialPoint(80.0f, -130.0f, -125.0f);
}

Scene* Example009::createScene() {
    Scene* scene = this -> setupScene();
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createFunctionMesh(
           displayFlags,
           &gridSpec, 
           refFn(&Example009::fn),
           (AbstractSpacialFunctionExample*) this);
    obj1 -> addMaterial(&mObj1);
    
    obj1 -> setName (Example009::LiteralSurface1);
    scene -> addSceneObject(obj1);  

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(4.0*r2, 0.0, 0.0);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    
    return scene;
}

void Example009::fn (float x, float y, Triple* t) {
    t -> fn =  a1*exp(-sqrt(pow(y - cy1, 2) + pow(x - cx1, 2))/b1)*cos(c1*sqrt(pow(y - cy1, 2) + pow(x - cx1, 2)))
             + a2*exp(-sqrt(pow(y - cy2, 2) + pow(x - cx2, 2))/b2)*cos(c2*sqrt(pow(y - cy2, 2) + pow(x - cx2, 2)));
    t -> fdx = -sqrt(pow(y - cy1, 2) + pow(x - cx1, 2))*exp(-sqrt(pow(y - cy1, 2) + pow(x - cx1, 2))/b1)
                    *(a1*b1*c1*x*sin(c1*sqrt(pow(y - cy1, 2) + pow(x - cx1, 2)))+a1*x*cos(c1*sqrt(pow(y - cy1, 2) + pow(x - cx1, 2))))
               -sqrt(pow(y - cy2, 2) + pow(x - cx2, 2))*exp(-sqrt(pow(y - cy2, 2) + pow(x - cx2, 2))/b2)
                    *(a2*b2*c2*x*sin(c1*sqrt(pow(y - cy2, 2) + pow(x - cx2, 2)))+a2*x*cos(c2*sqrt(pow(y - cy2, 2) + pow(x - cx2, 2))));

    t -> fdy = -sqrt(pow(y - cy1, 2) + pow(x - cx1, 2))*exp(-sqrt(pow(y - cy1, 2) + pow(x - cx1, 2))/b1)
                    *(a1*b1*c1*y*sin(c1*sqrt(pow(y - cy1, 2) + pow(x - cx1, 2)))+a1*y*cos(c1*sqrt(pow(y - cy1, 2) + pow(x - cx1, 2))))
               -sqrt(pow(y - cy2, 2) + pow(x - cx2, 2))*exp(-sqrt(pow(y - cy2, 2) + pow(x - cx2, 2))/b2)
                    *(a2*b2*c2*y*sin(c2*sqrt(pow(y - cy2, 2) + pow(x - cx2, 2)))+a2*y*cos(c2*sqrt(pow(y - cy2, 2) + pow(x - cx2, 2))));
}

