#include "Example010.h"
#include "Example010Strings.h"
#include "Sphere.h"
#include "PerspectiveProjectionSpec.h"
#include <math.h>


//
//  Zwei doppelseitig gefaerbte, einander durchdringende Flaechen.
//  Dieses Beispiel ist aus der Mathematica-Dokumentation entnommen.
//
Example010::Example010():
  AbstractExample(ColorSpec(0.5f, 0.5f, 0.5f), ColorSpec(0.5f, 0.5f, 0.5f)),
  mObj1(MaterialColoring(
               MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                                  ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                  ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                  ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                                  90.0f),
               MaterialProperties(ColorSpec(0.1f, 0.2f, 0.1f, 1.0f),
                                  ColorSpec(0.3f, 0.7f, 0.2f, 1.0f),
                                  ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                                  ColorSpec(0.1f, 0.3f, 0.0, 1.0f),
                                  90.0f))),
  mObj2(MaterialColoring(
               MaterialProperties(ColorSpec(0.2f, 0.3f, 0.6f),
                                  ColorSpec(0.2f, 0.3f, 0.6f),
                                  ColorSpec(0.6f, 0.6f, 0.3f),
                                  ColorSpec(0.0f, 0.0f, 0.2f),
                                  90.0f),
               MaterialProperties(ColorSpec(0.18f, 0.18f, 0.0f),
                                  ColorSpec(0.35f, 0.25f, 0.0f),
                                  ColorSpec(0.5f, 0.5f, 0.6f),
                                  ColorSpec(0.45f, 0.30f, 0.0f),
                                  90.0f))),
  fogSpec1(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0)),
  fogSpec2(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0))
{
    gridSpec.nx = 53;
    gridSpec.ny = 53;
    gridSpec.x0 = -3.2f;
    gridSpec.xStop= 3.2f;
    gridSpec.y0 = -3.2f;
    gridSpec.yStop = 3.2f;

    displayFlags1.setShowFog(false);
    displayFlags2.setShowFog(false);
    projectionSpec = new PerspectiveProjectionSpec();
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
}

Scene* Example010::createScene() {
    Scene* scene = this -> setupScene();
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createFunctionMesh(
          displayFlags1,
          &gridSpec, 
          refFn(&Example010::fn),
          (AbstractSpacialFunctionExample*) this);

    SpacialSurfaceObject* obj2 = 
       SpacialSurfaceBuilder::createFunctionMesh(
          displayFlags2,
          &gridSpec, 
          refFn(&Example010::fn2),
          (AbstractSpacialFunctionExample*) this);
          
    obj1 -> addMaterial(&mObj1);
    obj2 -> addMaterial(&mObj2);
    
    obj1 -> setName (Example010::LiteralSurface1);
    obj2 -> setName (Example010::LiteralSurface2);
    scene -> addSceneObject(obj1);
    scene -> addSceneObject(obj2);
    
    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = 3.0 / sqrt(3.0f);
    SpacialPoint displacement(r*r2, r*r2, r*r2);  //  why this ?
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    fogSpec2.setRegion(d1, d2);
    obj2 -> setFogSpec (&fogSpec2);
    
    return scene;
}

void Example010::fn (float x, float y, Triple* t) {
    t -> fn = (1.0f - sin(x))*(2.0f - cos(2*y))/2.0f;
    t -> fdx = -cos(x)*(2 - cos(2*y))/2.0f;
    t -> fdy = (1 - sin(x))*sin(2*y);
}

void Example010::fn2 (float x, float y, Triple* t) {
    t -> fn = (2.0f + sin(x))*(1.0f + cos(2*y))/2.0f;
    t -> fdx = cos(x)*(1.0f +  cos(2*y))/2.0f;
    t -> fdy = -(2.0f + sin(x))*sin(y);
}
