#include "Example015.h"
#include "Example015Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>



//
//  Borromäische Ringe
//  http://paulbourke.net/geometry/borromean
//
Example015::Example015() :
   AbstractExample(ColorSpec(0.5f, 0.5f, 0.5f), ColorSpec(0.5f, 0.5f, 0.5f)),
   mObj1(SimpleMaterialColoring(
             MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                                ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                ColorSpec(0.0f, 0.3f, 0.0f, 1.0f),
                                90.0f))),
   mObj2(SimpleMaterialColoring(
             MaterialProperties(ColorSpec(0.1f, 0.4f, 0.2f, 1.0f),
                                ColorSpec(0.3f, 0.4f, 0.2f, 1.0f),
                                ColorSpec(0.1f, 0.4f, 0.0f, 1.0f),
                                ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                90.0f))),
   mObj3(SimpleMaterialColoring(
             MaterialProperties(ColorSpec(0.1f, 0.3f, 0.6f, 1.0f),
                                ColorSpec(0.1f, 0.55f, 0.4f, 1.0f),
                                ColorSpec(0.1f, 0.0f, 0.4f, 1.0f),
                                ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                                90.0f))),
  fogSpec1(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0)),
  fogSpec2(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0)),
  fogSpec3(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0))
{    
    polygonSpec1.nx = 17;
    polygonSpec1.x0 = 0.0f;
    polygonSpec1.xStop = 2.0f*M_PI;
    polygonSpec1.adaptiveGrid = false;   
    polygonSpec2.nx = 17;
    polygonSpec2.x0 = 0.0f;
    polygonSpec2.xStop = 2.0f*M_PI;
    polygonSpec2.adaptiveGrid = false;  
    polygonSpec3.nx = 17;
    polygonSpec3.x0 = 0.0f;
    polygonSpec3.xStop = 2.0f*M_PI;
    polygonSpec3.adaptiveGrid = false;

    displayFlags1.setShowFog(false);
    displayFlags2.setShowFog(false);
    displayFlags3.setShowFog(false);
    
    projectionSpec = new OrthographicProjectionSpec();
    projectionSpec -> setZoomFactor(0.84f);
    
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
    
    curveSpec1.setInst(this);
    curveSpec1.setF(refCFn(&Example015::fn1));
    curveSpec1.setd1F(refCFn(&Example015::fn1d1));
    curveSpec1.setd2F(refCFn(&Example015::fn1d2));
    
    curveSpec2.setInst(this);
    curveSpec2.setF(refCFn(&Example015::fn2));
    curveSpec2.setd1F(refCFn(&Example015::fn1d1));
    curveSpec2.setd2F(refCFn(&Example015::fn1d2));
    
    curveSpec3.setInst(this);
    curveSpec3.setF(refCFn(&Example015::fn3));
    curveSpec3.setd1F(refCFn(&Example015::fn1d1));
    curveSpec3.setd2F(refCFn(&Example015::fn1d2));
}

Scene* Example015::createScene() {
    
    Scene* scene = this -> setupScene();
    
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags1,
          &polygonSpec1, 0.12f, 
          &curveSpec1);

    SpacialSurfaceObject* obj2 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags2,
          &polygonSpec2, 0.12f, 
          &curveSpec2);

    SpacialSurfaceObject* obj3 = 
       SpacialSurfaceBuilder::createTubularMesh(
          displayFlags3,
          &polygonSpec3, 0.12f, 
          &curveSpec3);
          
    obj1 -> addMaterial(&mObj1);
    obj2 -> addMaterial(&mObj2);
    obj3 -> addMaterial(&mObj3);
    
    obj1 -> setName (Example015::LiteralRing1);
    obj2 -> setName (Example015::LiteralRing2);
    obj3 -> setName (Example015::LiteralRing3);
    scene -> addSceneObject(obj1);
    scene -> addSceneObject(obj2);
    scene -> addSceneObject(obj3);

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(0.0, 0.0, 4.0*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 1, 0);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    fogSpec2.setRegion(d1, d2);
    obj2 -> setFogSpec (&fogSpec2);
    fogSpec3.setRegion(d1, d2);
    obj3 -> setFogSpec (&fogSpec3);
    return scene;
}

//  Formeln für den Ring 1
void Example015::fn1 (float t, SpacialPoint* ft) {
	float r = sqrt(3.0f)/3.0f;
    ft -> x = cos(t);
    ft -> y = sin(t) + r;
    ft -> z = cos(3*t)/5.0f;
}
void Example015::fn1d1 (float t, SpacialPoint* ft) {
    ft -> x = - sin(t);
    ft -> y =   cos(t);
    ft -> z =   -sin(3*t)*3.0f/5.0f;
}
void Example015::fn1d2 (float t, SpacialPoint* ft) {
    ft -> x = -cos(t);
    ft -> y = -sin(t);
    ft -> z =  - cos(3*t)*9.0f/5.0f;
}

//  Formeln für den Ring 2
void Example015::fn2 (float t, SpacialPoint* ft) {
	float r = sqrt(3.0f)/3.0f;
    ft -> x = cos(t) + 0.5f;
    ft -> y = sin(t) - r/2.0f;
    ft -> z = cos(3*t)/5.0f;
}



//  Formeln für den Ring 3
void Example015::fn3 (float t, SpacialPoint* ft) {
	float r = sqrt(3.0f)/3.0f;
    ft -> x = cos(t) - 0.5f;
    ft -> y = sin(t) - r/2.0f;
    ft -> z = cos(3*t)/5.0f;
}
