#include "Example017.h"
#include "Example017Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>



//
//  Drei Spotlights
//  Das Beispiel zeigt additive Farbmischung derLichtfarben dreier
//  punktförmiger Lichtquellen auf einer weißen Oberfläche
//
Example017::Example017() :
   AbstractExample(ColorSpec(0.5f, 0.5f, 0.5f),    //  clear color
                   ColorSpec(0.0f, 0.0f, 0.0f)),   // ambient light
   mObj1(MaterialColoring(
            MaterialProperties(ColorSpec(1.0f, 1.0f, 1.0f, 1.0f),
                               ColorSpec(1.0f, 1.0f, 1.0f, 1.0f),
                               ColorSpec(1.0f, 1.0f, 1.0f, 1.0f),
                               ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                               70.0f),
            MaterialProperties(ColorSpec(1.0f, 1.0f, 1.0f, 1.0f),
                               ColorSpec(1.0f, 1.0f, 1.0f, 1.0f),
                               ColorSpec(1.0f, 1.0f, 1.0f, 1.0f),
                               ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                               50.0f))),
  fogSpec(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  20.0,
                  120.0))
{ 
	float r = 2.1f;   // Radius des Kreises, aufdem die Lichtquellen liegen
    d = 9.4f;
    
    gridSpec.nx = 64;
    gridSpec.ny = 64;
    gridSpec.x0 = -d;
    gridSpec.xStop= d;
    gridSpec.y0 = -d;
    gridSpec.yStop = d;

    displayFlags.setShowFog(false);
    projectionSpec = new OrthographicProjectionSpec();
    projectionSpec -> setZoomFactor(0.8f);
    
    LightSourceProperties *ls = &(lightSources[0]);   // red spot light
    ls -> lightType = SPOT;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor  = ColorSpec(1.0f, 0.0f, 0.0f, 1.0f);
    ls -> diffuseColor  = ColorSpec(1.0f, 0.0f, 0.0f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 0.0f, 1.0f);
    ls -> direction = SpacialPoint(r*sin(0.0f*M_PI/180.0f), r*cos(0.0f*M_PI/180.0f), 35.0f);    // position
    ls -> spotDirection = SpacialPoint(0.0f, 0.0f, -1.0f);
    ls -> exponent = 128.0f;
    ls -> cutoff = 25.0f;
    
    ls = &(lightSources[1]);    // green spot light
    ls -> lightType = SPOT;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor  = ColorSpec(0.0f, 1.0f, 0.0f, 1.0f);
    ls -> diffuseColor  = ColorSpec(0.0f, 1.0f, 0.0f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 1.0f, 0.0f, 1.0f);
    ls -> direction = SpacialPoint(r*sin(120.0f*M_PI/180.0f), r*cos(120.0f*M_PI/180.0f), 35.0f);    // position
    ls -> spotDirection = SpacialPoint(0.0f, 0.0f, -1.0f);
    ls -> exponent = 128.0f;
    ls -> cutoff = 25.0f;
    
    ls = &(lightSources[2]);    //  blue spot light
    ls -> lightType = SPOT;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor  = ColorSpec(0.0f, 0.0f, 1.0f, 1.0f);
    ls -> diffuseColor  = ColorSpec(0.0f, 0.0f, 1.0f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(r*sin(240.0f*M_PI/180.0f), r*cos(240.0f*M_PI/180.0f), 35.0f);    // position
    ls -> spotDirection = SpacialPoint(0.0f, 0.0f, -1.0f);
    ls -> exponent = 128.0f;
    ls -> cutoff = 25.0f;
}

Scene* Example017::createScene() {
    Scene* scene = this -> setupScene();
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createParametricFunctionMesh(
          displayFlags,
          &gridSpec, refPFn(&Example017::fnp),
          (AbstractParametricFunctionExample*) this);

    obj1 -> addMaterial(&mObj1);
    
    obj1 -> setName (Example017::LiteralSurface1);
    scene -> addSceneObject(obj1);

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    
    SpacialPoint displacement(0.0f, 0.0f, 2.0*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 1, 0);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec);
    return scene;
}

//  Formeln für die Pluckerfläche
void Example017::fnp (float u, float v, VTriple* ft) {
	float sqr2 = sqrtf(2.0f)/2.0f;
    ft -> fn.x = sqr2*u - sqr2*v;
    ft -> fn.y = sqr2*u + sqr2*v;;
    ft -> fn.z = 0;
    
    ft -> fdx.x = sqr2;
    ft -> fdx.y = sqr2;
    ft -> fdx.z = 0.0f;
    
    ft -> fdy.x = -sqr2;
    ft -> fdy.y = sqr2;
    ft -> fdy.z = 0;
}
