#include "Example020.h"
#include "Example020Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>

//
//  zwei Helices
//
Example020::Example020() :
   AbstractExample(ColorSpec(0.3f, 0.3f, 0.3f, 0.5f), ColorSpec(0.3f, 0.3f, 0.3f)),
   mObj1(SimpleMaterialColoring(
             MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                                ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                                ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                                90.0f)/*,
              MaterialProperties(ColorSpec(0.1f, 0.2f, 0.1f, 1.0f),
                                 ColorSpec(0.3f, 0.7f, 0.2f, 1.0f),
                                 ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                                 ColorSpec(0.1f, 0.3f, 0.0, 1.0f),
                                 90.0f) */ )),
   mObj2(SimpleMaterialColoring(
              MaterialProperties(ColorSpec(0.0f, 0.0f, 0.6f, 1.0f),
                                 ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                                 ColorSpec(0.4f, 0.5f, 0.0f, 1.0f),
                                 ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                                 90.0f) /*,
              MaterialProperties(ColorSpec(0.3f, 0.3f, 0.1f, 1.0f),
                                 ColorSpec(0.7f, 0.7f, 0.2f, 1.0f),
                                 ColorSpec(0.5f, 0.6f, 0.4f, 1.0f),
                                 ColorSpec(0.3f, 0.3f, 0.0, 1.0f),
                                 90.0f) */)),
  fogSpec1(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0)),
  fogSpec2(FogSpec(LINEAR, 
                  ColorSpec(0.7f, 0.7f, 0.7f, 1.0f),
                  0.5,  // fog density
                  0.0,
                  100.0))
{
    polygonSpec.nx = 150;
    polygonSpec.x0 = -7.0f/2.0f*M_PI;
    polygonSpec.xStop = 7.0f/2.0f*M_PI;
    projectionSpec = new OrthographicProjectionSpec();

    displayFlags1.setShowFog(true);
    displayFlags2.setShowFog(true);
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
    
    c1.setInst(this);
    c1.setF(refCFn(&Example020::fn1));
    
    
    c2.setInst(this);
    c2.setF(refCFn(&Example020::fn2));
}

Scene* Example020::createScene() {
    
    Scene* scene = this -> setupScene();
 
    SpacialCurveObject* obj1 = 
       SpacialCurveBuilder::createSpacialCurveAndTexture(
          displayFlags1,
          &polygonSpec,
          &c1);


    SpacialCurveObject* obj2 = 
       SpacialCurveBuilder::createSpacialCurveAndTexture(
          displayFlags2,
          &polygonSpec,
          &c2);

    obj1 -> addMaterial(&mObj1);
    obj2 -> addMaterial(&mObj2);
    
    obj1 -> setName (Example020::LiteralHelix1);
    obj2 -> setName (Example020::LiteralHelix2);
    scene -> addSceneObject(obj1);
    scene -> addSceneObject(obj2);

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(2*r2, 2*r2, 2*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec1.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec1);
    fogSpec2.setRegion(d1, d2);
    obj2 -> setFogSpec (&fogSpec2);
    return scene;
}


void Example020::fn1 (float t, SpacialPoint* ft) {
    ft -> x = 3.0f*cos(t);
    ft -> y = 3.0f*sin(t);
    ft -> z = 2.0f/3.0f*t;
}

void Example020::fn2 (float t, SpacialPoint* ft) {
    ft -> x = 6.0f*cos(-t);
    ft -> y = 6.0f*sin(-t);
    ft -> z = 2.0f/3.0f*t;
}
