#include "ExampleKleinBottle.h"
#include "ExampleKleinBottleStrings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>

ExampleKleinBottle::ExampleKleinBottle() :
  AbstractExample(ColorSpec(0.8f, 0.8f, 0.8f), ColorSpec(0.3f, 0.3f, 0.3f)),
  mObj1(MaterialColoring(
            MaterialProperties(ColorSpec(0.6f, 0.0f, 0.0f, 1.0f),
                               ColorSpec(0.0f, 0.6f, 0.7f, 1.0f),
                               ColorSpec(0.0f, 0.9f, 0.0f, 1.0f),
                               ColorSpec(0.3f, 0.3f, 0.0f, 1.0f),
                               70.0f),
            MaterialProperties(ColorSpec(0.1f, 0.2f, 0.1f, 1.0f),
                               ColorSpec(0.3f, 0.7f, 0.2f, 1.0f),
                               ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                               ColorSpec(0.1f, 0.3f, 0.0, 1.0f),
                               50.0f))),
  fogSpec(FogSpec(LINEAR,       //  LINEAR, EXP, EXP2
                  ColorSpec(0.8f, 0.8f, 0.8f, 1.0f),
                  0.02,         //  fog density, forEXP, EXP" only
                  0.0,          //  distances, for LINEAR only
                  100.0))
{
    displayFlags.setShowFog(true);
    projectionSpec = new OrthographicProjectionSpec();
    projectionSpec -> setZoomFactor(0.8f);
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
}


Scene* ExampleKleinBottle::createScene() {
    Scene* scene = this -> setupScene();
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createParametricFunctionMesh(
          displayFlags,
          &gridSpec, refPFn(&ExampleKleinBottle::fn),
          (AbstractParametricFunctionExample*) this);

    obj1 -> addMaterial(&mObj1);
    
    obj1 -> setName (ExampleKleinBottle::LiteralSurface1);
    scene -> addSceneObject(obj1);

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    float r2 = r / sqrt(3.0f);
    
    SpacialPoint displacement(2.0*r2, 2.0*r2, 2.0*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec);
    return scene;
}

void ExampleKleinBottle::fn (float u, float v, VTriple* ft) {
	const float a = 3.0;
	/*  es wurde hier nicht angestrebt, gemeinsame Teilausdrücke
	 * zuisolieren.
	 */
    ft -> fn.x = (a + cos(u/2.0)*sin(v) - sin(u/2.0)*sin(2.0*v))*cos(u);
    ft -> fn.y = (a + cos(u/2.0)*sin(v) - sin(u/2.0)*sin(2.0*v))*sin(u);
    ft -> fn.z = sin(u/2.0)*sin(v) + cos(u/2.0)*sin(2.0*v);
    //  Ableitungen nach u
    ft -> fdx.x = (-sin(u/2.0)*sin(v)/2.0 - cos(u/2.0)*sin(2.0*v)/2.0)*cos(u) - (a + cos(u/2.0)*sin(v) - sin(u/2.0)*sin(2.0*v))*sin(u);
    ft -> fdx.y = (-sin(u/2.0)*sin(v)/2.0 - cos(u/2.0)*sin(2.0*v)/2.0)*sin(u) + (a + cos(u/2.0)*sin(v) - sin(u/2.0)*sin(2.0*v))*cos(u);
    ft -> fdx.z = cos(u/2.0)*sin(v)/2.0 - sin(u/2.0)*sin(2.0*v)/2.0;
    //  Ableitungen nach v
    ft -> fdy.x = (cos(u/2.0)*cos(v) - sin(u/2.0)*cos(2.0*v)*2.0)*cos(u);
    ft -> fdy.y = (cos(u/2.0)*cos(v) - sin(u/2.0)*cos(2.0*v)*2.0)*sin(u);
    ft -> fdy.z = sin(u/2.0)*cos(v) + cos(u/2.0)*cos(2.0*v)*2.0;
}
