#include "ExampleKleinBottle2.h"
#include "ExampleKleinBottle2Strings.h"
#include "Sphere.h"
#include "OrthographicProjectionSpec.h"
#include <math.h>
//#include <iostream>

ExampleKleinBottle2::ExampleKleinBottle2() :
  AbstractExample(ColorSpec(0.8f, 0.8f, 0.8f), ColorSpec(0.3f, 0.3f, 0.3f)),
  mObj1(MaterialColoring(
            MaterialProperties(ColorSpec(0.682f, 0.184f, 0.0f, 1.0f),
                               ColorSpec(0.0f, 0.714f, 0.1373f, 1.0f),
                               ColorSpec(0.0f, 0.9f, 0.0f, 1.9f),
                               ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                               70.0f),
            MaterialProperties(ColorSpec(0.098f, 0.251f, 0.3921f, 1.0f),
                               ColorSpec(0.298f, 0.698f, 0.5725f, 1.0f),
                               ColorSpec(0.0f, 0.8f, 0.6f, 1.0f),
                               ColorSpec(0.0f, 0.0f, 0.0f, 1.0f),
                               50.0f))),
  fogSpec(FogSpec(LINEAR,       //  LINEAR, EXP, EXP2
                  ColorSpec(0.8f, 0.8f, 0.8f, 1.0f),
                  0.02,         //  fog density, forEXP, EXP" only
                  0.0,          //  distances, for LINEAR only
                  100.0))
{
    displayFlags.setShowFog(true);
    projectionSpec = new OrthographicProjectionSpec();
    
    LightSourceProperties *ls = &(lightSources[0]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.7f, 0.7f, 0.7f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.3f, 0.7f, 0.8f, 1.0f);
    ls -> specularColor = ColorSpec(1.0f, 0.0f, 1.0f, 1.0f);
    ls -> direction = SpacialPoint(10.0f, 6.0f, 6.0f);
    
    ls = &(lightSources[1]);
    ls -> lightType = POSITIONAL;
    ls -> isOn = true;
    ls -> isFixed = true;
    ls -> ambientColor = ColorSpec(0.5f, 0.5f, 0.5f, 1.0f);
    ls -> diffuseColor = ColorSpec(0.1f, 0.6f, 0.2f, 1.0f);
    ls -> specularColor = ColorSpec(0.0f, 0.3f, 0.7f, 1.0f);
    ls -> direction = SpacialPoint(-10.0f, -6.0f, -6.0f);
}


Scene* ExampleKleinBottle2::createScene() {
    Scene* scene = this -> setupScene();
    SpacialSurfaceObject* obj1 = 
       SpacialSurfaceBuilder::createParametricFunctionMesh(
          displayFlags,
          &gridSpec, refPFn(&ExampleKleinBottle2::fn2),
          (AbstractParametricFunctionExample*) this);

    obj1 -> addMaterial(&mObj1);
    
    obj1 -> setName (ExampleKleinBottle2::LiteralSurface1);
    scene -> addSceneObject(obj1);

    Sphere sp = scene -> enclosingSphere();
    SpacialPoint centerPt = sp.getCenterPoint();
    float r = sp.getRadius();
    //std::cout << centerPt << " *** " << r <<std::endl;
    float r2 = r / sqrt(3.0f);
    SpacialPoint displacement(0.0, 5.0*r2, 0.0*r2);
    scene -> getCameraSettings() -> target = centerPt;
    scene -> getCameraSettings() -> upDirection = SpacialPoint(0, 0, 1);
    scene -> getCameraSettings() -> position = centerPt + displacement;
    
    float d1 = displacement.euclideanNorm() - r;
    float d2 = displacement.euclideanNorm() + r;
    fogSpec.setRegion(d1, d2);
    obj1 -> setFogSpec (&fogSpec);
    return scene;
}

/*
 * Es gibt sehr unterschiedliche Parametrisierungen der
 * Kleinschen Flaschen. Oft wird die Fläche aus zwei
 * Teilflächen zusammengesetzt.
 */

/*
 * https://de.wikipedia.org/wiki/Kleinsche_Flasche
 */
void ExampleKleinBottle2::fn2 (float u, float v, VTriple* ft) {
	const float b = 1.4f;
	const float h = 5.0f;
    ft -> fn.x = b*(1.0 - sin(u))*cos(u) + (2.0f - cos(u))*cos(-v)*(2*exp(-pow(u/2 - M_PI, 2)) - 1.0);
    ft -> fn.y = (2.0 - cos(u))*sin(-v);
    ft -> fn.z = h*sin(u) + 0.5*(2 -cos(u))*sin(u)*cos(-v)*exp(-pow(u - 3*M_PI/2.0, 2));
    //  Ableitungen nach u
    ft -> fdx.x = -(b*pow(cos(u), 2)) + (2*(M_PI - u/2.0)*(2 - cos(u))*cos(v))/
                    exp(pow(-M_PI + u/2.0, 2)) + 
                    (-1 + 2/exp(pow(-M_PI + u/2.0, 2)))*cos(v)*sin(u) - b*(1 - sin(u))*sin(u);
    ft -> fdx.y = -(sin(u)*sin(v));
    ft -> fdx.z = h*cos(u) + ((2 - cos(u))*cos(u)*cos(v))/(2.0*exp(pow((-3*M_PI)/2. + u, 2))) - 
                    (((-3*M_PI)/2. + u)*(2 - cos(u))*cos(v)*sin(u))/exp(pow((-3*M_PI)/2. + u, 2)) + 
                     (cos(v)*pow(sin(u), 2))/(2.0*exp(pow((-3*M_PI)/2. + u, 2)));
    //  Ableitungen nach v
    ft -> fdy.x = -((-1 + 2/exp(pow(-M_PI + u/2.0, 2)))*(2 - cos(u))*sin(v));
    ft -> fdy.y = -((2 - cos(u))*cos(v));
    ft -> fdy.z = -((2 - cos(u))*sin(u)*sin(v))/(2.0*exp(pow((-3*M_PI)/2. + u, 2)));
}


/*
 * http://mathword.wolfram.com/KleinBottle.html
 */
 /*
void ExampleKleinBottle2::fn2 (float u, float v, VTriple* ft) {
	//  es wurde hier nicht angestrebt, gemeinsame Teilausdrücke
	//  zuisolieren.

    ft -> fn.x = (cos(u/2.0)*(sqrt(2.0) + cos(v)) + sin(u/2.0)*sin(v)*cos(v))*cos(u);
    ft -> fn.y = (cos(u/2.0)*(sqrt(2.0) + cos(v)) + sin(u/2.0)*sin(v)*cos(v))*sin(u);
    ft -> fn.z = -sin(u/2.0)*(sqrt(2.0) + cos(v)) + cos(u/2.0)*sin(v)*cos(v);
    //  Ableitungen nach u
    ft -> fdx.x = (-sin(u/2.0)*sin(v)/2.0 - cos(u/2.0)*sin(2.0*v)/2.0)*cos(u) - (a + cos(u/2.0)*sin(v) - sin(u/2.0)*sin(2.0*v))*sin(u);
    ft -> fdx.y = (-sin(u/2.0)*sin(v)/2.0 - cos(u/2.0)*sin(2.0*v)/2.0)*sin(u) + (a + cos(u/2.0)*sin(v) - sin(u/2.0)*sin(2.0*v))*cos(u);
    ft -> fdx.z = cos(u/2.0)*sin(v)/2.0 - sin(u/2.0)*sin(2.0*v)/2.0;
    //  Ableitungen nach v
    ft -> fdy.x = (cos(u/2.0)*cos(v) - sin(u/2.0)*cos(2.0*v)*2.0)*cos(u);
    ft -> fdy.y = (cos(u/2.0)*cos(v) - sin(u/2.0)*cos(2.0*v)*2.0)*sin(u);
    ft -> fdy.z = sin(u/2.0)*cos(v) + cos(u/2.0)*cos(2.0*v)*2.0;
}
* */
