#include "FogSpecEditor.h"
#include "FogSpecEditorStrings.h"
#include "SceneObjectsEditor.h"


#ifdef TRACK_DESTROY_EVENTS
#include <iostream>

void* notifyR(void* data) {
	std::cout <<  "reglerR about to be destroyed" << std::endl;
	return (void*) 0;
}

void* notifyG(void* data) {
	std::cout <<  "reglerG about to be destroyed" << std::endl;
	return (void*) 0;
}

void* notifyB(void* data) {
	std::cout <<  "reglerB about to be destroyed" << std::endl;
	return (void*) 0;
}

void* notifyBoF(void* data) {
	std::cout <<  "reglerBoF about to be destroyed" << std::endl;
	return (void*) 0;
}

void* notifyEoF(void* data) {
	std::cout <<  "reglerEoF about to be destroyed" << std::endl;
	return (void*) 0;
}
#endif


FogSpecEditor::FogSpecEditor (SceneHolder& sm, SceneObjectsEditor& se)
: sceneModel(sm),
  parentEditor(se),
  red(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
  green(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
  blue(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
  beginOfFogRegion(Gtk::Adjustment::create(100.0, 0.0, 101.0, 1.0, 1.0, 1.0)),
  endOfFogRegion(Gtk::Adjustment::create(100.0, 0.0, 401.0, 1.0, 1.0, 1.0)),
  reglerR(red, Gtk::ORIENTATION_HORIZONTAL),
  reglerG(green, Gtk::ORIENTATION_HORIZONTAL),
  reglerB(blue, Gtk::ORIENTATION_HORIZONTAL),
  reglerBoF(beginOfFogRegion, Gtk::ORIENTATION_HORIZONTAL),
  reglerEoF(endOfFogRegion, Gtk::ORIENTATION_HORIZONTAL),
  suspendUpdates(false),
  container(Gtk::ORIENTATION_VERTICAL),
  container2(Gtk::ORIENTATION_VERTICAL)
{
	parentEditor.addObserver(this);
	
	reglerR.set_value_pos(Gtk::POS_LEFT);
    reglerG.set_value_pos(Gtk::POS_LEFT);
    reglerB.set_value_pos(Gtk::POS_LEFT);
    reglerBoF.set_value_pos(Gtk::POS_LEFT);
    reglerEoF.set_value_pos(Gtk::POS_LEFT);
    reglerR.set_digits(0);
    reglerG.set_digits(0);
    reglerB.set_digits(0);
    reglerBoF.set_digits(0);
    reglerEoF.set_digits(0);
    
#ifdef TRACK_DESTROY_EVENTS
    reglerR.add_destroy_notify_callback(this,  notifyR);
    reglerG.add_destroy_notify_callback(this,  notifyG);
    reglerB.add_destroy_notify_callback(this,  notifyB);
    reglerBoF.add_destroy_notify_callback(this,  notifyBoF);
    reglerEoF.add_destroy_notify_callback(this,  notifyEoF);
#endif
	add(container);
	
    container2.pack_start(reglerR, Gtk::PACK_SHRINK);
    container2.pack_start(reglerG, Gtk::PACK_SHRINK);
    container2.pack_start(reglerB, Gtk::PACK_SHRINK);
    container2.pack_start(reglerBoF, Gtk::PACK_SHRINK);
    container2.pack_start(reglerEoF, Gtk::PACK_SHRINK);
    
    container.pack_start(container2, true, true);

    red->signal_value_changed().connect(sigc::mem_fun(*this, &FogSpecEditor::onRValueChanged));
    green->signal_value_changed().connect(sigc::mem_fun(*this, &FogSpecEditor::onGValueChanged));
    blue->signal_value_changed().connect(sigc::mem_fun(*this, &FogSpecEditor::onBValueChanged));
    beginOfFogRegion->signal_value_changed().connect(sigc::mem_fun(*this, &FogSpecEditor::onBoFValueChanged));
    endOfFogRegion->signal_value_changed().connect(sigc::mem_fun(*this, &FogSpecEditor::onEoFValueChanged));

	container.show();
}

FogSpecEditor::~FogSpecEditor() {
	parentEditor.removeObserver(this);
}



void FogSpecEditor::onObjectSelectionChanged() {
	//std::cout << "FogSpecEditor::onObjectSelectionChanged" << std::endl;
	if (parentEditor.hasCurrentSelection()) {
		AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
		if (obj -> hasFog()) {
			
		    enableSliders();
		
	        //std::cout << "FogSpecEditor about to fetch color" << std::endl;
            ColorSpec* currentlySelectedColor = obj -> getFogSpec() -> getRefToColor();
	        //std::cout << "FogSpecEditor color fetched" << std::endl;
		    suspendUpdates = true;
		    red -> set_value(currentlySelectedColor -> getRedEditValue());
	        green -> set_value(currentlySelectedColor -> getGreenEditValue());
	        blue -> set_value(currentlySelectedColor -> getBlueEditValue());
	        beginOfFogRegion -> set_value(obj -> getFogSpec() -> getStartPosition());
	        endOfFogRegion -> set_value(obj -> getFogSpec() -> getEndPosition());
		    suspendUpdates = false;
		} else {
		    suspendUpdates = true;
		    suspendUpdates = false;
		    disableSliders();
		}
	} else {
		suspendUpdates = true;
		suspendUpdates = false;
		disableSliders();
	}
	
	//std::cout << "FogSpecEditor::onObjectSelectionChanged  finished" << std::endl;
}


//
//  Die nachfoldenden Behandlungsmethoden sind korrekt unter der Voraussetzung,
//  das die zugehörenden Bedienelemente deaktiviert werden, wenn keine Objektauswahl
//  vorliegt oder wenn das ausgewählte Objekt keinen Nebel hat.
//  Diese Voraussetzung erlaubt, auf die Wiederholung der Tests zu
//  verzichten, die in der Methode onObjectSelectionChanged
//  durchgeführt werden.
//



void FogSpecEditor::onRValueChanged() {
	if ((!suspendUpdates)) {
	    AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
        ColorSpec* currentlySelectedColor = obj -> getFogSpec() -> getRefToColor();
        currentlySelectedColor->setRedValue(reglerR.get_value()/255.0f);
        //currentColorDisplayField->updateRedColorComponent(reglerR.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void FogSpecEditor::onGValueChanged() {
	if ((!suspendUpdates)) {
	    AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
        ColorSpec* currentlySelectedColor = obj -> getFogSpec() -> getRefToColor();
        currentlySelectedColor->setGreenValue(reglerG.get_value()/255.0f);
        //currentColorDisplayField->updateGreenColorComponent(reglerG.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}
void FogSpecEditor::onBValueChanged() {
	if ((!suspendUpdates)) {
	    AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
        ColorSpec* currentlySelectedColor = obj -> getFogSpec() -> getRefToColor();
        currentlySelectedColor->setBlueValue(reglerB.get_value()/255.0f);
        //currentColorDisplayField->updateBlueColorComponent(reglerB.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void FogSpecEditor::onBoFValueChanged() {
	if ((!suspendUpdates)) {
	    AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
        //ColorSpec* currentlySelectedColor = obj -> getFogSpec() -> getRefToColor();
        //currentlySelectedColor->setBlueValue(reglerB.get_value()/255.0f);
        //currentColorDisplayField->updateBlueColorComponent(reglerB.get_value()/255.0f);
        obj -> getFogSpec() -> setStartPosition(reglerBoF.get_value());
        sceneModel.notifySceneAttributeChange();
	}
}



void FogSpecEditor::onEoFValueChanged() {
	if ((!suspendUpdates)) {
	    AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
        //ColorSpec* currentlySelectedColor = obj -> getFogSpec() -> getRefToColor();
        //currentlySelectedColor->setGreenValue(reglerG.get_value()/255.0f);
        //currentColorDisplayField->updateGreenColorComponent(reglerG.get_value()/255.0f);
        obj -> getFogSpec() -> setEndPosition(reglerEoF.get_value());
        sceneModel.notifySceneAttributeChange();
	}
}



void FogSpecEditor::enableSliders() {
	container2.set_sensitive(true); 
}

void FogSpecEditor::disableSliders() {
	container2.set_sensitive(false);
}
