#include "LightsEditorStrings.h"
#include <iostream>

LightsEditor::LightsEditor(SceneHolder& sm)
  : sceneModel(sm),
    currentlySelectedLight((LightSourceProperties*) 0),
    currentlySelectedColor((ColorSpec*) 0),
    suspendUpdates(false),
    btn1("1"),
    btn2(Glib::ustring("2")),
    btn3(Glib::ustring("3")),
    btn4(Glib::ustring("4")),
    btn5(Glib::ustring("5")),
    btn6(Glib::ustring("6")),
    btn7(Glib::ustring("7")),
    btn8(Glib::ustring("8")),
    fixedPositionBttn(LightsEditor::LiteralIsFixed), //gehört in die positionBox
    
    ambientLightBttn(LightsEditor::LiteralAmbient),
    diffuseLightBttn(LightsEditor::LiteralDiffuse),
    specularLightBttn(LightsEditor::LiteralSpecular),
    
    red(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    green(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    blue(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    
    constAttenuation(Gtk::Adjustment::create(1.0, 0.0, 101.0, 1.0, 1.0, 1.0)),
    linearAttenuation(Gtk::Adjustment::create(0.0, 0.0, 101.0, 1.0, 1.0, 1.0)),
    quadraticAttenuation(Gtk::Adjustment::create(0.0, 0.0, 101.0, 1.0, 1.0, 1.0)),
    
    reglerR(red, Gtk::ORIENTATION_HORIZONTAL),
    reglerG(green, Gtk::ORIENTATION_HORIZONTAL),
    reglerB(blue, Gtk::ORIENTATION_HORIZONTAL),
    reglerConstAttn(constAttenuation, Gtk::ORIENTATION_HORIZONTAL),
    reglerLinAttn(linearAttenuation, Gtk::ORIENTATION_HORIZONTAL),
    reglerQuadAttn(quadraticAttenuation, Gtk::ORIENTATION_HORIZONTAL),
    
    labelledFrameAttenuation(LightsEditor::LiteralAttenuation),
    container(Gtk::ORIENTATION_VERTICAL),
    lightSelectionBox(Gtk::ORIENTATION_HORIZONTAL),
    lightTypeSelectionBox(Gtk::ORIENTATION_HORIZONTAL),
    sliderBox(Gtk::ORIENTATION_VERTICAL),
    positionBox(Gtk::ORIENTATION_VERTICAL),
    spotlightPropertiesBox(Gtk::ORIENTATION_VERTICAL)
{
	sceneModel.addObserver(this);
	add(container);
	lightSelectionBox.pack_start(btn1, Gtk::PACK_SHRINK);
	lightSelectionBox.pack_start(btn2, Gtk::PACK_SHRINK);
	lightSelectionBox.pack_start(btn3, Gtk::PACK_SHRINK);
	lightSelectionBox.pack_start(btn4, Gtk::PACK_SHRINK);
	lightSelectionBox.pack_start(btn5, Gtk::PACK_SHRINK);
	lightSelectionBox.pack_start(btn6, Gtk::PACK_SHRINK);
	lightSelectionBox.pack_start(btn7, Gtk::PACK_SHRINK);
	lightSelectionBox.pack_start(btn8, Gtk::PACK_SHRINK);
	
	lightTypeSelectionBox.pack_start(ambientLight, Gtk::PACK_SHRINK);
	lightTypeSelectionBox.pack_start(ambientLightBttn);
	lightTypeSelectionBox.pack_start(diffuseLight, Gtk::PACK_SHRINK);
	lightTypeSelectionBox.pack_start(diffuseLightBttn);
	lightTypeSelectionBox.pack_start(specularLight, Gtk::PACK_SHRINK);
	lightTypeSelectionBox.pack_start(specularLightBttn);
	
	ambientLightBttn.set_mode(false);
	diffuseLightBttn.set_mode(false);
	specularLightBttn.set_mode(false);
	
    sliderBox.pack_start(reglerR, Gtk::PACK_SHRINK);
    sliderBox.pack_start(reglerG, Gtk::PACK_SHRINK);
    sliderBox.pack_start(reglerB, Gtk::PACK_SHRINK);
    
    positionBox.pack_start(reglerConstAttn, Gtk::PACK_SHRINK);
    positionBox.pack_start(reglerLinAttn, Gtk::PACK_SHRINK);
    positionBox.pack_start(reglerQuadAttn, Gtk::PACK_SHRINK);
	
	labelledFrameAttenuation.add(positionBox);
    labelledFrameAttenuation.set_border_width(2);
    labelledFrameAttenuation.set_shadow_type(Gtk::SHADOW_ETCHED_OUT);
	
    
	container.pack_start(lightSelectionBox, false, false);
	container.pack_start(fixedPositionBttn, false, false);
	container.pack_start(lightTypeSelectionBox, false, false);
	container.pack_start(sliderBox, false, false);
	container.pack_start(labelledFrameAttenuation, true, true);
	
	Gtk::RadioButton::Group group = noLightSourceSelected.get_group();
	btn1.set_group(group);
	btn2.set_group(group);
	btn3.set_group(group);
	btn4.set_group(group);
	btn5.set_group(group);
	btn6.set_group(group);
	btn7.set_group(group);
	btn8.set_group(group); 
	btn1.set_mode(false);
	btn2.set_mode(false);
	btn3.set_mode(false);
	btn4.set_mode(false);
	btn5.set_mode(false);
	btn6.set_mode(false);
	btn7.set_mode(false);
	btn8.set_mode(false);
	
	Gtk::RadioButton::Group group2 = noSelectionBttn.get_group();
	ambientLightBttn.set_group(group2);
	diffuseLightBttn.set_group(group2);
	specularLightBttn.set_group(group2);
	ambientLightBttn.set_mode(false);
	diffuseLightBttn.set_mode(false);
	specularLightBttn.set_mode(false);
	
	
	reglerR.set_value_pos(Gtk::POS_LEFT);
    reglerG.set_value_pos(Gtk::POS_LEFT);
    reglerB.set_value_pos(Gtk::POS_LEFT);
    reglerR.set_digits(0);
    reglerG.set_digits(0);
    reglerB.set_digits(0);
    
    
	reglerConstAttn.set_value_pos(Gtk::POS_LEFT);
    reglerLinAttn.set_value_pos(Gtk::POS_LEFT);
    reglerQuadAttn.set_value_pos(Gtk::POS_LEFT);
    reglerConstAttn.set_digits(0);
    reglerLinAttn.set_digits(0);
    reglerQuadAttn.set_digits(0);

	noLightSourceSelected.signal_clicked().connect(
         sigc::mem_fun(*this, &LightsEditor::noLightSourceSelectedClicked));	
	btn1.signal_clicked().connect(
         sigc::bind<Gtk::RadioButton*, int>
               (sigc::mem_fun(*this, &LightsEditor::lightSourceClicked),
                &btn1, 0));
	btn2.signal_clicked().connect(
         sigc::bind<Gtk::RadioButton*, int>
               (sigc::mem_fun(*this, &LightsEditor::lightSourceClicked),
                &btn2, 1));
	btn3.signal_clicked().connect(
         sigc::bind<Gtk::RadioButton*, int>
               (sigc::mem_fun(*this, &LightsEditor::lightSourceClicked),
                &btn3, 2));
	btn4.signal_clicked().connect(
         sigc::bind<Gtk::RadioButton*, int>
               (sigc::mem_fun(*this, &LightsEditor::lightSourceClicked),
                &btn4, 3));
	btn5.signal_clicked().connect(
         sigc::bind<Gtk::RadioButton*, int>
               (sigc::mem_fun(*this, &LightsEditor::lightSourceClicked),
                &btn5, 4));
	btn6.signal_clicked().connect(
         sigc::bind<Gtk::RadioButton*, int>
               (sigc::mem_fun(*this, &LightsEditor::lightSourceClicked),
                &btn6, 5));
	btn7.signal_clicked().connect(
         sigc::bind<Gtk::RadioButton*, int>
               (sigc::mem_fun(*this, &LightsEditor::lightSourceClicked),
                &btn7, 6));
	btn8.signal_clicked().connect(
         sigc::bind<Gtk::RadioButton*, int>
               (sigc::mem_fun(*this, &LightsEditor::lightSourceClicked),
                &btn8, 7));
    fixedPositionBttn.signal_toggled().connect(
         sigc::mem_fun(*this, &LightsEditor::fixedPositionBttnClicked));   
              
	noSelectionBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &LightsEditor::noSelectionBttnClicked));   
	ambientLightBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &LightsEditor::ambientLightBttnClicked));
	diffuseLightBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &LightsEditor::diffuseLightBttnClicked));     
	specularLightBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &LightsEditor::specularLightBttnClicked));
    
    red->signal_value_changed().connect(sigc::mem_fun(*this, &LightsEditor::onRValueChanged));
    green->signal_value_changed().connect(sigc::mem_fun(*this, &LightsEditor::onGValueChanged));
    blue->signal_value_changed().connect(sigc::mem_fun(*this, &LightsEditor::onBValueChanged));
    
    constAttenuation->signal_value_changed().connect(sigc::mem_fun(*this, &LightsEditor::onConstAttnChanged));
    linearAttenuation->signal_value_changed().connect(sigc::mem_fun(*this, &LightsEditor::onLinAttnChanged));
    quadraticAttenuation->signal_value_changed().connect(sigc::mem_fun(*this, &LightsEditor::onQuadAttnChanged));

	btn1.show();
	btn2.show();
	btn3.show();
	btn4.show();
	btn5.show();
	btn6.show();
	btn7.show();
	btn8.show();
	container.show();
    if (!sceneModel.hasScene()) {
		btn1.set_sensitive(false);
		btn2.set_sensitive(false);
		btn3.set_sensitive(false);
		btn4.set_sensitive(false);
		btn5.set_sensitive(false);
		btn6.set_sensitive(false);
		btn7.set_sensitive(false);
		btn8.set_sensitive(false);
	}
	labelledFrameAttenuation.set_sensitive(false);
	fixedPositionBttn.set_sensitive(false);
	disableButtons();
	disableSliders();
}

LightsEditor::~LightsEditor() {
	sceneModel.removeObserver(this);
}

bool LightsEditor::hasSelectedColor() {
	return currentlySelectedColor != (ColorSpec*) 0;
}

bool LightsEditor::hasSelectedLight() {
	return currentlySelectedLight != (LightSourceProperties*) 0;
}

void LightsEditor::onSceneChanged() {
	noLightSourceSelected.set_active();
	btn1.set_sensitive(true);
	btn2.set_sensitive(true);
	btn3.set_sensitive(true);
	btn4.set_sensitive(true);
	btn5.set_sensitive(true);
	btn6.set_sensitive(true);
	btn7.set_sensitive(true);
	btn8.set_sensitive(true);

	fixedPositionBttn.set_active(false);
	fixedPositionBttn.set_sensitive(false);

	disableButtons();
	disableSliders();
}

void LightsEditor::noLightSourceSelectedClicked() {
    currentlySelectedLight = (LightSourceProperties*) 0;
	noSelectionBttn.set_active(true); 
	suspendUpdates = true;
	red -> set_value(0);
	green -> set_value(0);
	blue -> set_value(0);
	suspendUpdates = false;
	ambientLight.setColor(0, 0, 0);
	diffuseLight.setColor(0, 0, 0);
	specularLight.setColor(0, 0, 0);
}

/*
 * Diese Methode behandelt die Reaktion auf Auswahl und Abwahl für
 * alle im Fenster angezeigten Buttons für die Auswahl einer Lichtquelle.
 */
void LightsEditor::lightSourceClicked(Gtk::RadioButton* bttn, int idx) {
	if (!bttn -> get_active())
	    return;
    if (sceneModel.hasScene()) {
	    currentlySelectedLight = sceneModel.getScene() -> getRefToLight(idx);
	    if (currentlySelectedLight != (LightSourceProperties*)0) {
	        ambientLight.setColor(currentlySelectedLight -> getPtrToAmbientColor());
	        diffuseLight.setColor(currentlySelectedLight -> getPtrToDiffuseColor());
	        specularLight.setColor(currentlySelectedLight -> getPtrToSpecularColor());
	        fixedPositionBttn.set_active(currentlySelectedLight -> isFixedLight());
	        noSelectionBttn.set_active(true); 
	        fixedPositionBttn.set_sensitive(true);
	        labelledFrameAttenuation.set_sensitive(currentlySelectedLight -> lightType != DIRECTIONAL);
	        // for positional lights and for spotlights
	        if (currentlySelectedLight -> lightType != DIRECTIONAL) {
				// Zugriff auf Umkugel der Szene:
				Sphere sp = sceneModel.getScene() -> enclosingSphere();
				SpacialPoint cp = sp.getCenterPoint();
				float r = sp.getRadius(); 
				float distanceToCenter = (cp - currentlySelectedLight -> direction).euclideanNorm();
				float minDistance = distanceToCenter - r;
				float maxDistance = distanceToCenter + r;
				std::cout << minDistance << "  max Distance  " << maxDistance << std::endl;
				minDistanceFromLightSource = distanceToCenter - r; 
				
				
				constAttenuation -> set_value(currentlySelectedLight -> constantAttenuation*100.0);
				linearAttenuation -> set_value(currentlySelectedLight -> linearAttenuation*100.0*minDistance);
				quadraticAttenuation -> set_value(currentlySelectedLight -> quadraticAttenuation*100.0*(minDistance*minDistance));
				//  Hier geht es mit der Behandlung der Eigenschaften von
				//  Spotlights weiter.	
					
			} else {
				constAttenuation ->set_value(0);
				linearAttenuation ->set_value(0);
				quadraticAttenuation -> set_value(0);
				
			}
	        disableSliders();
	        enableButtons();
	    } else {
	        ambientLight.setColor(0, 0, 0);
	        diffuseLight.setColor(0, 0, 0);
	        specularLight.setColor(0, 0, 0);
	        fixedPositionBttn.set_sensitive(false);
	        // for positional lights and for spotlights
	        
	        disableSliders();
	        disableButtons();
	    }
    }
}

void LightsEditor::noSelectionBttnClicked() {
    currentlySelectedColor = (ColorSpec*) 0;
    currentColorDisplayField = (ColorDisplayField*) 0;
	suspendUpdates = true;
	red -> set_value(0);
	green -> set_value(0);
	blue -> set_value(0);
	suspendUpdates = false;
}

void LightsEditor::ambientLightBttnClicked() {
	if (!ambientLightBttn.get_active())
	    return;
	if (currentlySelectedLight == (LightSourceProperties*) 0) 
	    return;
	currentlySelectedColor = currentlySelectedLight -> getPtrToAmbientColor();
	currentColorDisplayField = &ambientLight;
	suspendUpdates = true;
	red -> set_value(currentlySelectedColor -> getRedEditValue());
	green -> set_value(currentlySelectedColor -> getGreenEditValue());
	blue -> set_value(currentlySelectedColor -> getBlueEditValue());
	suspendUpdates = false;
	fixedPositionBttn.set_sensitive(true);
	enableSliders();
}

void LightsEditor::fixedPositionBttnClicked() {
	if (currentlySelectedLight == (LightSourceProperties*) 0) 
	    return;
	currentlySelectedLight -> isFixedLight(fixedPositionBttn.get_active());
	sceneModel.notifySceneAttributeChange();
}

void LightsEditor::diffuseLightBttnClicked() {
	if (!diffuseLightBttn.get_active())
	    return;
	if (currentlySelectedLight == (LightSourceProperties*) 0) 
	    return;
	currentlySelectedColor = currentlySelectedLight -> getPtrToDiffuseColor();
	currentColorDisplayField = &diffuseLight;
	suspendUpdates = true;
	red -> set_value(currentlySelectedColor -> getRedEditValue());
	green -> set_value(currentlySelectedColor -> getGreenEditValue());
	blue -> set_value(currentlySelectedColor -> getBlueEditValue());
	suspendUpdates = false;
	enableSliders();
}

void LightsEditor::specularLightBttnClicked() {
	if (!specularLightBttn.get_active())
	    return;
	if (currentlySelectedLight == (LightSourceProperties*) 0) 
	    return;
	currentlySelectedColor = currentlySelectedLight -> getPtrToSpecularColor();
	currentColorDisplayField = &specularLight;
	suspendUpdates = true;
	red -> set_value(currentlySelectedColor -> getRedEditValue());
	green -> set_value(currentlySelectedColor -> getGreenEditValue());
	blue -> set_value(currentlySelectedColor -> getBlueEditValue());
	suspendUpdates = false;
	enableSliders();
}


void LightsEditor::onRValueChanged() {
	if (hasSelectedColor() &&(!suspendUpdates)) {
        currentlySelectedColor->setRedValue(reglerR.get_value()/255.0f);
        currentColorDisplayField->updateRedColorComponent(reglerR.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void LightsEditor::onGValueChanged() {
	if (hasSelectedColor() &&(!suspendUpdates)) {
        currentlySelectedColor->setGreenValue(reglerG.get_value()/255.0f);
        currentColorDisplayField->updateGreenColorComponent(reglerG.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void LightsEditor::onBValueChanged() {
	if (hasSelectedColor() &&(!suspendUpdates)) {
        currentlySelectedColor->setBlueValue(reglerB.get_value()/255.0f);
        currentColorDisplayField->updateBlueColorComponent(reglerB.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}


void LightsEditor::onConstAttnChanged() {
	if (hasSelectedLight() &&(!suspendUpdates)) {
        currentlySelectedLight ->
          setConstantAttenuation(reglerConstAttn.get_value()/100.0f);
        sceneModel.notifySceneAttributeChange();
	}
}


void LightsEditor::onLinAttnChanged() {
	if (hasSelectedLight() &&(!suspendUpdates)) {
		float d = minDistanceFromLightSource * 100.0f;
        currentlySelectedLight ->
          setLinearAttenuation(reglerLinAttn.get_value()/d);
        sceneModel.notifySceneAttributeChange();
	}
}


void LightsEditor::onQuadAttnChanged() {
	if (hasSelectedLight() &&(!suspendUpdates)) {
		float d = minDistanceFromLightSource * minDistanceFromLightSource * 100.0f;
        currentlySelectedLight ->
          setQuadraticAttenuation(reglerQuadAttn.get_value()/d);
        sceneModel.notifySceneAttributeChange();
	}
}

void LightsEditor::enableButtons() {
	ambientLightBttn.set_sensitive(true);
	diffuseLightBttn.set_sensitive(true);
	specularLightBttn.set_sensitive(true);
}

void LightsEditor::disableButtons() {
	ambientLightBttn.set_sensitive(false);
	diffuseLightBttn.set_sensitive(false);
	specularLightBttn.set_sensitive(false);
}

void LightsEditor::enableSliders() {
	reglerR.set_sensitive(true);
	reglerG.set_sensitive(true);
	reglerB.set_sensitive(true); /*
	reglerConstAttn.set_sensitive(true);
	reglerLinAttn.set_sensitive(true);
	reglerQuadAttn.set_sensitive(true); */
}

void LightsEditor::disableSliders() {
	reglerR.set_sensitive(false);
	reglerG.set_sensitive(false);
	reglerB.set_sensitive(false); /*
	reglerConstAttn.set_sensitive(false);
	reglerLinAttn.set_sensitive(false);
	reglerQuadAttn.set_sensitive(false); */
}
