#include "MainWindowStrings.h"
#include <iostream>

#include "AbstractProjectionSpec.h"
#include "AbstractExample.h"
#include "Scene.h"


MainWindow::MainWindow(Glib::RefPtr<Gtk::Application> app, ApplicationModel& m) :
   adjustmentX(Gtk::Adjustment::create(0.0, 0.0, 361.0, 1.0, 10.0, 0.0)),
   adjustmentY(Gtk::Adjustment::create(0.0, 0.0, 361.0, 1.0, 10.0, 0.0)),
   adjustmentZ(Gtk::Adjustment::create(0.0, 0.0, 361.0, 1.0, 10.0, 0.0)),
   reglerX(adjustmentX,  Gtk::ORIENTATION_VERTICAL),
   reglerY(adjustmentY,  Gtk::ORIENTATION_VERTICAL),
   reglerZ(adjustmentZ,  Gtk::ORIENTATION_VERTICAL),
   vbox(Gtk::ORIENTATION_VERTICAL),
   grid(1, 3, false),
   paintArea(m.getSceneHolder()),
   model(m),
   suspendUpdates(false)
 { 
   Gtk::RadioButtonGroup group;
   
   set_title(MainWindow::windowTitle);
   set_position(Gtk::WIN_POS_CENTER);
   model.getSceneHolder().addObserver(this);
   set_reallocate_redraws(true); 
   set_default_size(580, 520);
   grid.set_border_width(10);
   reglerX.set_value_pos(Gtk::POS_LEFT);
   reglerX.set_inverted();
   reglerX.set_digits(0);
   reglerX.set_draw_value(false);
   reglerY.set_value_pos(Gtk::POS_LEFT);
   reglerY.set_inverted();
   reglerY.set_digits(0);
   reglerY.set_draw_value(false);
   reglerZ.set_value_pos(Gtk::POS_LEFT);
   reglerZ.set_inverted();
   reglerZ.set_digits(0);
   reglerZ.set_draw_value(false);
   
   //std::cout << "Observer count " << model.getSceneHolder().observerCount() << endl; 
   
   grid.attach(reglerX, 0, 1, 0, 1, Gtk::FILL, Gtk::FILL | Gtk::EXPAND);
   grid.attach(reglerY, 1, 2, 0, 1, Gtk::FILL, Gtk::FILL | Gtk::EXPAND);
   grid.attach(reglerZ, 2, 3, 0, 1, Gtk::FILL, Gtk::FILL | Gtk::EXPAND);
   grid.attach(paintArea, 3, 4, 0, 1); 
   
   signal_delete_event().
	     connect(sigc::mem_fun(*this, &MainWindow::onDeleteEvent));
   adjustmentX->signal_value_changed().connect(sigc::mem_fun(*this, &MainWindow::onXValueChanged));
   adjustmentY->signal_value_changed().connect(sigc::mem_fun(*this, &MainWindow::onYValueChanged));
   adjustmentZ->signal_value_changed().connect(sigc::mem_fun(*this, &MainWindow::onZValueChanged));
   

   Gtk::MenuItem* menuBar1;
   menuBar1 = Gtk::manage(new Gtk::MenuItem(MainWindow::sceneMenueTitle)); 
   Gtk::Menu* sceneMenu;
   sceneMenu = Gtk::manage(new Gtk::Menu);
   Gtk::RadioMenuItem* option;
   hiddenOption = new Gtk::RadioMenuItem(group, "--");
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem01));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                            (sigc::mem_fun(model, &ApplicationModel::onSelectItem01),
                             option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem02));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem01a),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem03));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem02),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem04));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem02a),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem05));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem02b),
                              option));
   sceneMenu->append(*option);
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem06));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem03),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem07));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem04),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem08));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem05),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem09));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem06),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem10));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem07),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem11));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem09),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem12));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem10),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem13));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem11),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem14));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem12),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem15));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem13),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem16));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem14),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem17));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem15),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem18));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem16),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem19));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem17),
                              option));
   sceneMenu->append(*option);
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem20));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem20),
                              option));
   sceneMenu->append(*option);
   
   
   option = Gtk::manage(new Gtk::RadioMenuItem(group, MainWindow::sceneMenueItem21));
   option->set_draw_as_radio(false);
   option->signal_activate().
           connect(sigc::bind<Gtk::RadioMenuItem*>
                             (sigc::mem_fun(model, &ApplicationModel::onSelectItem21),
                              option));
   sceneMenu->append(*option);
   menuBar1->set_submenu(*sceneMenu);
   menuBar.append(*menuBar1);
   
   Gtk::MenuItem* menuBar2;
   menuBar2 = Gtk::manage(new Gtk::MenuItem(MainWindow::settingsMenueTitle));
   sceneMenu = Gtk::manage(new Gtk::Menu);
   Gtk::MenuItem* option2 = Gtk::manage(new Gtk::MenuItem(MainWindow::settingsMenueItem01));
   option2->signal_activate().
            connect(sigc::mem_fun(model, &ApplicationModel::onSelectToggleControlPanel));
   sceneMenu->append(*option2);
   menuBar2->set_submenu(*sceneMenu);
   menuBar.append(*menuBar2); 
   
   add(vbox);
   vbox.pack_start(menuBar, Gtk::PACK_SHRINK, 0);
   vbox.pack_start(grid);
   vbox.show();
   menuBar1->show_all();
   menuBar2->show_all();
   menuBar.show();
   reglerX.show();
   reglerY.show();
   reglerZ.show();
   paintArea.show();
   
   grid.show();
   show();
}

void MainWindow::onSceneChanged() {
    //cout << "onSceneChanged received" << endl;
    //reinitialize Sliders:
    suspendUpdates = true;
    adjustmentX->set_value(0.0);
    adjustmentY->set_value(0.0);
    adjustmentZ->set_value(0.0);
    suspendUpdates = false;
    paintArea.refresh();  
}



bool MainWindow::onDeleteEvent(GdkEventAny* event) {
	model.aboutToCloseMainWindow();
	return false;  // Weiterbehandlung zulassen
};


MainWindow::~MainWindow() {
	model.getSceneHolder().removeObserver(this);
	delete hiddenOption;
}

void MainWindow::onXValueChanged() {
	if (!suspendUpdates) {
        paintArea.rotateX(adjustmentX->get_value());
	}
}

void MainWindow::onYValueChanged() {
	if (!suspendUpdates) {
        paintArea.rotateY(adjustmentY->get_value());
	}
}

void MainWindow::onZValueChanged() {
	if (!suspendUpdates) {
        paintArea.rotateZ(adjustmentZ->get_value());
	}
}

