#include "MaterialPropertiesEditor.h"
#include "MaterialPropertiesEditorStrings.h"
#include "SceneObjectsEditor.h"
#include "MaterialColoring.h"

MaterialPropertiesEditor::MaterialPropertiesEditor(SceneHolder& sm,
                                                   SceneObjectsEditor& se)
  : SimpleMaterialPropertiesEditor(sm, se),
    frontFaceBttn(MaterialPropertiesEditor::LiteralFront),
    backFaceBttn(MaterialPropertiesEditor::LiteralBack),
    bttnContainer(Gtk::ORIENTATION_HORIZONTAL)
{
}

MaterialPropertiesEditor::~MaterialPropertiesEditor() {
}


MaterialPropertiesEditor* MaterialPropertiesEditor::create(SceneHolder& sm, SceneObjectsEditor& se)
{
	MaterialPropertiesEditor* instance = new MaterialPropertiesEditor(sm, se);
	instance -> configureComponents();
	instance -> buildForm();
	return instance;
}

void MaterialPropertiesEditor::configureComponents() {
	Gtk::RadioButton::Group group = noFaceBttn.get_group();
	frontFaceBttn.set_group(group);
	backFaceBttn.set_group(group);
	
	frontFaceBttn.set_mode(false);
	backFaceBttn.set_mode(false);
	noFaceBttn.set_active(true);
	
	frontFaceBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &MaterialPropertiesEditor::frontFaceBttnClicked));
	backFaceBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &MaterialPropertiesEditor::backFaceBttnClicked));
         
	SimpleMaterialPropertiesEditor::configureComponents();
}

void MaterialPropertiesEditor::buildForm() {
	bttnContainer.pack_start(frontFaceBttn);
	bttnContainer.pack_start(backFaceBttn);
	container1.pack_start(ambientColor, Gtk::PACK_SHRINK);
	container1.pack_start(ambientColorBttn);
	container1.pack_start(diffuseColor, Gtk::PACK_SHRINK);
	container1.pack_start(diffuseColorBttn);
	container1.pack_start(specularColor, Gtk::PACK_SHRINK);
	container1.pack_start(specularColorBttn);
	container1.pack_start(emissiveColor, Gtk::PACK_SHRINK);
	container1.pack_start(emissiveColorBttn);
    container2.pack_start(reglerR, Gtk::PACK_SHRINK);
    container2.pack_start(reglerG, Gtk::PACK_SHRINK);
    container2.pack_start(reglerB, Gtk::PACK_SHRINK);
    container.pack_start(bttnContainer, false, false);
    container.pack_start(reglerShininess);
    container.pack_start(container1, false, false);
    container.pack_start(container2, true, true);
	reglerR.show();
	reglerG.show();
	container2.show();
	container1.show();
	container.show();
}

void MaterialPropertiesEditor::loadSceneObjectData() {
	AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
	AbstractMaterial* mat = obj -> getMaterial();
	currentMaterialSpec =  (MaterialProperties*) 0;
	noFaceBttn.set_active( true);
}

void MaterialPropertiesEditor::noFaceBttnClicked() {
	//noFaceBttn.set_active(true);
}

void MaterialPropertiesEditor::frontFaceBttnClicked() {
	if (!frontFaceBttn.get_active())
	    return;
	noSelectionBttn.set_active(true);
	if (parentEditor.hasCurrentSelection()) {
	    AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
	    AbstractMaterial* mat = obj -> getMaterial();
        MaterialColoring* mat2 = (MaterialColoring*)mat;
	    currentMaterialSpec =  mat2 -> getFrontColoring();
	    enableSecondSelection();
	    updateVisualSettings();
	}
}

void MaterialPropertiesEditor::backFaceBttnClicked() {
	if (!backFaceBttn.get_active())
	    return;
	noSelectionBttn.set_active(true);
	if (parentEditor.hasCurrentSelection()) {
	    AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
	    AbstractMaterial* mat = obj -> getMaterial();
        MaterialColoring* mat2 = (MaterialColoring*)mat;
	    currentMaterialSpec =  mat2 -> getBackColoring();
	    enableSecondSelection();
	    updateVisualSettings();
	}
}

void MaterialPropertiesEditor::enableFirstSelection() {
	frontFaceBttn.set_sensitive(true);
	backFaceBttn.set_sensitive(true);
}

void MaterialPropertiesEditor::disableFirstSelection() {
	SimpleMaterialPropertiesEditor::disableFirstSelection();
	frontFaceBttn.set_sensitive(false);
	backFaceBttn.set_sensitive(false);
}

void MaterialPropertiesEditor::enableSecondSelection() {
	SimpleMaterialPropertiesEditor::enableFirstSelection();
}

void MaterialPropertiesEditor::disableSecondSelection() {
	SimpleMaterialPropertiesEditor::disableFirstSelection();
}
