#include "MilneRule.h"

#include <math.h>

MilneRule::MilneRule(float start, float end, float e, int n) :
    Integrator (start, end, e, n)
{
}

float MilneRule::computeValue(ArcLengthIntegrand* integrand) {
    const float factors [] = { 32.0f, 12.0f, 14.0f, 32.0f, 32.0f, 14.0f, 12.0f, 32.0f };
    const float distances [] = { 1.0f, 2.0f, 4.0f, 5.0f, 7.0f, 8.0f, 10.0f, 11.0f };
    const int nroOfPoints = 8;
    float interval = (tEnd - tStart)/4.0f;
    float xi = tStart + interval;
    float xj = xi + interval;
    float xk = tEnd - interval;
    
    float area = (integrand->fn(xj)*12.0f
                  + (integrand->fn(xi) + integrand->fn(xk)) *32.0f
                  + (integrand->fn(tStart) + integrand->fn(tEnd))*7.0f
                 )*(tEnd - tStart)/90.0f;
    
    int n = 1;
    bool stop = false;
    
    while (!stop) {
        float oldArea = area;
        float sum = 0.0f;
        float delta = interval / 3.0f;
        for (int i = 0; i < n; i++) {
            float localStart = i*interval * 4 + tStart;
            float s = 0.0f;
            for (int j = 0; j < nroOfPoints; j++) {
                s += integrand->fn (localStart + distances[j]*delta) * factors[j];
            }
            
            sum += s;
        }
        n *= 3;
        area = area / 3.0f + sum*(tEnd - tStart) /(90.0f*n);
        interval /= 3.0f;
        stop = n > maxN || (fabs(area - oldArea) < fabs(area)*eps) ;
    }
    return area;
}
