#include "NewtonRule.h"

#include <math.h>

NewtonRule::NewtonRule(float start, float end, float e, int n) :
    Integrator (start, end, e, n)
{
}

float NewtonRule::computeValue(ArcLengthIntegrand* integrand) {
    float interval = (tEnd - tStart)/3.0f;
    float xi = tStart + interval;
    float xj;
    float xk = tEnd - interval;
    float area = ((integrand->fn(xi) + integrand->fn(xk)) *3.0f  + integrand->fn(tStart) + integrand->fn(tEnd))*(tEnd - tStart)/8.0f;
    
    int n = 1;
    bool stop = false;
    
    while (!stop) {
        float oldArea = area;
        float sum = 0.0F;
        for (int i = 0; i < n; i++) {
            float localStart = i*interval * 3 + tStart;
            xi = interval/2.0f + localStart;
            xj = interval * 1.5f + localStart;
            xk = interval * 2.5 + localStart;
            sum += integrand->fn(xj)*2.0f + (integrand->fn(xi) + integrand->fn(xk)) * 3.0f;
        }
        n *= 2;
        area = area / 2.0F + sum*(tEnd - tStart) /(8.0f*n);
        interval /= 2.0f;
        stop = n > maxN || (fabs(area - oldArea) < fabs(area)*eps) ;
    }
    return area;
}
