#include "PerspectiveProjectionSpec.h"
#include "Scene.h"
#include "ViewBox.h"
#include <math.h>

#define GLM_FORCE_RADIANS
#include <glm/gtc/matrix_transform.hpp>

PerspectiveProjectionSpec::~PerspectiveProjectionSpec() {
}

void PerspectiveProjectionSpec::setViewpoint(Scene* sc) {
    SpacialPoint cp = sc -> getCenterPoint();
    CameraSettings* cs = sc -> getCameraSettings();

    glm::mat4 view =
            glm::lookAt(
                cs -> position.asVec3(),     //  Position der Kamera
                cs -> target.asVec3(),       //  target
                cs -> upDirection.asVec3()); //  up-vector

    glMultMatrixf(glm::value_ptr(view));
}


//   frei programmierte Version
void PerspectiveProjectionSpec::setSpace(Scene* sc) {
    Sphere sp = sc -> enclosingSphere();
    CameraSettings* cs = sc -> getCameraSettings();
    SpacialPoint ep = cs -> position;
    SpacialPoint center = sp.getCenterPoint();
    float radius = sp.getRadius();
    float halfDistance = radius*zoomFactor;
    float distanceFromOrigin = (ep - center).euclideanNorm();
    float tanAlpha = halfDistance/sqrt(distanceFromOrigin*distanceFromOrigin - halfDistance*halfDistance);
    float nearZ = distanceFromOrigin - radius;
    float farZ = distanceFromOrigin + radius;
    float aspect = 1.0f;
    mat[0] = 1.0f/(aspect*tanAlpha);
    mat[1] = 0.0f;
    mat[2] = 0.0f;
    mat[3] = 0.0f;
    mat[4] = 0.0f;
    mat[5] = 1.0f/tanAlpha;   //  Aufrichtefunktion
    mat[6] = 0.0f;
    mat[7] = 0.0f;
    mat[7] = 0.0f;
    mat[9] = 0.0f;
    mat[10] = (nearZ + farZ)/(nearZ - farZ);
    mat[11] = -1.0f;
    mat[12] = 0.0f;
    mat[13] = 0.0f;
    mat[14] = 2.0f*farZ*nearZ/(nearZ - farZ);
    mat[15] = 0.0f;

    glFrontFace(GL_CCW);  //  erforderlich, falls die Transformation die
                         //  Bildaufrichtung leisten soll, ohne dass
                         //  Vorderseitenfarbe und Rückseitenfarbe
                         //  vertauscht werden.
                         //  Die Bildaufrichtung in der Transformation
                         //  ausführen zu lassen, ermöglicht eine
                         //  raschere Bildabfolge.
    glLoadMatrixf(mat);

}

/*

//   Version mit glm::perspective
void PerspectiveProjectionSpec::setSpace(Scene* sc) {
    Sphere sp = sc -> enclosingSphere();
    CameraSettings* cs = sc -> getCameraSettings();
    SpacialPoint ep = cs -> position;
    SpacialPoint center = sp.getCenterPoint();
    float radius = sp.getRadius();
    float distanceFromOrigin = (ep - center).euclideanNorm();
    float tanAlpha = radius/sqrt(distanceFromOrigin*distanceFromOrigin - radius*radius);
    float alpha = atanf(tanAlpha);
    glm::mat4 p = glm::perspective(2.0f*alpha, 1.0f,
                                   distanceFromOrigin - radius,
                                   distanceFromOrigin + radius);
    glLoadMatrixf(glm::value_ptr(p));

}
*/


/*
//   ---- old stuff    (this is creadibly wrong!)  ---
void PerspectiveProjectionSpec::setSpace(Scene* sc) {
    Sphere sp = sc -> enclosingSphere();
    CameraSettings* cs = sc -> getCameraSettings();
    SpacialPoint ep = cs -> position;
    ViewBox cube = ViewBox(SpacialPoint(0.0f, 0.0f, 0.0f));
    SpacialPoint center = sp.getCenterPoint();
    float radius = sp.getRadius();
    float distanceFromOrigin = (ep - center).euclideanNorm();
    float ff = radius;
    cube.setXRange(- ff, ff);
    cube.setYRange( - ff, ff);
    cube.setZRange(distanceFromOrigin -radius, distanceFromOrigin + radius);

    glFrustumf(cube.xMin, cube.xMax,          //  left, right
               cube.yMin, cube.yMax,          //  bottom, top
               cube.zMin,                     // near
               cube.zMax);                    // far
}
*/

