#include "RotationMatrix.h"

RotationMatrix::RotationMatrix() {
}

RotationMatrix::RotationMatrix(float angle, SpacialPoint axis, SpacialPoint centerPt){
	a11 = 1.0f;
	a12 = 0.0f;
	a13 = 0.0f;
	a14 = 0.0f;
	a21 = 0.0f;
	a22 = 1.0f;
	a23 = 0.0f;
	a24 = 0.0f;
	a31 = 0.0f;
	a32 = 0.0f;
	a33 = 1.0f;
	a34 = 0.0f;
	a41 = 0.0f;
	a42 = 0.0f;
	a43 = 0.0f;
	a44 = 1.0f;
	
}		

void RotationMatrix::setTranslation (SpacialPoint sp) {
	a14 = sp.x;
	a24 = sp.y;
	a34 = sp.z;
}

RotationMatrix operator * (RotationMatrix m1, RotationMatrix m2) {
	RotationMatrix result;
	result.a11 = m1.a11*m2.a11 + m1.a12*m2.a21 + m1.a13*m2.a31 + m1.a14*m2.a41;
	result.a12 = m1.a11*m2.a12 + m1.a12*m2.a22 + m1.a13*m2.a32 + m1.a14*m2.a42;
	result.a13 = m1.a11*m2.a13 + m1.a12*m2.a23 + m1.a13*m2.a33 + m1.a14*m2.a43;
	result.a14 = m1.a11*m2.a14 + m1.a12*m2.a24 + m1.a13*m2.a34 + m1.a14*m2.a44;
	
	result.a21 = m1.a21*m2.a11 + m1.a22*m2.a21 + m1.a23*m2.a31 + m1.a24*m2.a41;
	result.a22 = m1.a21*m2.a12 + m1.a22*m2.a22 + m1.a23*m2.a32 + m1.a24*m2.a42;
	result.a23 = m1.a21*m2.a13 + m1.a22*m2.a23 + m1.a23*m2.a33 + m1.a24*m2.a43;
	result.a24 = m1.a21*m2.a14 + m1.a22*m2.a24 + m1.a23*m2.a34 + m1.a24*m2.a44;
	
	result.a31 = m1.a31*m2.a11 + m1.a32*m2.a21 + m1.a33*m2.a31 + m1.a34*m2.a41;
	result.a32 = m1.a31*m2.a12 + m1.a32*m2.a22 + m1.a33*m2.a32 + m1.a34*m2.a42;
	result.a33 = m1.a31*m2.a13 + m1.a32*m2.a23 + m1.a33*m2.a33 + m1.a34*m2.a43;
	result.a34 = m1.a31*m2.a14 + m1.a32*m2.a24 + m1.a33*m2.a34 + m1.a34*m2.a44;
	
	result.a41 = m1.a41*m2.a11 + m1.a42*m2.a21 + m1.a43*m2.a31 + m1.a44*m2.a41;
	result.a42 = m1.a41*m2.a12 + m1.a42*m2.a22 + m1.a43*m2.a32 + m1.a44*m2.a42;
	result.a43 = m1.a41*m2.a13 + m1.a42*m2.a23 + m1.a43*m2.a33 + m1.a44*m2.a43;
	result.a44 = m1.a41*m2.a14 + m1.a42*m2.a24 + m1.a43*m2.a34 + m1.a44*m2.a44;
	
	return  result;
}

SpacialPoint RotationMatrix::localToGlobal (SpacialPoint sp) {
	float rx = sp.x*a11 + sp.y*a12 + sp.z*a13 + a14;
	float ry = sp.x*a21 + sp.y*a22 + sp.z*a23 + a24;
	float rz = sp.x*a31 + sp.y*a32 + sp.z*a33 + a34;
	float rw = sp.x*a41 + sp.y*a42 + sp.z*a43 + a44;
	return SpacialPoint (rx/rw, ry/rw, rz/rw);
}
