#include "SceneObjectsEditor.h"
#include "SceneObjectsEditorStrings.h"
#include <iostream>

#ifdef TRACK_DESTROY_EVENTS

void* notify1(void* data) {
	std::cout <<  "Liststore1 about to be destroyed" << std::endl;
	return (void*) 0;
}

void* notify2(void* data) {
	std::cout <<  "Liststore2 about to be destroyed" << std::endl;
	return (void*) 0;
}

void* notifyFogEditor(void* data) {
	std::cout <<  "FogEditor about to be destroyed" << std::endl;
	return (void*) 0;
}

void* notifyEmptyPage(void* data) {
	std::cout <<  "EmptyPage about to be destroyed" << std::endl;
	return (void*) 0;
}
#endif


SceneObjectsEditor::SceneObjectsEditor(SceneHolder& sm)
  : sceneModel(sm),
    noSelectionBttn(""),
    box3(Gtk::ORIENTATION_HORIZONTAL, 0),
    listContainer(Gtk::ORIENTATION_VERTICAL, 2),
    container(Gtk::ORIENTATION_VERTICAL, 0),
    selectMaterialEditor(SceneObjectsEditor::LiteralMaterial),
    selectTextureEditor(SceneObjectsEditor::LiteralTexture),
    selectFogEditor(SceneObjectsEditor::LiteralFog),
    editors()
{
	sceneModel.addObserver(this);
	currentlySelectedSceneObject = (AbstractSceneObject*) 0;
	for (int i = 0; i <Scene::NUMBER_OF_LIGHTS; i++) {
		adapters[i].setPositionIndex(i,this);
	}
	
	refSceneObjects = Gtk::ListStore::create(objectList);
    sceneObjectsList.set_model(refSceneObjects);
    
#ifdef TRACK_DESTROY_EVENTS
    refSceneObjects -> add_destroy_notify_callback
      (this,  notify1);
#endif
    
    editors.set_show_tabs(false);
    
	Gtk::RadioButton::Group group = noSelectionBttn.get_group();
    selectMaterialEditor.set_group(group);
	selectTextureEditor.set_group(group);
	selectFogEditor.set_group(group);
    selectMaterialEditor.set_mode(false);
    selectTextureEditor.set_mode(false);
    selectFogEditor.set_mode(false);
	noSelectionBttn.set_active(true);
    
    emptyPage = Gtk::manage(new NothingSelectedHint());
    textureEditor= new TexturePropertiesEditor(sceneModel, *this);
    materialEditor = MaterialPropertiesEditor::create(sceneModel, *this);
    simpleMaterialEditor = SimpleMaterialPropertiesEditor::create(sceneModel, *this);
    fogEditor = Gtk::manage(new FogSpecEditor(sceneModel, *this));
    
#ifdef TRACK_DESTROY_EVENTS     
    emptyPage -> add_destroy_notify_callback(this, notifyEmptyPage);      
    fogEditor -> add_destroy_notify_callback(this, notifyFogEditor);
#endif
    
    materialEditors.append_page(*simpleMaterialEditor, "");
    materialEditors.append_page(*materialEditor, "");
    materialEditors.set_show_tabs(false);
    materialEditors.set_show_border(false);
    
    editors.append_page(*emptyPage, "");
    editors.append_page(materialEditors, SceneObjectsEditor::LiteralMaterial);
    editors.append_page(*textureEditor, SceneObjectsEditor::LiteralTexture);
    editors.append_page(*fogEditor, SceneObjectsEditor::LiteralFog);
    
    Glib::RefPtr<Gtk::TreeView::Selection> selection;
    selection = sceneObjectsList.get_selection();
    selection->set_mode(Gtk::SELECTION_SINGLE);
    
	if (sceneModel.hasScene()) {
		this -> fillListstore();
	}
    
    sceneObjectsList.set_grid_lines(Gtk::TREE_VIEW_GRID_LINES_VERTICAL);
    scSceneObjectList.add(sceneObjectsList);
    scSceneObjectList.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    scSceneObjectList.set_size_request(-1, 110);  //  set a minimal height, but no minimal width
    scSceneObjectList.set_shadow_type(Gtk::SHADOW_IN);
    sceneObjectsList.append_column(SceneObjectsEditor::LiteralObjects, objectList.stringColumn);
    sceneObjectsList.append_column(SceneObjectsEditor::LiteralShow, objectList.showObject);
    sceneObjectsList.append_column(/*SceneObjectsEditor::LiteralShow */ "ds", objectList.useLightModelTwoSide);
    sceneObjectsList.append_column(/*SceneObjectsEditor::LiteralShow*/ "wire", objectList.showWireframe);
    sceneObjectsList.append_column(/*SceneObjectsEditor::LiteralShow */ "tex.", objectList.showTexture);
    sceneObjectsList.append_column(/*SceneObjectsEditor::LiteralShow */ "fog", objectList.showFog);
    Gtk::TreeView::Column *lightsColumn = Gtk::manage(new Gtk::TreeView::Column("Lights"));
    for (int i = 0; i < Scene::NUMBER_OF_LIGHTS; i++) {    /*
		Gtk::CellRendererToggle* renderer =
		  Gtk::manage(new Gtk::CellRendererToggle()); */
        lightsColumn-> pack_start(objectList.useLight[i], false);
       // renderer ->set_activatable(true);
        // signal hier
	}
	sceneObjectsList.append_column(*lightsColumn);
    
    /* Breite der Spalten festlegen: Auswahlspalten sind fix,
     * nur die erste Spalte (Objektname) kann sich der Widgetgröße
     * anpassen  */  
    sceneObjectsList.get_column(0) -> set_expand(true);
    sceneObjectsList.get_column(6) -> set_expand(false);
    
    Gtk::CellRendererToggle*  renderer [5];
    
    for (int i = 1; i <= 5; i++) {
		sceneObjectsList.get_column(i) -> set_expand(false);
		Gtk::CellRendererToggle* cr 
         = (Gtk::CellRendererToggle*) sceneObjectsList.get_column_cell_renderer(i);
        cr -> set_activatable(true);
        renderer[i - 1] = cr;
	}
    
    std::vector<Gtk::CellRenderer*> renderers = lightsColumn -> get_cells();
    for (int i = 0; i < Scene::NUMBER_OF_LIGHTS; i++) {
		Gtk::CellRendererToggle* renderer
            = (Gtk::CellRendererToggle*) renderers.at(i);
        renderer -> set_activatable(true); 
        renderer -> signal_toggled().connect(
                    sigc::mem_fun(adapters[i], &Adapter::onToggleClicked)
                   ); 
	}
    Gdk::RGBA columnColor;
    Glib::PropertyProxy<Gdk::RGBA> cp = renderer[3]-> property_cell_background_rgba();
    columnColor.set_rgba(0.9, 0.9, 0.6, 1.0);
    cp.set_value(columnColor);
    Glib::PropertyProxy<Gdk::RGBA> cp2 = renderer[2] -> property_cell_background_rgba();
    columnColor.set_rgba(0.6, 0.9, 0.6, 1.0);
    cp2.set_value(columnColor);
    
    sceneObjectsList.get_selection() -> signal_changed().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onSelectionChanged)); 

    selectMaterialEditor.signal_clicked().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onMaterialButtonClicked));
    selectTextureEditor.signal_clicked().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onTextureButtonClicked));
    selectFogEditor.signal_clicked().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onFogButtonClicked));
    renderer[0] -> signal_toggled().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onShowObjectToggleClicked));
    renderer[1] -> signal_toggled().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onLightModelToggleClicked));
    renderer[2] -> signal_toggled().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onShowWireframeToggleClicked));
    renderer[3] -> signal_toggled().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onShowTextureToggleClicked));
    renderer[4] -> signal_toggled().connect(
         sigc::mem_fun(*this, &SceneObjectsEditor::onShowFogToggleClicked));
         
    container.set_border_width(5);
    add(container);
   
    listContainer.pack_start(scSceneObjectList, false, true, 0);
    box3.pack_start(selectMaterialEditor);
    box3.pack_start(selectTextureEditor);
    box3.pack_start(selectFogEditor);
    container.pack_start(listContainer, false, true, 0);

    container.pack_start(box3, false, true, 0);
    container.pack_start(editors, false, true, 0);
    sceneObjectsList.show();
    scSceneObjectList.show();
    listContainer.show();
    container.show();
    show();
    disablePropertyEditors();
}

SceneObjectsEditor::~SceneObjectsEditor() {
	sceneModel.removeObserver(this);
	delete simpleMaterialEditor;
	delete materialEditor;
	delete textureEditor;
	//delete fogEditor;  //  wird gebraucht, wenn Gtk::manage nicht verwendet wird 
}



void SceneObjectsEditor::addObserver(ObjectSelectionObserver* obs) {
    observer.push_back(obs);
}

void SceneObjectsEditor::removeObserver(ObjectSelectionObserver* obs) {
    std::list<ObjectSelectionObserver*>::iterator it = observer.begin();
    while (it != observer.end()) {
        if ((*it) == obs) {
	        observer.erase(it); 
			it++;
			break;
	    } else {
			++it;
		}
    }
}

void SceneObjectsEditor::notifyChange() {
    for (std::list<ObjectSelectionObserver*>::iterator it = observer.begin();
         it != observer.end();
         ++it)
    {
        (*it) -> onObjectSelectionChanged();
    }
}

bool SceneObjectsEditor::hasCurrentSelection() {
	return currentlySelectedSceneObject != (AbstractSceneObject*) 0;
}

AbstractSceneObject* SceneObjectsEditor::getCurrentlySelectedObject() {
	return currentlySelectedSceneObject;
}

void SceneObjectsEditor::fillListstore() {
	std::list<AbstractSceneObject*>& list = sceneModel.getScene()->getObjectList();

    int cnt = 0;
    for (std::list<AbstractSceneObject*>::iterator Iterator = list.begin();
         Iterator != list.end();
         ++Iterator)
    {
		Gtk::TreeModel::iterator iter = refSceneObjects -> append();
		Gtk::TreeModel::Row r = *iter;
		r[objectList.stringColumn] = Glib::ustring((*Iterator) -> getName());
		r[objectList.indexColumn] = cnt;
		r[objectList.showObject] = (*Iterator) -> isVisible();
		r[objectList.useLightModelTwoSide] = (*Iterator) -> useLightModelTwoSide();
		r[objectList.showWireframe] = (*Iterator) ->showWireframe();
		r[objectList.showTexture] = (*Iterator) -> showTexture();
		r[objectList.showFog] = (*Iterator) -> showFog();
		for (int i = 0; i < Scene::NUMBER_OF_LIGHTS; i++) {
			r[objectList.useLight[i]] = (*Iterator) -> getLightSettings() [i];
		}
		cnt ++;
	}
}

void SceneObjectsEditor::onSceneChanged() {
	//std::cout << "---------  onSceneChanged entered   ---------" << std::endl;
	currentlySelectedSceneObject = (AbstractSceneObject*) 0;
	//  Bevor der ListStore gelöscht werden kann, müssen alle
    //  Auswahlen aufgehoben werden. Wenn das vergessen wird,
	//  verursacht das Löschen des ListStore eine Speicherschutzverletzung.
	sceneObjectsList.get_selection() -> unselect_all();
	if (sceneModel.hasScene()) {
		refSceneObjects = Gtk::ListStore::create(objectList);
        //std::cout << "new Liststore created" << std::endl;
#ifdef TRACK_DESTROY_EVENTS
        refSceneObjects -> add_destroy_notify_callback
              (this,  notify2);
#endif    
        //std::cout << "about to install model" << std::endl;
        sceneObjectsList.set_model(refSceneObjects);
        //std::cout << "model installed" << std::endl;
        this -> fillListstore();
	    noSelectionBttn.set_active(true);
	    editors.set_current_page(0);
	} else {
	    sceneObjectsList.get_selection() -> unselect_all();
	}
	disablePropertyEditors();
	notifyChange();
	//std::cout << "---------  onSceneChanged left  -------" << std::endl;
}

// Reaktion auf Auswahl eines Szeneobjekts:
void SceneObjectsEditor::onSelectionChanged() {
    int objectIndex;
    Gtk::TreeModel::iterator iter = sceneObjectsList.get_selection() -> get_selected();
    if (iter) {
        Gtk::TreeModel::Row selectedRow = *iter;
        objectIndex = selectedRow[objectList.indexColumn];
		std::list<AbstractSceneObject*>& list = sceneModel.getScene()->getObjectList();

        int cnt = 0;
        for (std::list<AbstractSceneObject*>::iterator iterator2 = list.begin();
             iterator2 != list.end();
             ++iterator2)
        {
			if (cnt == objectIndex) {
				currentlySelectedSceneObject = (*iterator2);
				break;
			}
			cnt++;
		}
		bool * lightSettings = currentlySelectedSceneObject -> getLightSettings();
	    noSelectionBttn.set_active(true);
	    enablePropertyEditors();
		notifyChange();
	    editors.set_current_page(0);
	    if (currentlySelectedSceneObject -> getMaterial() -> isSimpleColoring()) {
			materialEditors.set_current_page(0);
		} else {
			materialEditors.set_current_page(1);
		}
    }   
}


void SceneObjectsEditor::onLightToggled(const Glib::ustring& path, int pos) {
	if (currentlySelectedSceneObject != (AbstractSceneObject*) 0) {
		int selectionIndex = atoi(path.c_str());
		 Gtk::TreeModel::iterator iter = sceneObjectsList.get_model() -> get_iter(path);
         if (iter) {
            Gtk::TreeModel::Row selectedRow = *iter;
            selectedRow[objectList.useLight[pos]] = ! selectedRow[objectList.useLight[pos]];
		}

		std::list<AbstractSceneObject*>& list = sceneModel.getScene()->getObjectList();
        int cnt = 0;
        for (std::list<AbstractSceneObject*>::iterator Iterator = list.begin();
             Iterator != list.end();
             ++Iterator)
        {
			if (cnt == selectionIndex) {
				(*Iterator) -> toggleLightSource(pos);
				break;
			} 
			cnt++;
		}
	    sceneModel.notifySceneAttributeChange();
    }
}

void SceneObjectsEditor::onShowObjectToggleClicked(const Glib::ustring& path) {
	if (currentlySelectedSceneObject != (AbstractSceneObject*) 0) {
		int selectionIndex = atoi(path.c_str());
		 Gtk::TreeModel::iterator iter = sceneObjectsList.get_model() -> get_iter(path);
         if (iter) {
            Gtk::TreeModel::Row selectedRow = *iter;
            selectedRow[objectList.showObject] = ! selectedRow[objectList.showObject];
		}

		std::list<AbstractSceneObject*>& list = sceneModel.getScene()->getObjectList();
        int cnt = 0;
        for (std::list<AbstractSceneObject*>::iterator Iterator = list.begin();
             Iterator != list.end();
             ++Iterator)
        {
			if (cnt == selectionIndex) {
				(*Iterator) -> toggleShowObject();
				break;
			} 
			cnt++;
		}
	    sceneModel.notifySceneAttributeChange();
    }
}



void SceneObjectsEditor::onLightModelToggleClicked(const Glib::ustring& path) {
	if (currentlySelectedSceneObject != (AbstractSceneObject*) 0) {
		int selectionIndex = atoi(path.c_str());
		 Gtk::TreeModel::iterator iter = sceneObjectsList.get_model() -> get_iter(path);
         if (iter) {
            Gtk::TreeModel::Row selectedRow = *iter;
            selectedRow[objectList.useLightModelTwoSide] = ! selectedRow[objectList.useLightModelTwoSide];
		}

		std::list<AbstractSceneObject*>& list = sceneModel.getScene()->getObjectList();
        int cnt = 0;
        for (std::list<AbstractSceneObject*>::iterator Iterator = list.begin();
             Iterator != list.end();
             ++Iterator)
        {
			if (cnt == selectionIndex) {
				(*Iterator) -> toggleLightModelTwoSide();
				break;
			} 
			cnt++;
		}
	    sceneModel.notifySceneAttributeChange();
    }
}


void SceneObjectsEditor::onShowWireframeToggleClicked(const Glib::ustring& path) {
	if (currentlySelectedSceneObject != (AbstractSceneObject*) 0) {
		int selectionIndex = atoi(path.c_str());
		 Gtk::TreeModel::iterator iter = sceneObjectsList.get_model() -> get_iter(path);
         if (iter) {
            Gtk::TreeModel::Row selectedRow = *iter;
            selectedRow[objectList.showWireframe] = ! selectedRow[objectList.showWireframe];
		}

		std::list<AbstractSceneObject*>& list = sceneModel.getScene()->getObjectList();
        int cnt = 0;
        for (std::list<AbstractSceneObject*>::iterator Iterator = list.begin();
             Iterator != list.end();
             ++Iterator)
        {
			if (cnt == selectionIndex) {
				(*Iterator) -> toggleShowWireframe();
				break;
			} 
			cnt++;
		}
	    sceneModel.notifySceneAttributeChange();
    }
}

void SceneObjectsEditor::onShowTextureToggleClicked(const Glib::ustring& path) {
	if (currentlySelectedSceneObject != (AbstractSceneObject*) 0) {
		int selectionIndex = atoi(path.c_str());
		 Gtk::TreeModel::iterator iter = sceneObjectsList.get_model() -> get_iter(path);
         if (iter) {
            Gtk::TreeModel::Row selectedRow = *iter;
            selectedRow[objectList.showTexture] = ! selectedRow[objectList.showTexture];
		}

		std::list<AbstractSceneObject*>& list = sceneModel.getScene()->getObjectList();
        int cnt = 0;
        for (std::list<AbstractSceneObject*>::iterator Iterator = list.begin();
             Iterator != list.end();
             ++Iterator)
        {
			if (cnt == selectionIndex) {
				(*Iterator) -> toggleShowTexture();
				break;
			} 
			cnt++;
		}
	    sceneModel.notifySceneAttributeChange();
    }
}

void SceneObjectsEditor::onShowFogToggleClicked(const Glib::ustring& path) {
	if (currentlySelectedSceneObject != (AbstractSceneObject*) 0) {
		int selectionIndex = atoi(path.c_str());
		 Gtk::TreeModel::iterator iter = sceneObjectsList.get_model() -> get_iter(path);
         if (iter) {
            Gtk::TreeModel::Row selectedRow = *iter;
            selectedRow[objectList.showFog] = ! selectedRow[objectList.showFog];
		}

		std::list<AbstractSceneObject*>& list = sceneModel.getScene()->getObjectList();
        int cnt = 0;
        for (std::list<AbstractSceneObject*>::iterator Iterator = list.begin();
             Iterator != list.end();
             ++Iterator)
        {
			if (cnt == selectionIndex) {
				(*Iterator) -> toggleShowFog();
				break;
			} 
			cnt++;
		}
	    sceneModel.notifySceneAttributeChange();
    }
}



void SceneObjectsEditor::onMaterialButtonClicked() {
	if (selectMaterialEditor.get_active()) { 
	    editors.set_current_page(1);
	}
}

void SceneObjectsEditor::onTextureButtonClicked() {
	if (selectTextureEditor.get_active()) {
	    editors.set_current_page(2);
	}
}

void SceneObjectsEditor::onFogButtonClicked() {
	if (selectFogEditor.get_active()) {
	    editors.set_current_page(3);
	}
}


//  Da alle PropertyEditors im Container  box3 platziert sind,
//  genügt es, diesen Container passiv zu machen.
void SceneObjectsEditor::disablePropertyEditors() { 
	box3.set_sensitive(false);
} 


//  Da alle PropertyEditors im Container  box3 platziert sind,
//  genügt es, diesen Container zu aktivieren.
void SceneObjectsEditor::enablePropertyEditors() {
	box3.set_sensitive(true);
}
