#include "SimpleMaterialPropertiesEditor.h"
#include "SimpleMaterialPropertiesEditorStrings.h"
#include "SceneObjectsEditor.h"

SimpleMaterialPropertiesEditor::SimpleMaterialPropertiesEditor (
                SceneHolder& sm,
                SceneObjectsEditor& se)
  : sceneModel(sm),
    parentEditor(se),
	currentMaterialSpec((MaterialProperties*) 0),
    currentlySelectedColor((ColorSpec*) 0),
    red(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    green(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    blue(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    shininess(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    reglerR(red, Gtk::ORIENTATION_HORIZONTAL),
    reglerG(green, Gtk::ORIENTATION_HORIZONTAL),
    reglerB(blue, Gtk::ORIENTATION_HORIZONTAL),
    reglerShininess(shininess, Gtk::ORIENTATION_HORIZONTAL),
    noSelectionBttn(""),
    ambientColorBttn(SimpleMaterialPropertiesEditor::LiteralAmbient),
    diffuseColorBttn(SimpleMaterialPropertiesEditor::LiteralDiffuse),
    specularColorBttn(SimpleMaterialPropertiesEditor::LiteralSpecular),
    emissiveColorBttn(SimpleMaterialPropertiesEditor::LiteralEmissive),
    container(Gtk::ORIENTATION_VERTICAL, 12),
    container1(Gtk::ORIENTATION_HORIZONTAL),
    container2(Gtk::ORIENTATION_VERTICAL),
	suspendUpdates(false),
	scaleTransformation(shininess, 2, -7, 7)
{
	parentEditor.addObserver(this);
	add(container);
}

SimpleMaterialPropertiesEditor::~SimpleMaterialPropertiesEditor() {
	parentEditor.removeObserver(this);
}

SimpleMaterialPropertiesEditor* SimpleMaterialPropertiesEditor::create(SceneHolder& sm, SceneObjectsEditor& se)
{
	SimpleMaterialPropertiesEditor* instance = new SimpleMaterialPropertiesEditor(sm, se);
	instance -> configureComponents();
	instance -> buildForm();
	return instance;
}

void SimpleMaterialPropertiesEditor::configureComponents() {
	reglerR.set_value_pos(Gtk::POS_LEFT);
    reglerG.set_value_pos(Gtk::POS_LEFT);
    reglerB.set_value_pos(Gtk::POS_LEFT);
    reglerR.set_digits(0);
    reglerG.set_digits(0);
    reglerB.set_digits(0);
    reglerShininess.set_draw_value(false);
    
    //  Die Dokumentation zu Gtkmm3 besagt:
    //   RadioButtons are "off" when created; this means that when you
    //   first make a group of them, they will all be off. Don't
    //   forget to turn one of them on using  set_active().
    //  Das stimmt aber nicht und als Ausweg wird der unsichtbare
    //  RadioButton empfohlen, der immer dann aktiviert wird, wenn
    //  kein sichtbarer Button aktiviert sein darf.
	Gtk::RadioButton::Group group = noSelectionBttn.get_group();
	ambientColorBttn.set_group(group);
	diffuseColorBttn.set_group(group);
	specularColorBttn.set_group(group);
	emissiveColorBttn.set_group(group);
	
	ambientColorBttn.set_mode(false);
	diffuseColorBttn.set_mode(false);
	specularColorBttn.set_mode(false);
	emissiveColorBttn.set_mode(false);
	noSelectionBttn.set_active(true);
	
	noSelectionBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::noSelectionBttnClicked));
	ambientColorBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::ambientColorBttnClicked));
	diffuseColorBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::diffuseColorBttnClicked));
	specularColorBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::specularColorBttnClicked));
	emissiveColorBttn.signal_clicked().connect(
         sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::emissiveColorBttnClicked));
	
    red->signal_value_changed().connect(sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::onRValueChanged));
    green->signal_value_changed().connect(sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::onGValueChanged));
    blue->signal_value_changed().connect(sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::onBValueChanged));
    shininess->signal_value_changed().connect(sigc::mem_fun(*this, &SimpleMaterialPropertiesEditor::onShininessValueChanged));

}



void SimpleMaterialPropertiesEditor::buildForm() {
	
	container1.pack_start(ambientColor, Gtk::PACK_SHRINK);
	container1.pack_start(ambientColorBttn);
	container1.pack_start(diffuseColor, Gtk::PACK_SHRINK);
	container1.pack_start(diffuseColorBttn);
	container1.pack_start(specularColor, Gtk::PACK_SHRINK);
	container1.pack_start(specularColorBttn);
	container1.pack_start(emissiveColor, Gtk::PACK_SHRINK);
	container1.pack_start(emissiveColorBttn);
    container2.pack_start(reglerR, Gtk::PACK_SHRINK);
    container2.pack_start(reglerG, Gtk::PACK_SHRINK);
    container2.pack_start(reglerB, Gtk::PACK_SHRINK);
    container.pack_start(reglerShininess);
    container.pack_start(container1, false, false);
    container.pack_start(container2, true, true);
	container2.show();
	container1.show();
	container.show();
}


void SimpleMaterialPropertiesEditor::onObjectSelectionChanged() {
	if (parentEditor.hasCurrentSelection()) {
		loadSceneObjectData();
		disableFirstSelection();  //  macht das Richtige, sollte anders heissen
		enableFirstSelection();
	} else {
	    currentMaterialSpec = (MaterialProperties*) 0;
	    currentlySelectedColor = (ColorSpec*) 0;
	    currentColorDisplayField = (ColorDisplayField*) 0;
	    disableFirstSelection();
	}
	noSelectionBttn.set_active(true);
    updateVisualSettings();
}

void SimpleMaterialPropertiesEditor::loadSceneObjectData() {
	AbstractSceneObject* obj = parentEditor.getCurrentlySelectedObject();
	AbstractMaterial* mat = obj -> getMaterial();
	SimpleMaterialColoring* mat2 = (SimpleMaterialColoring*)mat;
	currentMaterialSpec =  mat2 -> getFaceColoring();
}

void SimpleMaterialPropertiesEditor::onRValueChanged() {
	if (hasSelectedColor() &&(!suspendUpdates)) {
        currentlySelectedColor->setRedValue(reglerR.get_value()/255.0f);
        currentColorDisplayField->updateRedColorComponent(reglerR.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void SimpleMaterialPropertiesEditor::onGValueChanged() {
	if (hasSelectedColor() &&(!suspendUpdates)) {
        currentlySelectedColor->setGreenValue(reglerG.get_value()/255.0f);
        currentColorDisplayField->updateGreenColorComponent(reglerG.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void SimpleMaterialPropertiesEditor::onBValueChanged() {
	if (hasSelectedColor() &&(!suspendUpdates)) {
        currentlySelectedColor->setBlueValue(reglerB.get_value()/255.0f);
        currentColorDisplayField->updateBlueColorComponent(reglerB.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void SimpleMaterialPropertiesEditor::onShininessValueChanged() {
	if ((hasSelectedMaterial()) &&(!suspendUpdates)) { 
		int selValue = reglerShininess.get_value();
		currentMaterialSpec -> setShininess(scaleTransformation.scaleToValue(selValue));
	    sceneModel.notifySceneAttributeChange(); 
	}
}

void SimpleMaterialPropertiesEditor::setSelectedColor(ColorSpec* cs,
                                   ColorDisplayField* cdf)
{
	currentlySelectedColor = cs;
	currentColorDisplayField = cdf;
}

bool SimpleMaterialPropertiesEditor::hasSelectedColor() {
	return currentlySelectedColor != (ColorSpec*) 0;
}

bool SimpleMaterialPropertiesEditor::hasSelectedMaterial() {
	return currentMaterialSpec != (MaterialProperties*) 0;
}

void SimpleMaterialPropertiesEditor::noSelectionBttnClicked() {
	if (!noSelectionBttn.get_active())
	    return;
	suspendUpdates = true;
	red -> set_value(0);
	green -> set_value(0);
	blue -> set_value(0);
	suspendUpdates = false;
}

void SimpleMaterialPropertiesEditor::ambientColorBttnClicked() {
	if (!ambientColorBttn.get_active())
	    return;
    if (hasSelectedMaterial()) {
		setSelectedColor(currentMaterialSpec -> getPtrToAmbientColor(),
		                 &ambientColor);
		currentColorDisplayField -> setColor(currentlySelectedColor);
		enableSliders();
		updateSliderSettings();
	}
}

void SimpleMaterialPropertiesEditor::diffuseColorBttnClicked() {
	if (!diffuseColorBttn.get_active())
	    return;
    if (hasSelectedMaterial()) {
		setSelectedColor(currentMaterialSpec -> getPtrToDiffuseColor(),
		                 &diffuseColor);
		currentColorDisplayField -> setColor(currentlySelectedColor);
		enableSliders();
		updateSliderSettings();
	}
}

void SimpleMaterialPropertiesEditor::specularColorBttnClicked() {
	if (!specularColorBttn.get_active())
	    return;
    if (hasSelectedMaterial()) {
		setSelectedColor(currentMaterialSpec -> getPtrToSpecularColor(),
		                 &specularColor);
		currentColorDisplayField -> setColor(currentlySelectedColor);
		enableSliders();
		updateSliderSettings();
	}
}

void SimpleMaterialPropertiesEditor::emissiveColorBttnClicked() {
	if (!emissiveColorBttn.get_active())
	    return;
    if (hasSelectedMaterial()) {
		setSelectedColor(currentMaterialSpec -> getPtrToEmissiveColor(),
		                 &emissiveColor);
		currentColorDisplayField -> setColor(currentlySelectedColor);
		enableSliders();
		updateSliderSettings();
	}
}

void SimpleMaterialPropertiesEditor::updateSliderSettings() {
	suspendUpdates = true;
	red -> set_value(currentlySelectedColor -> getRedEditValue());
	green -> set_value(currentlySelectedColor -> getGreenEditValue());
	blue -> set_value(currentlySelectedColor -> getBlueEditValue());
	suspendUpdates = false;
}

void SimpleMaterialPropertiesEditor::updateVisualSettings() {
	if (hasSelectedMaterial()) {
		ambientColor.setColor(currentMaterialSpec -> getPtrToAmbientColor());
		diffuseColor.setColor(currentMaterialSpec -> getPtrToDiffuseColor());
		specularColor.setColor(currentMaterialSpec -> getPtrToSpecularColor());
		emissiveColor.setColor(currentMaterialSpec -> getPtrToEmissiveColor());
		suspendUpdates = true;
		shininess -> set_value(scaleTransformation.valueToScale(currentMaterialSpec -> getShininess()));
		suspendUpdates = false;
	} else {
		ambientColor.setColor(0, 0, 0);
		diffuseColor.setColor(0, 0, 0);
		specularColor.setColor(0, 0, 0);
		emissiveColor.setColor(0, 0, 0);
		suspendUpdates = true;
		shininess -> set_value(0);
		suspendUpdates = false;
	}
}

void SimpleMaterialPropertiesEditor::enableFirstSelection() {
	ambientColorBttn.set_sensitive(true);
	diffuseColorBttn.set_sensitive(true);
	specularColorBttn.set_sensitive(true);
	emissiveColorBttn.set_sensitive(true);
	reglerShininess.set_sensitive(true);
}

void SimpleMaterialPropertiesEditor::disableFirstSelection() {
	disableSliders();
	ambientColorBttn.set_sensitive(false);
	diffuseColorBttn.set_sensitive(false);
	specularColorBttn.set_sensitive(false);
	emissiveColorBttn.set_sensitive(false);
	reglerShininess.set_sensitive(false);
}

void SimpleMaterialPropertiesEditor::enableSliders() {
	container2.set_sensitive(true); 
}

void SimpleMaterialPropertiesEditor::disableSliders() {
	container2.set_sensitive(false);
}
