#include "SimpsonRule.h"

#include <math.h>

SimpsonRule::SimpsonRule(float start, float end, float e, int n) :
    Integrator (start, end, e, n)
{
}

float SimpsonRule::computeValue(ArcLengthIntegrand* integrand) {
    const float factors [] = { 4.0f, 2.0f, 2.0f, 4.0f };
    const float distances [] = { 1.0f, 2.0f, 4.0f, 5.0f };
    const int nroOfPoints = 4;
    float interval = (tEnd - tStart)/2.0f;
    float xi = tStart + interval;
    
    float area = (integrand->fn(xi)*4.0f
                  + integrand->fn(tStart) + integrand->fn(tEnd)
                 )*(tEnd - tStart)/6.0f;
    
    int n = 1;
    bool stop = false;
    
    while (!stop) {
        float oldArea = area;
        float sum = 0.0f;
        float delta = interval / 3.0f;
        for (int i = 0; i < n; i++) {
            float localStart = i*interval * 2 + tStart;
            float s = 0.0f;
            for (int j = 0; j < nroOfPoints; j++) {
                s += integrand->fn (localStart + distances[j]*delta) * factors[j];
            }
            
            sum += s;
        }
        n *= 3;
        area = area / 3.0f + sum*(tEnd - tStart) /(6.0f*n);
        interval /= 3.0f;
        stop = n > maxN || (fabs(area - oldArea) < fabs(area)*eps) ;
    }
    return area;
}
