#include "SpacialCurveBuilder.h"


SpacialCurveObject* SpacialCurveBuilder::createSpacialCurve
	         (ObjectDisplayFlags &objectSettings,
               PolylineSpec *polygonSpec,
              ParametricSpacialCurveEquation* functions) {
	int nx = polygonSpec -> nx;
	float x0 = polygonSpec-> x0;
	float xStop = polygonSpec -> xStop;
	float stepX = (xStop - x0)/(nx - 1);
	BoundingBox boundingBox;
	GLfloat* vertices;
	
    vertices =  new GLfloat[3*nx];
    GLfloat* vertexPtr = vertices;
    SpacialPoint pt;
    
    for (int i = 0; i < nx; i++) {
		float t = x0 + i*stepX;
        functions -> fn(t, &pt);
        boundingBox.processSpacialPoint(pt);
        *vertexPtr++ = pt.x;
        *vertexPtr++ = pt.y;
        *vertexPtr++ = pt.z;
	}	
    return new SpacialCurveObject(objectSettings, boundingBox, nx, vertices);
}


SpacialCurveObject* SpacialCurveBuilder::createSpacialCurveAndTexture
	         (
               ObjectDisplayFlags &objectSettings,
               PolylineSpec *polygonSpec, 
              ParametricSpacialCurveEquation* functions) {
	int nx = polygonSpec -> nx;
	float x0 = polygonSpec-> x0;
	float xStop = polygonSpec -> xStop;
	float stepX = (xStop - x0)/(nx - 1);
	BoundingBox boundingBox;
	GLfloat* vertices;
    GLfloat *textureCoordinates;
    char* textureBuffer;
	
	textureCoordinates = new GLfloat[2*nx];
    int textureWidth = 128;
    int textureHeight = 1;
    int textureBufferSize = 3*textureWidth*textureHeight;
    textureBuffer = new char[textureBufferSize];
    char* textureBufferPtr = textureBuffer;
    
    for (int i = 0; i < textureWidth; i++) {
            char colorToUse;
            if (i < textureWidth/2 ) {
                   colorToUse = (char)250;
               } else {
                   colorToUse = (char) 80;
               }
            *textureBufferPtr++ = colorToUse;
            *textureBufferPtr++ = colorToUse;
            *textureBufferPtr++ = colorToUse;
    }
	
    vertices =  new GLfloat[3*nx];
    GLfloat* vertexPtr = vertices;
    GLfloat* texCoordPtr = textureCoordinates;
    SpacialPoint pt;
    
    for (int i = 0; i < nx; i++) {
		float t = x0 + i*stepX;
		float tt = i * 0.5;
        functions -> fn(t, &pt);
        boundingBox.processSpacialPoint(pt);
		*vertexPtr++ = pt.x;
        *vertexPtr++ = pt.y;
        *vertexPtr++ = pt.z;
        *texCoordPtr++ = tt;
        *texCoordPtr++ = 0;
	}	

    return new SpacialCurveObject(objectSettings, boundingBox, nx, vertices,
                   textureCoordinates, textureBuffer);
}
