#include "SpacialCurveObject.h"
#include "MaterialColoring.h"
#include "SimpleMaterialColoring.h"

SpacialCurveObject::SpacialCurveObject(ObjectDisplayFlags &flags, BoundingBox bb, int nv, GLfloat* v):
   AbstractSceneObject(flags)
{
	boundingBox = bb;
	nroOfVertices = nv;
	vertices = v;
}
SpacialCurveObject::SpacialCurveObject(ObjectDisplayFlags &flags,
               BoundingBox bb, int nv, GLfloat* v, GLfloat* tc, char* tb):
   AbstractSceneObject(flags)
{
	boundingBox = bb;
	nroOfVertices = nv;
	vertices = v;
	textureCoordinates = tc;
	textureBuffer = tb;
}

int SpacialCurveObject::getNumberOfVertices() {
	return nroOfVertices;
}

void SpacialCurveObject::renderTexturizedObject() {
	GLuint textureHandles [1];
    
    glGenTextures(1, &textureHandles[0]);
    
    glBindTexture(GL_TEXTURE_2D, textureHandles[0]);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 128, 1, 0,
                GL_RGB, GL_UNSIGNED_BYTE, textureBuffer);
    
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (GLfloat)GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (GLfloat)GL_NEAREST);  
    glEnable(GL_TEXTURE_2D);
    //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glBlendFunc(GL_SRC_COLOR, GL_DST_COLOR);
    glEnable(GL_BLEND);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    glTexCoordPointer(2, GL_FLOAT, 0, textureCoordinates); 
     //  in OpenGL ES gibt es keine eindimensionalen Koordinatenarrays!
     //  (in OpenGL stehen sie zur Verfügung).
    
    renderNonTexturizedObject();
    
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDeleteTextures(1, textureHandles);

}

void SpacialCurveObject::renderNonTexturizedObject() {
	
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_SMOOTH);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);  // antialiasing bei line drawing
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FLOAT, 0, vertices);
	if (vertexColors != 0) {
		glColorPointer(4, GL_UNSIGNED_BYTE, 0, vertexColors);
		glEnableClientState(GL_COLOR_ARRAY);
		glEnable(GL_COLOR_MATERIAL);
		
        glBlendFunc(GL_SRC_COLOR, GL_DST_COLOR);
        glEnable(GL_BLEND);
	}

	if ((fogSpec != (FogSpec*) 0) && showFog() ) {
		glEnable(GL_FOG);
		fogSpec -> install();
	} 
	getMaterial() -> renderColoredSurface(this);

	if ((fogSpec != (FogSpec*) 0) && showFog()) {
		glDisable(GL_FOG);
	} 
    if (vertexColors != 0) {
		glDisableClientState(GL_COLOR_ARRAY);
		glDisable(GL_COLOR_MATERIAL);
        glDisable(GL_BLEND);
	}
	glDisable(GL_VERTEX_ARRAY);
	glDisable(GL_LINE_SMOOTH);
	glDisable(GL_SMOOTH);
}


void SpacialCurveObject::renderObject() {
	if (this->showTexture() && (textureCoordinates != (GLfloat*)0)) {
		renderTexturizedObject();
	} else {
		renderNonTexturizedObject();
	}
	
}

void SpacialCurveObject::renderSimpleColoring(SimpleMaterialColoring* m) {
	m -> install();
    glDrawArrays(GL_LINE_STRIP, 0, nroOfVertices);
}

void SpacialCurveObject::renderDoubleSidedColoring(MaterialColoring* m) {
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	m -> setFrontColoring();
    glDrawArrays(GL_LINE_STRIP, 0, nroOfVertices);
      
    glCullFace(GL_FRONT);  // front unterdruecken
      
    m -> setBackColoring();
    glDrawArrays(GL_LINE_STRIP, 0, nroOfVertices);  /// Rueckseite zeichnen
      
	glDisable(GL_CULL_FACE);
}
