#define PI 3.14159265359f
#include "SpacialRotation.h"
#include <math.h>

SpacialRotation::SpacialRotation(float angle, SpacialPoint axis) {
    float halfAngle = angle * PI/ 360.0f;
    float coshA = cos(halfAngle);
    float sinhA = sin(halfAngle);
    a = coshA;
    b = axis.x * sinhA;
    c = axis.y * sinhA;
    d = axis.z * sinhA;
}

SpacialRotation::SpacialRotation(float angle, float axisX, float axisY, float axisZ) {
    a = angle;
    b = axisX;
    c = axisY;
    d = axisZ;
    if (a < 0.0f) {
		a = -a;
		b = -b;
		c = -c;
		d = -d;
	}
}

/*
 * Hintereinanderausführung zweier Rotationen
 */
SpacialRotation operator * (SpacialRotation sp1, SpacialRotation sp2) {
	float alpha = sp1.getAngleInRadians()*sp2.getAngleInRadians()
	              - sp1.getAxis().dot(sp2.getAxis());
	SpacialPoint v1 = sp1.getAxis() * sp2.getAngleInRadians();
	SpacialPoint v2 = sp2.getAxis() * sp1.getAngleInRadians();
	SpacialPoint v3 = sp2.getAxis().cross(sp2.getAxis());
	SpacialPoint v4 = v1 + v2 + v3;
	
	return  SpacialRotation (alpha, v4.x, v4.y, v4.z);
}

SpacialPoint SpacialRotation::getAxis() {
	return SpacialPoint(b, c, d);
}

float SpacialRotation::getAngleInRadians() {
	return a;
}
