#include "SpacialSurfaceBuilder.h"
#include "AbstractSpacialBox.h"

#include <math.h>
#include <iostream>

using namespace std;


//
//   Diese Methode erzeugt das Profil einer Funktion
//   zweier Veraenderlicher uber einem rechteckigen
//   Grundgebiet.
//
SpacialSurfaceObject* SpacialSurfaceBuilder::createFunctionMesh(
               ObjectDisplayFlags &objectSettings,
               MeshGrid * gridSpec,
               void (AbstractSpacialFunctionExample::*fptr) (float, float, Triple*),
               AbstractSpacialFunctionExample* inst) {
    GLfloat  *vertices;
    GLfloat  *vertexNormals;
    GLushort   *indices;
    GLfloat *textureCoordinates;
    char* textureBuffer;
    int sizeIndices;
    int textureWidth, textureHeight, textureBufferSize;
	int nx = gridSpec -> nx;
	float x0 = gridSpec-> x0;
	float xStop = gridSpec -> xStop;
	int ny = gridSpec -> ny;
	float y0 = gridSpec-> y0;
	float yStop = gridSpec -> yStop;
    
    BoundingBox spacialBox;
   
    float stepPx = (xStop - x0)/(nx - 1);
    float stepPy = (yStop - y0)/(ny - 1);
    Triple t;
    SpacialPoint sp, spNormalized;
    vertices =  new GLfloat[3*nx*ny];
    vertexNormals = new GLfloat[3*nx*ny];
    sizeIndices = 6*(nx - 1)*(ny - 1);
    indices =  new GLushort[sizeIndices];  // fuer Dreiecksmesh
    
    textureCoordinates = new GLfloat[2*nx*ny];   //    move
    textureWidth = 128;
    textureHeight = 128;
    textureBufferSize = 3*textureWidth*textureHeight;
    textureBuffer = new char[textureBufferSize];
    char* textureBufferPtr = textureBuffer;
    
    for (int i = 0; i < textureWidth; i++) {
        for (int j = 0; j < textureHeight; j++) {
            char colorToUseR, colorToUseG, colorToUseB;
            if ((i < textureWidth/2 && j < textureHeight/2)
               || (i >= textureWidth/2 && j >= textureHeight/2)) {
                   colorToUseR = (char)240;
                   colorToUseG = (char)200;
                   colorToUseB = (char)200;
               } else {
                   colorToUseR = (char) 220;
                   colorToUseG = (char) 180;
                   colorToUseB = (char) 180;
               }
            *textureBufferPtr++ = colorToUseR;
            *textureBufferPtr++ = colorToUseG;
            *textureBufferPtr++ = colorToUseB;
        }
    }   //  --  move 
    
    spacialBox.setXRange(x0, xStop);
    spacialBox.setYRange(y0, yStop);
    
    GLfloat* vertexPtr = vertices;
    GLfloat* vertexNormalsPtr = vertexNormals;
    GLfloat* texCoordPtr = textureCoordinates;
    
    for (int i = 0; i < nx; i++) {
        float x = x0 + i*stepPx;
        float tx = i *0.5f;
        for (int j = 0; j < ny; j++) {
            float y = y0 + j*stepPy;
            float ty = j*0.5f;
            (inst->*fptr)(x, y, &t);
            spacialBox.processZValue(t.fn);
            *vertexPtr++ = x;
            *vertexPtr++ = y;
            *vertexPtr++ = t.fn;
            *texCoordPtr++ = tx;  //  move
            *texCoordPtr++ = ty;  //  --  move
            sp.x = -t.fdx;
            sp.y = -t.fdy;
            sp.z = 1.0f;
            spNormalized = sp.safelyNormalized();
            *vertexNormalsPtr++ = spNormalized.x;
            *vertexNormalsPtr++ = spNormalized.y;
            *vertexNormalsPtr++ = spNormalized.z;
        }
    }
    //  Erzeugung der Indizes
    GLushort *indexPtr = indices; 
    for (int i = 0; i < ny - 1; i++) {
        for (int j = 0; j < nx - 1; j++) {
            unsigned int k = (unsigned int)(j*ny + i); /*
            *indexPtr++ = k;
            *indexPtr++ = k + ny;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + ny;
            *indexPtr++ = k + ny + 1;
            *indexPtr++ = k + 1; */
            *indexPtr++ = k;
            *indexPtr++ = k + ny;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + ny;
            *indexPtr++ = k + ny + 1;
            // Diese Abfolge der Punkte ist für das Zeichnen des
            // wire frame zu bevorzugen
        }
    }
    //  hier wird die Textur vorbereitet - das ist nochmal ziemlich tricky
    
    SpacialSurfaceObject* obj = new SpacialSurfaceObject(
               objectSettings,
               spacialBox, vertices, vertexNormals,
               indices, /**/ textureCoordinates,
               textureBuffer, textureWidth, textureHeight, /**/
               sizeIndices);
    return obj;
}

//
//   Die Methode erzeugt das Profil einer parametrischen Funktion
//   zweier Parameter. Das Ergebnis ist eine Raumflaeche.
//
SpacialSurfaceObject* SpacialSurfaceBuilder::createParametricFunctionMesh(
                    ObjectDisplayFlags &objectSettings,
                    MeshGrid * gridSpec, 
                    void (AbstractParametricFunctionExample::*fptr2) (float, float, VTriple*),
                    AbstractParametricFunctionExample* instPF) {
                        
    GLfloat  *vertices;
    GLfloat  *vertexNormals;
    GLushort   *indices;
    GLfloat *textureCoordinates;
    char* textureBuffer;
    int sizeIndices;
    int textureWidth, textureHeight, textureBufferSize;
	int nx = gridSpec -> nx;
	float x0 = gridSpec-> x0;
	float xStop = gridSpec -> xStop;
	int ny = gridSpec -> ny;
	float y0 = gridSpec-> y0;
	float yStop = gridSpec -> yStop;
    
    BoundingBox spacialBox;
   
    float stepPx = (xStop - x0)/(nx - 1);
    float stepPy = (yStop - y0)/(ny - 1);
    VTriple triple;
    
    SpacialPoint sp, spNormalized;
    vertices =  new GLfloat[3*nx*ny];
    vertexNormals = new GLfloat[3*nx*ny];
    sizeIndices = 6*(nx - 1)*(ny - 1);
    indices =  new GLushort[sizeIndices];  // fuer Dreiecksmesh
    
    textureCoordinates = new GLfloat[2*nx*ny];
    textureWidth = 128;
    textureHeight = 128;
    textureBufferSize = 3*textureWidth*textureHeight;
    textureBuffer = new char[textureBufferSize];
    char* textureBufferPtr = textureBuffer;
    
    for (int i = 0; i < textureWidth; i++) {
        for (int j = 0; j < textureHeight; j++) {
            char colorToUse;
            if ((i < textureWidth/2 && j < textureHeight/2)
               || (i >= textureWidth/2 && j >= textureHeight/2)) {
                   colorToUse = (char)220;
               } else {
                   colorToUse = (char) 200;
               }
            *textureBufferPtr++ = colorToUse;
            *textureBufferPtr++ = colorToUse;
            *textureBufferPtr++ = colorToUse;
        }
    }
    
    GLfloat* vertexPtr = vertices;
    GLfloat* vertexNormalsPtr = vertexNormals;
    GLfloat* texCoordPtr = textureCoordinates;
       
    for (int i = 0; i < nx; i++) {
        float x = x0 + i*stepPx;
        float tx = i *0.5f;
        for (int j = 0; j < ny; j++) {
            float y = y0 + j*stepPy;
            float ty = j *0.5f;
            (instPF->*fptr2)(x, y, &triple);
            spacialBox.processSpacialPoint(triple.fn);
            *vertexPtr++ = triple.fn.x;
            *vertexPtr++ = triple.fn.y;
            *vertexPtr++ = triple.fn.z;
            SpacialPoint n = triple.fdx.cross(triple.fdy);
            n = n.safelyNormalized();
            *vertexNormalsPtr++ = n.x;
            *vertexNormalsPtr++ = n.y;
            *vertexNormalsPtr++ = n.z;
            *texCoordPtr++ = tx;
            *texCoordPtr++ = ty;
        }
    }
        //  Erzeugung der Indizes
    GLushort *indexPtr = indices; 
    for (int i = 0; i < ny - 1; i++) {
        for (int j = 0; j < nx - 1; j++) {
            unsigned int k = (unsigned int)(j*ny + i); 
            *indexPtr++ = k;
            *indexPtr++ = k + ny;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + ny;
            *indexPtr++ = k + ny + 1;
            //  Diese Reihenfolge der Dreieckskoordinaten
            //  ist für das Zeichnen des Gittermodells  (wire frame)
            //  die richtige.  
           
        }
    }
    //  hier wird die Textur vorbereitet - das ist nochmal ziemlich tricky
    
    SpacialSurfaceObject* obj = new SpacialSurfaceObject(
               objectSettings,
               spacialBox, vertices, vertexNormals,
               indices, textureCoordinates,
               textureBuffer, textureWidth, textureHeight,
               sizeIndices);
    return obj;
}

//
//   
//  Hier wird das Verfahren fuer einen texturierten Tubus
//  realisiert. Dieses Verfahren verlangt ein etwas groesseres
//  Feld fuer die Vertices.
//
SpacialSurfaceObject* SpacialSurfaceBuilder::createTubularMesh(
                    ObjectDisplayFlags &objectSettings,
                    TubularGridSpec *polygonSpec,
                    float radius,
                    ParametricSpacialCurveSpec* functions) {
    GLfloat  *vertices;
    GLfloat  *vertexNormals;
    GLushort   *indices;
    GLfloat *textureCoordinates;
    char* textureBuffer;
    int sizeIndices;
    int textureWidth, textureHeight, textureBufferSize;
	int nx = polygonSpec -> nx;
	float y0 = polygonSpec-> x0;
	float yStop = polygonSpec -> xStop;
    
    BoundingBox spacialBox;
   
    ArcLengthIntegrand integrand(functions);
    Integrator* integrator = new NewtonRule(y0, yStop, 1.0e-8, 32000);
    float bogenlaenge = computeArcLength(&integrand, integrator);
    
    int ny = (int) (bogenlaenge/(2.0f*M_PI*radius / nx));
    if (ny % 2 == 0) {
        ny += 1;
    } 
    polygonSpec -> ny = ny;
    
    delete integrator;
    
    float stepPy = (yStop - y0)/(ny - 1);
    float deltaT = 2.0f*M_PI/(nx - 1);
    
    SpacialPoint sp, spNormalized;
    vertices =  new GLfloat[3*nx*ny];
    vertexNormals = new GLfloat[3*nx*ny];
    sizeIndices = 6*(nx - 1)*(ny - 1);
    indices =  new GLushort[sizeIndices];  // fuer Dreiecksmesh
    
    //  Herstellung einer 2D-Texture mit zwei unterschiedlichen
    //  Grauwerten.
    textureCoordinates = new GLfloat[2*nx*ny];
    textureWidth = 128;
    textureHeight = 128;
    textureBufferSize = 3*textureWidth*textureHeight;
    textureBuffer = new char[textureBufferSize];
    char* textureBufferPtr = textureBuffer;
    
    for (int i = 0; i < textureWidth; i++) {
        for (int j = 0; j < textureHeight; j++) {
            char colorToUse;
            if ((i < textureWidth/2 && j < textureHeight/2)
               || (i >= textureWidth/2 && j >= textureHeight/2)) {
                   colorToUse = (char)220;
               } else {
                   colorToUse = (char) 200;
               }
            *textureBufferPtr++ = colorToUse;
            *textureBufferPtr++ = colorToUse;
            *textureBufferPtr++ = colorToUse;
        }
    }
    
    SpacialPoint fValue;
    SpacialPoint d1Value;
    SpacialPoint d2Value;
    SpacialPoint tangente, binormale, normale;
    SpacialPoint xx, pt;
    
    GLfloat* vertexPtr = vertices;
    GLfloat* vertexNormalsPtr = vertexNormals;
    GLfloat* texCoordPtr = textureCoordinates;
    
    float deltaX = 0.5f;  //  preliminary
    float deltaY = 0.5f;  //  preliminary
       
    for (int i = 0; i < ny; i++) {
        float t = y0 + i*stepPy;
        float tt = i*deltaY;
        functions -> fn(t, &fValue);
        functions -> d1fn(t, &d1Value);
        functions -> d2fn(t, &d2Value);
        tangente = d1Value / d1Value.euclideanNorm();
        binormale = d1Value.cross(d2Value);
        binormale = binormale / binormale.euclideanNorm();
        normale = binormale.cross(tangente);
        GLfloat* firstIndex = vertexPtr;
        GLfloat* firstNormalIdx = vertexNormalsPtr;
        for (int j = 0; j < nx  - 1; j++) {
            float phi = j * deltaT;
            float tu = j*deltaX;
            xx = normale * sin(phi) + binormale*cos(phi);
            pt = fValue + xx*radius;
            spacialBox.processSpacialPoint(pt);  //  hier
            *vertexPtr++ = pt.x;
            *vertexPtr++ = pt.y;
            *vertexPtr++ = pt.z;
            *vertexNormalsPtr++ = xx.x;
            *vertexNormalsPtr++ = xx.y;
            *vertexNormalsPtr++ = xx.z;
            
            *texCoordPtr++ = tt;
            *texCoordPtr++ = tu;
        }
        for (int j = 0; j < 3; j++) {
            *vertexPtr++ = *firstIndex++;
            *vertexNormalsPtr++ = *firstNormalIdx++;
        }
        *texCoordPtr++ = tt;
        *texCoordPtr++ = (nx - 1)*deltaX;
    }
    //  Berechnung der Mesh-Indices
    GLushort *indexPtr = indices; 
    for (int i = 0; i < nx - 1; i++) {
        for (int j = 0; j < ny - 1; j++) {
            unsigned int k = (unsigned int)(j*nx + i);
            *indexPtr++ = k;
            *indexPtr++ = k + nx;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + nx;
            *indexPtr++ = k + nx + 1;
        }
    }

    SpacialSurfaceObject* obj = new SpacialSurfaceObject(
               objectSettings,
               spacialBox, vertices, vertexNormals,
               indices, textureCoordinates,
               textureBuffer, textureWidth, textureHeight,
               sizeIndices);
    return obj;
}





//
//   
//  Hier wird das Verfahren fuer einen texturierten Tubus
//  realisiert. Dieses Verfahren verlangt ein etwas groesseres
//  Feld fuer die Vertices.
//  ****  Erzeugung einer EmphasizedTexture   ******
//
SpacialSurfaceObject* SpacialSurfaceBuilder::createTubularMesh2(
                    ObjectDisplayFlags &objectSettings,
                    TubularGridSpec *polygonSpec,
                    float radius,
                    ParametricSpacialCurveSpec* functions) {
    GLfloat  *vertices;
    GLfloat  *vertexNormals;
    GLushort   *indices;
    GLfloat *textureCoordinates;
    char* textureBuffer;
    int sizeIndices;
    int textureWidth, textureHeight, textureBlockWidth, textureBufferSize;
	int nx = polygonSpec -> nx;
	float y0 = polygonSpec-> x0;
	float yStop = polygonSpec -> xStop;
	int nextPowerOfTwo = 2;
	while (nx > nextPowerOfTwo) {
		nextPowerOfTwo <<= 1;
	}
    
    BoundingBox spacialBox;
    //std::cout << "nx = " << nx <<  " power " << nextPowerOfTwo << std::endl;
    
    ArcLengthIntegrand integrand(functions);
    Integrator* integrator = new NewtonRule(y0, yStop, 1.0e-8, 32000);
    float bogenlaenge = computeArcLength(&integrand, integrator);
    
    int ny = (int) (bogenlaenge/(2.0f*M_PI*radius / nx));
    if (ny % 2 == 0) {
        ny += 1;
    } 
    polygonSpec -> ny = ny;
    
    delete integrator;
    
    float stepPy = (yStop - y0)/(ny - 1);
    float deltaT = 2.0f*M_PI/(nx - 1);
    
    SpacialPoint sp, spNormalized;
    vertices =  new GLfloat[3*nx*ny];
    vertexNormals = new GLfloat[3*nx*ny];
    sizeIndices = 6*(nx - 1)*(ny - 1);
    indices =  new GLushort[sizeIndices];  // fuer Dreiecksmesh
    
    
    
    //  Herstellung einer 2D-Texture mit zwei unterschiedlichen
    //  Grauwerten.
    textureCoordinates = new GLfloat[2*nx*ny];
    textureWidth = 64*nextPowerOfTwo;
    textureBlockWidth = 128;
    textureHeight = 128;
    textureBufferSize = 3*textureWidth*textureHeight;
    textureBuffer = new char[textureBufferSize];
    char* textureBufferPtr = textureBuffer;
    
    for (int i = 0; i < 64; i++) {
		for (int j = 0; j < textureHeight; j++) {
            char colorToUse;
            if (( /* i < 128/2 && */ j < textureHeight/2)
               /*|| (i >= 128/2 && j >= textureHeight/2) */ ) {
                   colorToUse = (char)200;
               } else {
                   colorToUse = (char) 220;
               }
            *textureBufferPtr++ = colorToUse;
            *textureBufferPtr++ = 110;
            *textureBufferPtr++ = 100;
        }
	}
	//std::cout << "Markierungsstreifen ist fertig" <<std::endl;
	
    for (int cnt = 1; cnt <= nx/2; cnt++) {
		//std::cout << "cnt = " << cnt << std::endl;
		for (int i = 0; i < textureBlockWidth; i++) {
			for (int j = 0; j < textureHeight; j++) {
				char colorToUse;
				if ((i < textureBlockWidth/2 && j < textureHeight/2)
				   || (i >= textureBlockWidth/2 && j >= textureHeight/2)) {
					   colorToUse = (char)220;
				   } else {
					   colorToUse = (char) 200;
				   }
				*textureBufferPtr++ = colorToUse;
				*textureBufferPtr++ = colorToUse;
				*textureBufferPtr++ = colorToUse;
			}
		}
	}
	//std::cout << "Übrige Streifen sind fertig" << std::endl;
    
    SpacialPoint fValue;
    SpacialPoint d1Value;
    SpacialPoint d2Value;
    SpacialPoint tangente, binormale, normale;
    SpacialPoint xx, pt;
    
    GLfloat* vertexPtr = vertices;
    GLfloat* vertexNormalsPtr = vertexNormals;
    GLfloat* texCoordPtr = textureCoordinates;
    
    float deltaX = 1.0f/nextPowerOfTwo; 
    float deltaY = 0.5f;  
       
    for (int i = 0; i < ny; i++) {
        float t = y0 + i*stepPy;
        float tt = i*deltaY;
        float angularOffset = - M_PI_2 - M_PI/nx;
        functions -> fn(t, &fValue);
        functions -> d1fn(t, &d1Value);
        functions -> d2fn(t, &d2Value);
        tangente = d1Value / d1Value.euclideanNorm();
        binormale = d1Value.cross(d2Value);
        binormale = binormale / binormale.euclideanNorm();
        normale = binormale.cross(tangente);
        GLfloat* firstIndex = vertexPtr;
        GLfloat* firstNormalIdx = vertexNormalsPtr;
        for (int j = 0; j < nx  - 1; j++) {
            float phi = j * deltaT + angularOffset;
            float tu = j*deltaX;
            xx = normale * sin(phi) + binormale*cos(phi);
            pt = fValue + xx*radius;
            spacialBox.processSpacialPoint(pt);  //  hier
            *vertexPtr++ = pt.x;
            *vertexPtr++ = pt.y;
            *vertexPtr++ = pt.z;
            *vertexNormalsPtr++ = xx.x;
            *vertexNormalsPtr++ = xx.y;
            *vertexNormalsPtr++ = xx.z;
            
            *texCoordPtr++ = tt;
            *texCoordPtr++ = tu;
        }
        for (int j = 0; j < 3; j++) {
            *vertexPtr++ = *firstIndex++;
            *vertexNormalsPtr++ = *firstNormalIdx++;
        }
        *texCoordPtr++ = tt;
        *texCoordPtr++ = (nx- 1)*deltaX;
    }
    //  Berechnung der Mesh-Indices
    GLushort *indexPtr = indices; 
    for (int i = 0; i < nx - 1; i++) {
        for (int j = 0; j < ny - 1; j++) {
            unsigned int k = (unsigned int)(j*nx + i);
            *indexPtr++ = k;
            *indexPtr++ = k + nx;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + 1;
            *indexPtr++ = k + nx;
            *indexPtr++ = k + nx + 1;
        }
    }

    SpacialSurfaceObject* obj = new SpacialSurfaceObject(
               objectSettings,
               spacialBox, vertices, vertexNormals,
               indices, textureCoordinates,
               textureBuffer, textureHeight, textureWidth, 
               sizeIndices);
    return obj;
}

