#include "SpacialSurfaceObject.h"
#include "MaterialColoring.h"
#include "SimpleMaterialColoring.h"

using namespace std;

SpacialSurfaceObject::SpacialSurfaceObject(ObjectDisplayFlags &flags) :
   AbstractSceneObject(flags)
{
	vertexColors = 0;
}

SpacialSurfaceObject::~SpacialSurfaceObject() {
    
    delete [] vertexNormals;
    delete [] indices;
}

SpacialSurfaceObject::SpacialSurfaceObject(ObjectDisplayFlags &flags, BoundingBox bb, GLfloat* v,
                    GLfloat* vn, GLushort* idx, GLfloat* tc,
                    int sIdx) :
   AbstractSceneObject(flags)
{
    boundingBox = bb;
    vertices = v;
    vertexNormals = vn;
    indices = idx;
    textureCoordinates = tc;
    sizeIndices = sIdx;
}
SpacialSurfaceObject::SpacialSurfaceObject(ObjectDisplayFlags &flags,  BoundingBox bb, GLfloat* v,
                    GLfloat* vn, GLushort* idx, GLfloat* tc,
                    char* texBuffer, int tw, int th,
                    int sIdx)  :
   AbstractSceneObject(flags)
{
    boundingBox = bb;
    vertices = v;
    vertexNormals = vn;
    indices = idx;
    textureBuffer = texBuffer;
    textureWidth = tw;
    textureHeight = th;
    textureCoordinates = tc;
    sizeIndices = sIdx;
}


void SpacialSurfaceObject::renderSimpleColoring(SimpleMaterialColoring* m) {
    //  set uniform coloring
      m -> install();
      glDrawElements(this -> showWireframe() ? GL_LINES : GL_TRIANGLES,  sizeIndices, GL_UNSIGNED_SHORT, indices);
}

void SpacialSurfaceObject::renderDoubleSidedColoring(MaterialColoring* m) {
      glEnable(GL_CULL_FACE);
      glCullFace(GL_BACK);
      
      m -> setFrontColoring();
      glDrawElements(this -> showWireframe() ? GL_LINES : GL_TRIANGLES,  sizeIndices, GL_UNSIGNED_SHORT, indices);
      
      glCullFace(GL_FRONT);  // front unterdruecken
      
      m -> setBackColoring();
      glDrawElements(this -> showWireframe() ? GL_LINES : GL_TRIANGLES,  sizeIndices, GL_UNSIGNED_SHORT, indices);  /// Rueckseite zeichnen
      
      glDisable(GL_CULL_FACE);

}

void SpacialSurfaceObject::renderObject() {
	if (hasFog() && showFog()) {
		fogSpec -> install();
		glEnable(GL_FOG);
	}
	if (this -> showTexture()) {
		renderTexturizedObject();
	} else {
		renderNonTexturizedObject();
	}
	if (hasFog() && showFog()) {
		glDisable(GL_FOG);
	}
}

//  glEnable(GL_COLOR_MATERIAL);
//  glEnable(GL_LIGHTING);
//  glEnable(GL_BLEND);
//  sind beide auszuprobieren
void SpacialSurfaceObject::renderNonTexturizedObject() {
      glEnableClientState(GL_VERTEX_ARRAY);
      glVertexPointer(3, GL_FLOAT, 0, vertices);
      glEnableClientState(GL_NORMAL_ARRAY);
      glNormalPointer(GL_FLOAT, 0, vertexNormals);
      if (vertexColors != 0) {
		  glColorPointer(4, GL_UNSIGNED_BYTE, 0, vertexColors);
		  glEnableClientState(GL_COLOR_ARRAY);
		  glEnable(GL_COLOR_MATERIAL);
		  glEnable(GL_LIGHTING);
		  glEnable(GL_BLEND);
          glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	  }
	  if ((fogSpec != (FogSpec*) 0) && showFog()) {
		  glEnable(GL_FOG);
		  fogSpec -> install();
	  } 

      getMaterial() -> renderColoredSurface(this); 
      
      if (vertexColors != 0) {
		  glDisableClientState(GL_COLOR_ARRAY);
		  glDisable(GL_COLOR_MATERIAL);
		  glDisable(GL_BLEND);
	  }
	  
	  if ((fogSpec != (FogSpec*) 0) && showFog()) {
		  glDisable(GL_FOG);
	  } 
      glDisableClientState(GL_NORMAL_ARRAY);
      glDisableClientState(GL_VERTEX_ARRAY);
      
} 

/*
 * Bei der Behandlung von Texturen werden die meisten möglichen
 * Konfigurationen im Augenblick nicht unterstützt.
 */
void SpacialSurfaceObject::renderTexturizedObject() {
    /*  tex  */
    GLfloat colorMask[4];
    GLuint textureHandles [1];
    colorMask[0] = 0.0f;    // red
    colorMask[1] = 0.0f;    // green
    colorMask[2] = 0.0f;    // blue
    colorMask[3] = 1.0f;    // alpha
    glGenTextures(1, &textureHandles[0]);
    
    glBindTexture(GL_TEXTURE_2D, textureHandles[0]);
    glActiveTexture(GL_TEXTURE0);  //  später angefügt, aendert nichts
    glTexEnvi(GL_TEXTURE_ENV,
              GL_TEXTURE_ENV_MODE,
              GL_MODULATE); //default: GL_MODULATE, others: GL_ADD, GL_DECAL, GL_BLEND, GL_REPLACE, GL_COMBINE
    glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, colorMask);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, textureWidth, textureHeight, 0,
                GL_RGB, GL_UNSIGNED_BYTE, textureBuffer);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (GLfloat)GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (GLfloat)GL_NEAREST);  
    glEnable(GL_TEXTURE_2D);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    //glBlendFunc(GL_SRC_COLOR, GL_DST_COLOR);
    glEnable(GL_BLEND);
    /* -- */

      glEnableClientState(GL_VERTEX_ARRAY);
      glVertexPointer(3, GL_FLOAT, 0, vertices);
      glEnableClientState(GL_NORMAL_ARRAY);
      glNormalPointer(GL_FLOAT, 0, vertexNormals);
      if (vertexColors != 0) {
		  glColorPointer(4, GL_UNSIGNED_BYTE, 0, vertexColors);
		  glEnableClientState(GL_COLOR_ARRAY);
		  glEnable(GL_COLOR_MATERIAL);
	  }
      /* tex */
      glEnableClientState(GL_TEXTURE_COORD_ARRAY);
      glTexCoordPointer(2, GL_FLOAT, 0, textureCoordinates);
      /* --- */
      
	  if ((fogSpec != (FogSpec*) 0) && showFog()) {
		  glEnable(GL_FOG);
		  fogSpec -> install();
	  } 
      getMaterial() -> renderColoredSurface(this); 
      
      /* tex */
      glDisable(GL_TEXTURE_2D);
      glDisable(GL_BLEND);
      glDisableClientState(GL_TEXTURE_COORD_ARRAY);
      /* --- */
      if (vertexColors != 0) {
		  glDisableClientState(GL_COLOR_ARRAY);
		  glDisable(GL_COLOR_MATERIAL);
	  }
	  
	  if ((fogSpec != (FogSpec*) 0) && showFog()) {
		  glDisable(GL_FOG);
	  } 
      glDisableClientState(GL_NORMAL_ARRAY);
      glDisableClientState(GL_VERTEX_ARRAY);
      /* tex */
      glDeleteTextures(1, textureHandles);
      /* --- */
}
