#include "StagePropertiesPanel.h"
#include "StagePropertiesPanelStrings.h"
#include "ColorSpec.h"

StagePropertiesPanel::StagePropertiesPanel(SceneHolder& sm)
  : sceneModel(sm),
    red(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    green(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    blue(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    bg_red(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    bg_green(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    bg_blue(Gtk::Adjustment::create(0.0, 0.0, 256.0, 1.0, 1.0, 1.0)),
    labelLight(StagePropertiesPanel::LiteralAmbientLight),
    labelBackground(StagePropertiesPanel::LiteralBackgroundColor),
    reglerR(red, Gtk::ORIENTATION_HORIZONTAL),
    reglerG(green, Gtk::ORIENTATION_HORIZONTAL),
    reglerB(blue, Gtk::ORIENTATION_HORIZONTAL),
    bg_reglerR(bg_red, Gtk::ORIENTATION_HORIZONTAL),
    bg_reglerG(bg_green, Gtk::ORIENTATION_HORIZONTAL),
    bg_reglerB(bg_blue, Gtk::ORIENTATION_HORIZONTAL),
    container1(Gtk::ORIENTATION_VERTICAL, 12),
    container2(Gtk::ORIENTATION_VERTICAL),
    container3(Gtk::ORIENTATION_VERTICAL),
    suspendUpdates(false)
{
    sceneModel.addObserver(this);
    
    container1.set_margin_left(10);
    container1.set_margin_right(10);
    container1.set_margin_top(10);
    
    container2.set_margin_left(10);
    container2.set_margin_right(10);
    container2.set_margin_top(10);
    container2.set_margin_bottom(10);
    
    container3.set_margin_left(10);
    container3.set_margin_right(10);
    container3.set_margin_top(10);
    container3.set_margin_bottom(10);
    
    reglerR.set_value_pos(Gtk::POS_LEFT);
    reglerG.set_value_pos(Gtk::POS_LEFT);
    reglerB.set_value_pos(Gtk::POS_LEFT);
    reglerR.set_digits(0);
    reglerG.set_digits(0);
    reglerB.set_digits(0);
    bg_reglerR.set_value_pos(Gtk::POS_LEFT);
    bg_reglerG.set_value_pos(Gtk::POS_LEFT);
    bg_reglerB.set_value_pos(Gtk::POS_LEFT);
    bg_reglerR.set_digits(0);
    bg_reglerG.set_digits(0);
    bg_reglerB.set_digits(0);
    
    if (sceneModel.hasScene()) {
		ColorSpec c = sceneModel.getScene()->getAmbientLightColor();
		reglerR.set_value(c.getRedEditValue());
		reglerG.set_value(c.getGreenEditValue());
		reglerB.set_value(c.getBlueEditValue());
		c = sceneModel.getScene()->getClearColor();
		bg_reglerR.set_value(c.getRedEditValue());
		bg_reglerG.set_value(c.getGreenEditValue());
		bg_reglerB.set_value(c.getBlueEditValue());
	}

    red->signal_value_changed().connect(sigc::mem_fun(*this, &StagePropertiesPanel::onRValueChanged));
    green->signal_value_changed().connect(sigc::mem_fun(*this, &StagePropertiesPanel::onGValueChanged));
    blue->signal_value_changed().connect(sigc::mem_fun(*this, &StagePropertiesPanel::onBValueChanged));
   

    bg_red->signal_value_changed().connect(sigc::mem_fun(*this, &StagePropertiesPanel::onBG_RValueChanged));
    bg_green->signal_value_changed().connect(sigc::mem_fun(*this, &StagePropertiesPanel::onBG_GValueChanged));
    bg_blue->signal_value_changed().connect(sigc::mem_fun(*this, &StagePropertiesPanel::onBG_BValueChanged));
    
    add(container1);
    container2.pack_start(reglerR, Gtk::PACK_SHRINK);
    container2.pack_start(reglerG, Gtk::PACK_SHRINK);
    container2.pack_start(reglerB, Gtk::PACK_SHRINK);
    labelLight.add(container2);
    labelLight.set_border_width(2);
    labelLight.set_shadow_type(Gtk::SHADOW_ETCHED_OUT);
    
    container3.pack_start(bg_reglerR, Gtk::PACK_SHRINK);
    container3.pack_start(bg_reglerG, Gtk::PACK_SHRINK);
    container3.pack_start(bg_reglerB, Gtk::PACK_SHRINK);
    labelBackground.add(container3);
    labelBackground.set_border_width(2);
    labelBackground.set_shadow_type(Gtk::SHADOW_ETCHED_OUT);
    
    container1.pack_start(labelLight, Gtk::PACK_SHRINK, 0);
    container1.pack_start(labelBackground, Gtk::PACK_SHRINK, 0);
    /*
    labelLight.show();
    labelBackground.show();
    reglerR.show();
    reglerG.show();
    reglerB.show(); */
    container1.show();
}

StagePropertiesPanel::~StagePropertiesPanel()
{
	sceneModel.removeObserver(this);
}

void StagePropertiesPanel::onSceneChanged() {
    suspendUpdates = true;
	if (sceneModel.hasScene()) {
		ColorSpec c = sceneModel.getScene()->getAmbientLightColor();
		reglerR.set_value(c.getRedEditValue());
		reglerG.set_value(c.getGreenEditValue());
		reglerB.set_value(c.getBlueEditValue());
		c = sceneModel.getScene()->getClearColor();
		bg_reglerR.set_value(c.getRedEditValue());
		bg_reglerG.set_value(c.getGreenEditValue());
		bg_reglerB.set_value(c.getBlueEditValue());
	} else {
		reglerR.set_value(0);
		reglerG.set_value(0);
		reglerB.set_value(0);
		bg_reglerR.set_value(0);
		bg_reglerG.set_value(0);
		bg_reglerB.set_value(0);
	}
	suspendUpdates = false;
}

void StagePropertiesPanel::onRValueChanged() {
	if (sceneModel.hasScene() &&(!suspendUpdates)) {
        sceneModel.getScene()->getAmbientLightColorRef().setRedValue(reglerR.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void StagePropertiesPanel::onGValueChanged() {
	if (sceneModel.hasScene() &&(!suspendUpdates)) {
        sceneModel.getScene()->getAmbientLightColorRef().setGreenValue(reglerG.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void StagePropertiesPanel::onBValueChanged() {
	if (sceneModel.hasScene() &&(!suspendUpdates)) {
        sceneModel.getScene()->getAmbientLightColorRef().setBlueValue(reglerB.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}




void StagePropertiesPanel::onBG_RValueChanged() {
	if (sceneModel.hasScene() &&(!suspendUpdates)) {
        sceneModel.getScene()->getClearColorRef().setRedValue(bg_reglerR.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void StagePropertiesPanel::onBG_GValueChanged() {
	if (sceneModel.hasScene() &&(!suspendUpdates)) {
        sceneModel.getScene()->getClearColorRef().setGreenValue(bg_reglerG.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}

void StagePropertiesPanel::onBG_BValueChanged() {
	if (sceneModel.hasScene() &&(!suspendUpdates)) {
        sceneModel.getScene()->getClearColorRef().setBlueValue(bg_reglerB.get_value()/255.0f);
        sceneModel.notifySceneAttributeChange();
	}
}
