#include "TrapezoidRule.h"

#include <math.h>

TrapezoidRule::TrapezoidRule(float start, float end, float e, int n) :
    Integrator (start, end, e, n)
{
}

float TrapezoidRule::computeValue(ArcLengthIntegrand* integrand) {
    float interval = (tEnd - tStart)/2.0f;
    float xi = tStart;
    float area = (integrand->fn(tStart) + integrand->fn(tEnd))*interval;
    
    int n = 2;
    bool stop = false;
    
    while (!stop) {
        float oldArea = area;
        float sum = 0.0F;
        for (int i = 0; i < n; i = i + 2) {
            float localStart = i*interval * 3 + tStart;
            xi = interval*i + localStart;
            sum += integrand->fn(xi);
        }
        n *= 2;
        area = area / 2.0F + sum*interval;
        interval /= 2.0f;
        stop = n > maxN || (fabs(area - oldArea) < fabs(area)*eps) ;
    }
    return area;
}
